/* Based on sednterp documentation...
**
**
** "Private" functions, denoted by a prefixed underscore ('_') are 
** intended for internal calculations only. All other calculations are
** expected to return an Object or combination of Objects which to be of any use
** must eventually boil down to a "primitive" value (string, number, etc).
** 
** The application expects values to be returned in an object with attributes
** of NAME and VALUE, where NAME specifies the string name of the widget to
** populate the specificed primitive VALUE with. All other attributes of the
** returned object are transferred to the application, but have no specific use
** at this time.
**
** This may not be the simplest approach, but it allows for a nearly lossless
** translation between JavaScript objects and the application
*/

// Constants
/*! 
 *! There are places in the Gui/Web app code which require access to
 *! these constants. For consistency, we probably want to "set" these constants
 *! from a call to a script function eventually. 
 */
// R Gas Constant ("in cgs")
//var R  = 8.3144 * Math.pow(10, 7);
var R = 83145100;
// Avogadros number (1/mole)
var Na = 6.0221367 * Math.pow(10, 23);
// Viscosity of water at 20*c
//var N_20_W = 1.002;
// Density water at 20*C (Rho)
var Density_20_Water = 0.998234;
var Rho_Water = Density_20_Water;
// Viscosity of water at 20C (Vis)
var Viscosity_20_Water = 0.01002;
// PI as defined in original sednterp
var PI = 3.1415926536;

var DEFAULT_VelocityTemp = 20.0;
var DEFAULT_Vbar = 0.7300;

var aminonumber   = 0;
var connumber     = 1;
var aminopka      = 2;
var conpka        = 3;
var aminocharge   = 4;
var concharge     = 5;
var AminoBase     = 6;
var ConjugateBase = 7;
// Public Methods
/******
function getModuleName()
function getModuleVersion()
function getModuleInteractionXml()
function getModuleDate()
function getModuleUI()
function calculationsFromSigma(T, sigma, RPM, Vbar, density, tellerMethod)
function calculationsFromMr(T, RPM, Mr, Vbar, density, tellerMethod)
function calculateMr (T, sigma, RPM, Vbar, density)
function calculateMbFromSigma(T, sigma, RPM)
function calculateMbDirectly(Mr, Vbar, Density)
function calculateVBarAtT (Vbar, T)
function calculateVBar20(Vbar, T)
function calculateRp(Mr)
function calculateRo(Mr, Vbar)
function calculateSigmaFromMr (T, RPM, Mr, Vbar, density)
function calculateDensityAt20(T, Density)
function calculateDensityAtT(T, Density)
function calculateVisc20(Visc, T)
function calculateViscAtT(Visc, T)
function calculateS20W(Tone, Ttwo, S, Vbar20, Visc, Vbar, Density, aVbar)
function calculateSo20W(Tone, Ttwo, S, Vbar20, Visc, Vbar, Density, aVbar, K, Cp)
function calculationsFromComposition(pH, oligomerN, aminos, conjugates)
function calculateBufferDensity(params)
function calculateBufferViscosity(params)
function correctBufferViscosityAtT(viscosity, temperature)
function correctBufferDensityAtT_Normal(density, temperature)
function correctBufferDensityAtT_Heavy(density, temperature, D2O, D2O18, H2O18)
function sqlFetchSampleComponents()
function sqlGetSampleByID(id)
function sqlFetchBufferComponents()
function sqlFetchComboBoxes ()
function calculateResults(Mr, Vbar,  S20W, Mb, T,  Sigma, RPM,  Delta1, Delta1c, HydrationValue, Hydration, Shape)
******/

/* Required Accessor Functions */
function getModuleName()
{
    return "MainScreen";
}
function getModuleVersion()
{
    return "20120828";
}
function getModuleInteractionXml()
{
    return "No Interactions";
}
function getModuleDate()
{
    return "2010/06/22";
}
function getModuleUI()
{
    return "qt_sednterp.ui";
}
function getModuleHTMLUI()
{
    return "./modules/QtScript/module1.html";
}

//-------------------------------------------------------------
// "From Sigma:"
// calculateMr
// calculateRp
// calculateMbFromSigma
function calculationsFromSigma(T, sigma, RPM, Vbar, density, tellerMethod)
{
    print(">>Function: calculationsFromSigma(T, sigma, RPM, Vbar, density, tellerMethod)");
    print("\tT=" + T.VALUE);
    print("\tsigma=" + sigma.VALUE);
    print("\tRPM=" + RPM.VALUE);
    print("\tVbar=" + Vbar.VALUE);
    print("\tdensity=" + density.VALUE);
    print("\ttellerMethod=" + tellerMethod.VALUE);
    /*
    ** Don't need any additional information from the incoming object params
    */
    T            = parseFloat(T.VALUE);
    sigma        = parseFloat(sigma.VALUE);
    RPM          = parseFloat(RPM.VALUE);
    Vbar         = parseFloat(Vbar.VALUE);
    density      = parseFloat(density.VALUE);
    tellerMethod = parseInt(tellerMethod.VALUE);
    
    var Mr = _calculateMr(T, sigma, RPM, Vbar, density);
    var Mb = _calculateMbFromSigma(T, sigma, RPM);
    var Rp;
    
    if (tellerMethod == 1)
    {
	Rp = _calculateRp(Mr);
    }
    else
    {
	Rp = _calculateRo(Mr, Vbar);
    }
    
    var obj = new Array();
    obj[0] = new Object();
    obj[0].NAME = "qtSedEquilibriumMR";
    obj[0].VALUE = Mr;
    
    obj[1] = new Object();
    obj[1].NAME = "qtExperimentResultsMb";
    obj[1].VALUE = Mb;
    
    obj[2] = new Object();
    obj[2].NAME = "qtExperimentResultsRp";
    obj[2].VALUE = Rp.toExponential(4);
    
    obj[3] = new Object();
    obj[3].NAME = "qtExperimentResultsRpLabel";
    obj[3].ISLABEL = "true";
    
    if (tellerMethod == 1)
    {
	obj[3].VALUE = "Rp";
    }
    else
    {
	obj[3].VALUE = "Ro";
    }
    return obj;
    
    
    //   obj.qtSedEquilibriumMR    = Mr;
    //  obj.qtExperimentResultsMb = Mb;
    // obj.qtExperimentResultsRp = Rp;
    // return obj
}



//-------------------------------------------------------------
// "From Mr:"
// calculateMbDirectly
// calculateRp
// calculateSigmaFromMr
function calculationsFromMr(T, RPM, Mr, Vbar, density, tellerMethod)
{
    print(">>Function: calculationsFromMr(T, RPM, Mr, Vbar, density, tellerMethod)");
    print("\tT=" + T.VALUE);
    print("\tRPM=" + RPM.VALUE);
    print("\tMr=" + Mr.VALUE);
    print("\tVbar=" + Vbar.VALUE);
    print("\tdensity=" + density.VALUE);
    print("\ttellerMethod=" + tellerMethod.VALUE);
    /*
    ** Don't need any additional information from the incoming object params
    */
    T            = parseFloat(T.VALUE);
    RPM          = parseFloat(RPM.VALUE);
    Mr           = parseFloat(Mr.VALUE);
    Vbar         = parseFloat(Vbar.VALUE);
    density      = parseFloat(density.VALUE);
    tellerMethod = parseInt(tellerMethod.VALUE);


    var Mb    = _calculateMbDirectly(Mr, Vbar, density);
    var Rp;

    if (tellerMethod == 1)
    {
	Rp = _calculateRp(Mr);
    }
    else
    {
	Rp = _calculateRo(Mr, Vbar);
    }
    var Sigma = _calculateSigmaFromMr(T, RPM, Mr, Vbar, density);

    var obj = new Array();
    obj[0] = new Object();
    obj[0].NAME = "qtExperimentResultsMb";
    obj[0].VALUE = Mb;

    obj[1] = new Object();
    obj[1].NAME = "qtExperimentResultsRp";
    obj[1].VALUE = Rp.toExponential(4);

    obj[2] = new Object();
    obj[2].NAME = "qtSedEquilibriumSigma";
    obj[2].VALUE = Sigma.toExponential(4);

    obj[3] = new Object();
    obj[3].NAME = "qtExperimentResultsRpLabel";
    obj[3].ISLABEL = "true";
    if (tellerMethod == 1)
    {
	obj[3].VALUE = "Rp";
    }
    else
    {
	obj[3].VALUE = "Ro";
    }

    return obj;
    //var obj = new Object; 
    //obj.qtExperimentResultsMb = Mb;
    //obj.qtExperimentResultsRp = Rp;
    //obj.qtSedEquilibriumSigma = Sigma;
    //return obj;
}



// Public interface for _calcluateMr
function calculateMr (T, sigma, RPM, Vbar, density)
{
    print(">>Function: calculateMr(T, sigma, RPM, Vbar, density)");
    //print("\tT=" + T.VALUE);
    //print("\tsigma=" + sigma.VALUE);
    //print("\tRPM=" + RPM.VALUE);
    //print("\tVbar=" + Vbar.VALUE);
    //print("\tdensity=" + density.VALUE);

    var obj = new Object();
    obj.NAME ="qtSedEquilibriumMR";
    obj.VALUE = _calculateMr(T.VALUE, sigma.VALUE, RPM.VALUE, Vbar.VALUE, 
			     density.VALUE);
    return obj;
}


//-------------------------------------------------------------
// Preceded by an underscore to mark as "private" by "convention"
function _calculateMr (T, sigma, RPM, Vbar, density)
{
    print(">>Function: _calculateMr(T, sigma, RPM, Vbar, density)");
    //print("\tT=" + T);
    //print("\tsigma=" + sigma);
    //print("\tRPM=" + RPM);
    //print("\tVbar=" + Vbar);
    //print("\tdensity=" + density);
    /*
    ** Don't need any additional information from the incoming object params
    */
    T       = parseFloat(T);
    sigma   = parseFloat(sigma);
    RPM     = parseFloat(RPM);
    Vbar    = parseFloat(Vbar);
    density = parseFloat(density);

    /*
    print("T " + T);
    print("sigma " + sigma);
    print("RPM " + RPM);
    print("Vbar " + Vbar);
    print("density " + density);
    */

    /*
    ** Calculating Molecular Weight (Mr)
    **
    ** T:       Sed Equilibrium Temperature
    ** sigma:   Sed Equilibrium Sigma
    ** RPM:     Sed Equilibrium RPM
    ** Vbar:    Sample VBar
    ** Density: Buffer Density
    **
    **1) Exact Molecular Weight may be input by user (if known)
    **2) Sednterp may calculate Mr from "equation 4"
    **3) Sedneterp may calculate Mr based on composition 
    **   (uses Composition Button)
    ** 
    ** This is implementation for the second option
    ** Mr = (sigma * R * T) / ((1 - (Vbar * density)) * w^2)
    ** See documentation "list of symbols used" for symbol info
    */
    
    // Convert RPM to angular Velocity
    var w = (RPM * PI) / 30.0;
    
    // Convert T from Celsius (as input on form) to K)
    T += 273.15;

    return (sigma * R * T) / ((1.0 - (Vbar * density)) * (Math.pow(w, 2)));
}


// Public interface for _calculateMbFromSigma
function calculateMbFromSigma(T, sigma, RPM)
{
    print(">>Function: calculateMbFromSigma(T, sigma, RPM) ");
    //print("\tT=" + T.VALUE);
    //print("\tsigma=" + sigma.VALUE);
    //print("\tRPM=" + RPM.VALUE);
/*
    print ("T: " + T.VALUE);
    print ("sigma: " + sigma.VALUE);
    print ("RPM: " + RPM.VALUE);
*/

    var obj = new Object();
    obj.NAME = "qtExperimentResultsMb";
    obj.VALUE = _calculateMbFromSigma(T.VALUE, sigma.VALUE, RPM.VALUE);

    return obj;
}


//-------------------------------------------------------------
// Calculate MB from sigma (Preceded by _ to denote "private")
function _calculateMbFromSigma(T, sigma, RPM)
{
    print(">>Function: _calculateMbFromSigma(T, sigma, RPM)");
    print("\tT=" + T);
    print("\tsigma=" + sigma);
    print("\tRPM=" + RPM);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    T       = parseFloat(T);//.toPrecision(SINGLE); 
    sigma   = parseFloat(sigma);//.toPrecision(SINGLE); 
    RPM     = parseFloat(RPM);//.toPrecision(SINGLE);
    
    print (T);
    print (sigma);
    print (RPM);
    
 
    /*
    ** Calculating Buoyant Molecular Weight (Mb)
    ** Mb may be calculated one of two ways:
    **  
    ** 2) If Mr was entered directly, or it was calculated from Composition,
    **    calculate "directly" using "equation 6" (Mb = M(1 - (Vbar * density)))
    */
    
    // Convert RPM to angular Velocity
    var w = (RPM * PI) / 30.0;
    
    // Convert T from Celsius (as input on form) to K)
    T += 273.15;
    print("And the answer is = " + (sigma * R * T)/Math.pow(w,2));
    return (sigma * R * T) / Math.pow(w, 2);
}


//-------------------------------------------------------------
// Calculate MB directly 
function calculateMbDirectly(Mr, Vbar, Density)
{
    print(">>Function: calculateMbDirectly(Mr, Vbar, Density)");
    //print("\tMr=" + Mr.VALUE);
    //print("\tVbar=" + Vbar.VALUE);
    //print("\tDensity=" + Density.VALUE);

    var obj = new Object();
    obj.NAME = "qtExperimentResultsMb";
    obj.VALUE = _calculateMbDirectly(Mr.VALUE, Vbar.VALUE, Density.VALUE);
    return obj;
}


//-------------------------------------------------------------
// Calculate MB directly 
function _calculateMbDirectly(Mr, Vbar, Density)
{
    print(">>Function: _calculateMbDirectly(Mr, Vbar, Density)");
    print("\tMr=" + Mr);
    print("\tVbar=" + Vbar);
    print("\tDensity=" + Density);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    Mr      = parseFloat(Mr);
    Vbar    = parseFloat(Vbar);
    Density = parseFloat(Density);

    /*
    ** Calculating Buoyant Molecular Weight (Mb)
    ** Mb may be calculated one of two ways:
    ** 
    ** 2) If Mr was entered directly, or it was calculated from Composition,
    **    calculate "directly" using "equation 6" (Mb = M(1 - (Vbar * density)))
    */
    print("\n The answer is = " + Mr * (1-(Vbar*Density)));
    return Mr * (1 - (Vbar * Density));
}


//-------------------------------------------------------------
// "Calculates Vbar at 20C given inputs of vbar at some other temperature"
// Call only when T changes.
// Vbar = Vbar + 0.000425 * (T - 20)
function calculateVBarAtT (Vbar, T)
{
    print(">>Function: calculateVBarAtT (Vbar, T)");
    //print("\tVbar=" + Vbar.VALUE);
    //print("\tT=" + T.VALUE);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    Vbar = parseFloat(Vbar.VALUE);
    T    = parseFloat(T.VALUE);

    //ErrorChecking?
    
    //    return "qtSampleVbar," + str(_calculateVBarAtT(Vbar, T))
    var obj = new Object();
    obj.NAME  = "qtSampleVbar";
    obj.VALUE = _calculateVBarAtT(Vbar, T);
    return obj;
}


//-------------------------------------------------------------
// Calculates VbarAtT from Vbar and eqTemp
function _calculateVBarAtT (Vbar, T)
{
    print(">>Function: _calculateVBarAtT (Vbar, T)");
    //print("\tVbar=" + Vbar);
    //print("\tT=" + T);
    /*
    ** Don't need any additional attributes from incoming params
    */
    Vbar = parseFloat(Vbar);
    T    = parseFloat(T);

    return Vbar + 0.000425 * (T - 20);
}


//-------------------------------------------------------------
// Calculate Vbar Estimates ("Standard" Vbar button)
// "Calculates Vbar at 20C given inputs of vbar at some other temperature"
// Vbar = Vbar - 0.000425 * (T - 20)
function calculateVBar20(Vbar, T)
{
    print(">>Function: calculateVBar20 (Vbar, T)");
    //print("\tVbar=" + Vbar.VALUE);
    //print("\tT=" + T.VALUE);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    Vbar = parseFloat(Vbar.VALUE);
    T    = parseFloat(T.VALUE);

    //ErrorChecking?
    
    //    return "qtSampleVbar," + str(_calculateVBarAtT(Vbar, T))
    var obj = new Object();
    obj.NAME  = "qtSampleVbar";
    obj.VALUE = _calculateVBar20(Vbar, T);
    return obj;
}


//-------------------------------------------------------------
// Calculates VbarAtT from Vbar and eqTemp
function _calculateVBar20(Vbar, T)
{
    print(">>Function: _calculateVBar20 (Vbar, T)");
    //print("\tVbar=" + Vbar);
    //print("\tT=" + T);
    /*
    ** Don't need any additional attributes from incoming params
    */
    Vbar = parseFloat(Vbar);
    T    = parseFloat(T);

    return Vbar - 0.000425 * (T - 20);
}


// Public interaface for _calculateRp from Mr
function calculateRp(Mr)
{
    print(">>Function: calculateRp (Mr)");
    //print("\tMr=" + Mr.VALUE);
    var obj = new Array();

    obj[0] = new Object();
    obj[0].NAME  = "qtExperimentResultsRp";
    obj[0].VALUE = _calculateRp(Mr.VALUE).toExponential(4);
    obj[1] = new Object();
    obj[1].NAME = "qtExperimentResultsRpLabel";
    obj[1].VALUE = "Rp";
    obj[1].ISLABEL = "true";
    return obj;
}

// Public interaface for _calculateRo from Mr and Vbar
function calculateRo(Mr, Vbar)
{
    print(">>Function: calculateRo (Mr, Vbar)");
    //print("\tMr=" + Mr.VALUE);
    //print("\tVbar=" + Vbar.VALUE);
    var obj = new Array();
    obj[0] = new Object();
    obj[0].NAME  = "qtExperimentResultsRp";
    obj[0].VALUE = _calculateRo(Mr.VALUE, Vbar.VALUE);
    obj[1] = new Object();
    obj[1].NAME = "qtExperimentResultsRpLabel";
    obj[1].VALUE = "Ro";
    obj[1].ISLABEL = "true";
    return obj;
}


//-------------------------------------------------------------
// Calculates Rof from Mr and Vbar
function _calculateRo(Mr, Vbar)
{
    print(">>Function: _calculateRo (Mr, Vbar)");
    //print("\tMr=" + Mr);
    //print("\tVbar=" + Vbar);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    Mr   = parseFloat(Mr);
    Vbar = parseFloat(Vbar);
    
    //print("Mr: " + Mr);
    //print("Vbar: " + Vbar);

    /*
    ** Calculate Rp
    ** calculate Rp using temp and vbar 
    ** Rp = cubert((3Mr*Vbar) / 4pi * No)
    */
    // calcs by vbar method?
    return Math.pow(((3.0 * Mr * Vbar) /
    		     (4.0 * PI * Na)), (1.0 / 3.0));
}

//-------------------------------------------------------------
// Calculates Rp from Mr 
function _calculateRp(Mr)
{
    print(">>Function: _calculateRp (Mr)");
    //print("\tMr=" + Mr);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    Mr   = parseFloat(Mr);
    
    //print("Mr: " + Mr);

    /*
    ** Calculate Rp
    ** calculate Rp using temp and vbar 
    ** Rp = cubert((3Mr*Vbar) / 4pi * No)
    */
    
    // calcs by teller method?
    return 0.00000000672 * Math.pow(Mr, (1.0 / 3.0));
}


// Public interface for _calculateSigmaFromMr
function calculateSigmaFromMr (T, RPM, Mr, Vbar, density)
{
    print(">>Function: calculateSigmaFromMr (T, RPM, Mr, Vbar, density)"); 
    //print("\tT=" + T.VALUE);
    //print("\tRPM=" + RPM.VALUE);
    //print("\tMr=" + Mr.VALUE);
    //print("\tVbar=" + Vbar.VALUE);
    //print("\tdensity=" + density.VALUE);

    var obj = new Object();
    obj.NAME  = "qtSedEquilibriumSigma";
    obj.VALUE = _calculateSigmaFromMr(T.VALUE, RPM.VALUE, Mr.VALUE, Vbar.VALUE,
				      density.VALUE).toExponential(4);
    return obj;
}


// Calculates Sigma from Mr if Mr was entered directly from hand
// (Preceded by _ to indicate this is "private")
function _calculateSigmaFromMr (T, RPM, Mr, Vbar, density)
{
    print(">>Function: _calculateSigmaFromMr (T, RPM, Mr, Vbar, density)"); 
    //print("\tT=" + T);
    //print("\tRPM=" + RPM);
    //print("\tMr=" + Mr);
    //print("\tVbar=" + Vbar);
    //print("\tdensity=" + density);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    T       = parseFloat(T);
    RPM     = parseFloat(RPM);
    Mr      = parseFloat(Mr);
    Vbar    = parseFloat(Vbar);
    density = parseFloat(density);
 
    /*
    ** Calculate Sigma from Mr (if Mr is entered "exactly" by hand)
    */
    // Convert RPM to angular Velocity
    var w = (RPM * PI) / 30.0;
    return (((1.0 - (Vbar * density)) * Mr * 
	     Math.pow(w, 2)) / (R * (T + 273.15)));
}


// Yanked directly from VB code...
//
// this routine is called internally to provide scaling
// of density vs. T, but not to give absolute densities
// because they differ slightly from our standard .998234 at 20
//
// returns density of water versus T from Kell formula
// this is supposed to be accurate to about 1 in 10^6
// so do it at double precision so we don't lose that
//
// //! Where these values come from I have no idea...
function _RhoTKell(T)
{
    print(">>Function: _RhoTKell(T)");
    //print("\tT=" + T);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    T       = parseFloat(T);
    
    var denom = 1 + 0.01687985 * T;
    var term1 = 999.83952 + 16.945176 * T;
    var term2 = (0.0079870401 * T) * T + 0.000046170461 * T * T * T;
    var term3 = 0.00000010556302 * T * T * T *
	T - (((2.8054253 * Math.pow(10, -10)) * T) * T * T * T * T);
    
    return 0.001000028 * (term1 - term2 + term3) / denom;
}


//
// calculates density corrected to 20 C
// Want to use this when clicking "standard" button for calculating density
// (I think?)
function calculateDensityAt20(T, Density)
{ 
    print(">>Function: calculateDensityAt20(T, Density)");
    //print("\tT=" + T.VALUE);
    //print("\tDensity=" + Density.VALUE);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    T       = parseFloat(T.VALUE);
    Density = parseFloat(Density.VALUE);
    
    var obj = new Object;
    obj.NAME  = "qtBufferDensity"; 
    obj.VALUE = _calculateDensityAt20(T, Density);
    return obj;
}


//
// calculates density corrected to 20 C
// Want to use this when clicking "standard" button for calculating density
// (I think?)
function _calculateDensityAt20(T, Density)
{
    print(">>Function: _calculateDensityAt20(T, Density)");
    //print("\tT=" + T);
    //print("\tDensity=" + Density);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    T       = parseFloat(T);
    Density = parseFloat(Density);
    
    return Density * _RhoTKell(20) / _RhoTKell(T);
}


//
// calculates density at T
// Want to use this when changing qtSedEquilibriumTemp
// (I think?)
function calculateDensityAtT(T, Density)
{
    print(">>Function: calculateDensityAtT(T, Density)");
    //print("\tT=" + T.VALUE);
    //print("\tDensity=" + Density.VALUE);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    T       = parseFloat(T.VALUE);
    Density = parseFloat(Density.VALUE);
    
    var obj = new Object;
    obj.NAME  = "qtBufferDensity";
    obj.VALUE = _calculateDensityAtT(T, Density);
    return obj;
}


// calculates density at T
// Want to use this when changing qtSedEquilibriumTemp
// (I think?)
function _calculateDensityAtT(T, Density)
{
    print(">>Function: _calculateDensityAtT(T, Density)");
    //print("\tT=" + T);
    //print("\tDensity=" + Density);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    T       = parseFloat(T);
    Density = parseFloat(Density);
   
    return Density * _RhoTKell(T) / _RhoTKell(20);
}


// Calculates the viscosity of water at temperature T
// Yanked Directly From VB
//
// No idea where these values come from...
// "calculates viscosity (poise) of normal water (natural isotopic composition)
// at the input temperature (C); returns zero for error if T outside valid
// range of 0 to 100"
//
// //! I ripped out the bounds checking (Assuming the form is handling it)
function _calculateViscWaterAtT(T)
{
    print(">>Function: _calculateViscWaterAtT(T)");
    //print("\tT=" + T);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    T       = parseFloat(T);
   
    if (T == 20)
        return Viscosity_20_Water;

    else if (T < 20)
    {
        var vistw = (1301 / (998.333 + 8.1855 * (T - 20) +
                             0.00585 * Math.pow((T - 20), 2))) - 3.3023047;
	vistw = Math.pow(10, vistw);
        return vistw;
    }
    
    else if (T > 20)
{
    return Viscosity_20_Water * (Math.pow(10, ((1.3272 * (20 - T) - 0.001053 * Math.pow((T - 20), 2)) /(T + 105))));
}
}


// Calculates Viscosity corrected to 20 C
// Want to use this when clicking "standard" button for calculating viscosity
// (I think?)
function calculateVisc20(Visc, T)
{
    print(">>Function: calculateVisc20(Visc,T)");
    //print("\tT=" + T.VALUE);
    //print("\tVisc=" + Visc.VALUE);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    Visc = parseFloat(Visc.VALUE);
    T    = parseFloat(T.VALUE);
 
    var obj = new Object();
    obj.NAME = "qtBufferViscosity";            //! HUH? only takes one param!
    //obj.VALUE = Visc * Viscosity_20_Water / _calculateViscWaterAtT(Visc, T);
 /////I multiplied by .01002 instead of Visc this gives the right answer Visc causes multiple inputs to diverge.
    obj.VALUE = 0.01002 * Viscosity_20_Water /_calculateViscWaterAtT(T);
    return obj;
}
    

// Calculates viscosity at T 
//
// Want to call this when qtSedEquilibriumTemp is changed
function calculateViscAtT(Visc, T)
{
    print(">>Function: calculateViscAtT(Visc,T)");
    //print("\tT=" + T.VALUE);
    //print("\tVisc=" + Visc.VALUE);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    Visc = parseFloat(Visc.VALUE);
    T    = parseFloat(T.VALUE);
 
    var obj = new Object(); 
    obj.NAME = "qtBufferViscosity";
    obj.VALUE = _calculateViscAtT(Visc, T);
    //Visc * _calculateViscWaterAtT(T) / Viscosity_20_Water;
    return obj;
}


// Calculates viscosity at T 
//
// Want to call this when qtSedEquilibriumTemp is changed
function _calculateViscAtT(Visc, T, term)
{
    print(">>Function: _calculateViscAtT(Visc,T)");
    //print("\tT=" + T);
    //print("\tVisc=" + Visc);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    Visc = parseFloat(Visc);
    T    = parseFloat(T);
    //Visc * _calculateViscWaterAtT(T) / Viscosity_20_Water;
 /////I multiplied by .01002 instead of Visc this gives the right answer Visc causes multiple inputs to diverge.

if (term !=1)
{
    return 0.01002 * _calculateViscWaterAtT(T) / Viscosity_20_Water;
}
else 
{
return Visc * _calculateViscWaterAtT(T) / Viscosity_20_Water;
}

}



//
// For fun, returns a results CSV for which an error was encountered
//
//
//function returnErrorCSV(T, RPM):
//    T   = float(T)
//    RPM = float(RPM)
//    
//    retCSV  = "ERROR, Generic Error Message\n"
//    retCSV += "qtSedEquilibriumTemp," + str(T) + "\n"
//    retCSV += "qtSedEquilibriumRPM," + str(RPM) + "\n"
//
//    return retCSV
//


// For fun, returns a results CSV for which a "Warning" was generated
//
//
//function returnWarningCSV(T, RPM):
//    T   = float(T)
//    RPM = float(RPM)
//    
//    retCSV  = "\n"
//    retCSV += "qtSedEquilibriumTemp," + str(T) + "\n"
//    retCSV += "qtSedEquilibriumRPM," + str(RPM) + "\n"
// 
//    return retCSV
//


//
// Calculate S*20,w
// 
// S*20,w = S20,w / (1 - KCp)
//
// S20,1 = S* (1 - (Vbar * density) 20,w Nt,b) / (1 - (Vbar * density))T,b N20,w)
// See equation 19
//

//CalcVbar20 = fVbar - 0.000425 * (fT - 20)

//function calculateS_20_W(sStart, VbarBuf, densityBuf):
 //   raise RuntimeError("Not implimented")

//
//function main():
    //   print calculateS_20_W(1,2,3)

//if __name__ == "__main__": main()


//function processInput():
 //   '''Process Input and call calculation method'''
    //qtTemperature.setText("arg!");
  //  return 0
    
    
//N_20_W = 1.002 
//Density water at 20*C
//Density_20_W = 0.98234




//Error checking!??
    
  //  retCSV  = "\n"
   // retCSV += "qtExperimentResultsS,"
    
// S20W    = S * (((1 - Vbar20 * Density_20_Water) * Visc) /
//1 - Vbar * Density) * N_20_Water))
    
    //Convert S20W to So20W
    
    //retCSV += str(S20W / (1 - (k * cp)))
    
   // return retCSV


//
// For Internal Use only: Calculate S20W
// Expects params are floats
//

function _calculateS20W(Tone, Ttwo, S, Vbar20, Visc, Vbar, Density, aVbar, vbarwater, aSo20)
{

    //print(">>Function: _calculateS20W(Tone, Ttwo, S, Vbar20, Visc, Vbar, Density)");
   print("\t_calculateS20W(Tone, Ttwo, S, Vbar20, Vbar, aVbar, vbarwater)")
   print("\tTone=" + Tone);
   print("\tTtwo=" + Ttwo);
   print("\tS=" + S);
   print("\tVbar20=" + Vbar20);
   print("\tVbar=" + Vbar);
   print("\taVbar=" + aVbar);
   print("\tvbarwater=" + vbarwater);
  
  if (aVbar == 0)
  {
     Density =  _calculateDensityAtT(Ttwo, Density_20_Water)
     Visc    =  _calculateViscWaterAtT(Ttwo);
     print("Visc = " +  _calculateViscWaterAtT(Ttwo));
     print("Density = " + Density);
  }
   var vbarrho = 0;
   var vbarrhow = 0;
   var tempans = 0;

   if ((Ttwo >= 0) && (Ttwo <= 45))
   {
       if (aSo20 == 1)
       {
           vbarrho = 1 - (Vbar+(.000425*(Ttwo - Tone))) * Density;
       }
       else
       {
           vbarrho = 1 - (Vbar20+(.000425*(Ttwo - Tone))) * Density;
       }
       if (vbarwater != 0)
       {

	 vbarrhow = 1 - vbarwater * Density_20_Water;
       } 
       else
       {
	 vbarrhow = 1 - Vbar20 * Density_20_Water;
       }


       tempans = S * ( vbarrhow * Visc ) / ( vbarrho * Viscosity_20_Water);
//       tempans = S * ( vbarrhow * Visc ) / ( vbarrho * Viscosity_20_Water);
       print("S20W = " + tempans);
   }
   else
   {
	var vbaropt = 0;
	var rhoopt = 0;

       print("\t 2: Case Ttwo is valid aka 0 > Ttwo || 45 > Ttwo");
	vbaropt = Vbar + .000425 * (Ttwo - Tone);

	if (vbarwater != 0)
	{
	  print("\t A : Case vbarwater != 0");
	  vbarrhow = 1 - vbarwater * Density_20_Water;
	}
	else
	{
	  print("\t B : Case vbarwater == 0");
	  vbarrhow = 1 - Vbar20 * Density_20_Water;
	}

	rhoopt = _calculateDensityAtT(Ttwo,Density_20_Water); 
	vbarrho = 1 - (vbaropt * rhoopt);

	tempans = S * (vbarrhow * _calculateViscWaterAtT(Ttwo)) / (vbarrho * Viscosity_20_Water);
	print("S20W = " + tempans);
   }



    var ret = tempans;
    return ret;  

}
/*function _calculateS20W(Tone, Ttwo, S, Vbar20, Vbar, aVbar,vbarwater)
{
    print(">>Function: _calculateS20W(Tone, Ttwo, S, Vbar20, Visc, Vbar, Density)");
    print("\tTone=" + Tone);
    print("\tTtwo=" + Ttwo);
    print("\tS=" + S);
    print("\tVbar20=" + Vbar20);
    print("\tVisc=" + Visc);
    print("\tVbar=" + Vbar);
    print("\tDensity=" + Density);
    print("\taVbar=" + aVbar);
    print("\tvbarwater=" + vbarwater);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
/*    Tone    = parseFloat(Tone);
    Ttwo    = parseFloat(Ttwo);
    S       = parseFloat(S);
    Visc    = parseFloat(Visc);
    Vbar20  = parseFloat(Vbar20);
    Vbar    = parseFloat(Vbar);
    Density = parseFloat(Density);
    //aVbar   = parseFloat(aVbar.VALUE);
    // Need to adjust Vbar to 20 C
    // Not entirely sure if this will work 100% with non "Direct Entry"
    // Because... changing Tone changes VBar in the sednterp form 
    if (Tone != Ttwo)
    {
      print("Ttwo != Tone")
      var diff = Tone - Ttwo;
     // Vbar = Vbar - (.000425 * diff);
    }
     
     if (aVbar.VALUE == 1)
     {
	Vbar20 = Vbar + (0.000425 * (20 - Tone));
     }
     else 
     { 
	Vbar20  = Vbar + (0.000425*(20 - Tone));
//	Vbar    = 0.73 + (0.000425 * (Tone - 20.0)); 
     }
    print("Vbar 20 and Vbar = " + Vbar20 + "    " + Vbar+ "  ");
    //Density = parseFloat(_calculateDensityAtT(Tone, Density_20_Water));
    
    //print ("_calculateS20W");
    //print ("T1 " + Tone);
    //print ("T2 " + Ttwo);
    //print ("S " + S);
    //print ("Visc " + Visc);
    //print ("Vbar20 " + Vbar20);
    //print ("Vbar " + Vbar);
    //print ("Density " + Density);

//_calculateDensityAtT(T, Density);
//_calculateDensityAt20(T, Density)Density_20_Water
  

  // TODO:
    // How do we adjust for Ttwo? (qtVelocityTemp)
    
    //print((((1.0 - (Vbar20 * Density_20_Water)) * Visc) /
    //               ((1.0 - (Vbar * Density)) * Viscosity_20_Water)));

    //print("\tTone=" + Tone);
    //print("\tTtwo=" + Ttwo);
    //print("\tS=" + S);
    //print("\tVbar20=" + Vbar20);
    //print("\tVisc=" + Visc);
    //print("\tVbar=" + Vbar);
    //print("\tDensity=" + Density);
    print("Testing 1, 2, 3, . . .");
    print("Density_20_Water = " + Density_20_Water+";");
    print("S = " + S+ ";");
    print("Vbar20 = " +Vbar20 +";");
    print("Visc = " +Visc+";");
    print("Vbar = " +Vbar+";");
    print("Density = " +Density+";");
    print("Viscosity_20_Water = " + Viscosity_20_Water+";");
    
    var ret = S * (((1.0 - (Vbar20 * Density_20_Water)) * Visc) /
                   ((1.0 - (Vbar * Density)) * Viscosity_20_Water));
 
    print (ret);
    return ret;
}
  */  











//
// Calculate S20W
// Tone:    qtSedEquilibriumTemp
// Ttwo:    qtVelocityTemp
// S:       s "star"
// Vbar20:  function of .73 (from data base?)
// Visc:    Viscosity from "buffer"
// Vbar:    Vbar from "buffer"
// Density: Density from "buffer"
// aVbar: logical (1 or 0) indicating adjust Vbar or not
//
function calculateS20W(Tone, Ttwo, S, Vbar20, Visc, Vbar, Density, aVbar, vbarwater)
{
    print(">>Function: calculateS20W(Tone, Ttwo, S, Vbar20, Vbar, vbarwater)");
/*    print("\tTone=" + Tone.VALUE);
    print("\tTtwo=" + Ttwo.VALUE);
    print("\tS=" + S.VALUE);
    print("\tVbar20=" + Vbar20.VALUE);
    print("\tVisc=" + Visc.VALUE);
    print("\tVbar=" + Vbar.VALUE);
    print("\tDensity=" + Density.VALUE);
    print("\taVbar=" + aVbar.VALUE);
    print("\tvbarwater=" + vbarwater.VALUE);
*/
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    Tone    = parseFloat(Tone.VALUE);
    Ttwo    = parseFloat(Ttwo.VALUE);
    S       = parseFloat(S.VALUE);
    Vbar20  = parseFloat(Vbar20.VALUE);
    Visc    = parseFloat(Visc.VALUE);
    Vbar    = parseFloat(Vbar.VALUE);
    Density = parseFloat(Density.VALUE);
    aVbar   = parseFloat(aVbar.VALUE);
    vbarwater = parseFloat(vbarwater.VALUE);
    aSo20   = 0;
    var obj = new Object;
    obj.NAME  = "qtExperimentResultsS";
    obj.VALUE = _calculateS20W(Tone, Ttwo, S, Vbar20, Visc, Vbar, Density, aVbar, vbarwater, aSo20);
    return obj;
}


//
// Calculate So20W 
//
// (See equation 19)
// 
// S       - S*
// Vbar20  - Not entirely sure where this comes from yet...
// Visc    - Viscosity from the buffer form
// Vbar    - Vbar from the buffer (?)
// Density - Density from the buffer
// K       - function .009, but can be changed by clicking the So20W label
// Cp      - Concentration mg/ml (?)
//
function calculateSo20W(Tone, Ttwo, S, Vbar20, Visc, Vbar, Density, aVbar,vbarwater, K, Cp)
{
    print(">>Function: calculateSo20W(Tone, Ttwo, S, Vbar20, Visc, Vbar, Density, aVbar, K, Cp)");
    print("\tTone=" + Tone.VALUE);
    print("\tTtwo=" + Ttwo.VALUE);
    print("\tS=" + S.VALUE);
    print("\tVbar20=" + Vbar20.VALUE);
    print("\tVbar=" + Vbar.VALUE);
    print("\taVbar=" + aVbar.VALUE);
    print("\tvbarwater=" + vbarwater.VALUE);
    print("\tK=" + K.VALUE);
    print("\tCp=" + Cp.VALUE);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    Tone    = parseFloat(Tone.VALUE);
    Ttwo    = parseFloat(Ttwo.VALUE);
    S       = parseFloat(S.VALUE);
    Vbar20  = parseFloat(Vbar20.VALUE);
    Visc    = parseFloat(Visc.VALUE);
    Vbar    = parseFloat(Vbar.VALUE);
    Density = parseFloat(Density.VALUE);
//    aVbar   = parseFloat(aVbar.VALUE);
    vbarwater = parseFloat(vbarwater.VALUE);
    K       = parseFloat(K.VALUE);
    Cp      = parseFloat(Cp.VALUE);
    
    /*
    print (Tone);
    print (Ttwo);
    print (S);
    print (Visc);
    print (Vbar);
    print (Density);
    print (K);
    print (Cp);
    */

    var obj = new Object;
    obj.NAME  = "qtExperimentResultsS"
    obj.VALUE = _calculateSo20W(Tone, Ttwo, S, Vbar20, Visc, Vbar, Density, aVbar,vbarwater, K, Cp);
    return obj;
}


//
// Calculate So20W 
//
// (See equation 19)
// 
// S       - S*
// Vbar20  - Not entirely sure where this comes from yet...
// Visc    - Viscosity from the buffer form
// Vbar    - Vbar from the buffer (?)
// Density - Density from the buffer
// K       - function .009, but can be changed by clicking the So20W label
// Cp      - Concentration mg/ml (?)
//
function _calculateSo20W(Tone, Ttwo, S, Vbar20, Visc, Vbar, Density, aVbar,vbarwater, K, Cp)
{    print(">>Function: _calculateSo20W(Tone, Ttwo, S, Vbar20, Visc, Vbar, Density, aVbar, K, Cp)");
    print("\tTone=" + Tone);
    print("\tTtwo=" + Ttwo);
    print("\tS=" + S);
    print("\tVbar20=" + Vbar20);
    print("\tVbar=" + Vbar);
    print("\taVbar=" + aVbar.VALUE);
    print("\tK=" + K);
    print("\tCp=" + Cp);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    Tone    = parseFloat(Tone);
    Ttwo    = parseFloat(Ttwo);
    S       = parseFloat(S);
    Vbar20  = parseFloat(Vbar20);
    Vbar    = parseFloat(Vbar);
    vbarwater = parseFloat(vbarwater);
    K       = parseFloat(K);
    Cp      = parseFloat(Cp);
    aVbar   = parseFloat(aVbar.VALUE);
    aSo20   = 1;
    return _calculateS20W(Tone, Ttwo, S, Vbar20, Visc, Vbar, Density, aVbar,vbarwater, aSo20) / (1 - (K * Cp));
}
    

// Calculates density corrected for temp and isotopes of water
// \retVal calculated density wrapped in a CSV
//function calculateDensityAtTHeavy(density20, T, D2O, D2O18, H2O18)
//{
 //   /*
 //   ** Params could come in as strings, so we need to type convert to
 //   ** ensure they are only used as floats 
 //   */
 //   density20 = parseFloat(density20.VALUE);
 //   T         = parseFloat(T.VALUE);
 //   D2O       = parseFloat(D2O.VALUE);
 //   D2O18     = parseFloat(D2O18.VALUE);
 //   H2O18     = parseFloat(H2O18.VALUE);
//
//    var obj = new Object;
//    obj.NAME = "qtCorrectedDensity"; 
//    obj.VALUE = _calculateDensityHeavyWater(density20, T, D2O, D2O18, H2O18);
//    return obj;
//}
//
//
//// "Private" function: calculates Density of heavy water at T
////param density20 Density at 20C
////param T buffer temperature
////param D2O percentage of D2O in water composition
////param D2O18 percentage of D2O18 in water composition
////param H2O18 percentage of H2O18 in water composition
////retVal float corrected density of buffer
//function _calculateDensityAtTHeavy(density20, T, D2O, D2O18, H2O18)
//{
//    /*
//    ** Params could come in as strings, so we need to type convert to
//    ** ensure they are only used as floats 
//    */
//    density20 = parseFloat(density20);
//    T         = parseFloat(T);
//    D2O       = parseFloat(D2O);
//    D2O18     = parseFloat(D2O18);
//    H2O18     = parseFloat(H2O18);
//
//    //Need to handle freezing point error checking eventually, although
//    //Not sure the original actually does anything?
//    
//    // If (fD2O > 0 And fT < 3.81) Or (fD2O18 > 0 And fT < 4.02) 
//    //     Or (fH2O18 > 0 And fT < 0.28) Then
//    //            'we are below freezing point for
//    //            'one of the isotopes that is present
//    //            'so return error
//    //            CalcRhoAtTHeavy = 0
//    
//    var delrho   = 0;
//    var rhowater = _calculateDensityAtT(T, Density_20_Water);
//    
//    if (D2O > 0)
//        delrho = delrho + D2O * (_CalcRhoD2OAtT(T) - rhowater);
//    
//    if (D2O18 > 0)
//	delrho = delrho + D2O18 * (_CalcRhoD2O18AtT(T) - rhowater);
//
//    if (H2O18 > 0)
//        delrho = delrho + H2O18 * (_CalcRhoH2O18AtT(T) - rhowater);
//    
//    return density20 * rhowater / Density_20_Water + delrho;
//}


//// Calculates density of D2O at T
// Sednterp documentation claims constants come from the database,
// but the VB code hard codes them. For now, we'll hardcode them too
function _CalcRhoD2OAtT(T)
{
    print(">>Function: _CalcRhoD20AtT(T)");
    //print("\tT=" + T);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    T         = parseFloat(T);

    // Need to handle temp bounds checking somehow...
    //     If fT >= 3.81 And fT <= 80 Then

    return 1.10602 * (1 - 1.0555 * Math.pow((T - 11.24), 2) * 
                      (1.74224 + 482.502 / (T - 11.24 + 77.861)) * 0.000001);
}


// Calculates density of D2O18 at T
// Sednterp documentation claims constants come from the database,
// but the VB code hard codes them. For now, we'll hardcode them too
function _CalcRhoD2O18AtT(T)
{
    print(">>Function: _CalcRhoD2018AtT(T)");
    //print("\tT=" + T);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    T         = parseFloat(T);
    // Need to handle temp bounds checking somehow
    //     If fT >= 4.02 And fT <= 80 Then
    return 1.21691 * (1 - 1.0555 * Math.pow((T - 11.46), 2) * 
                      (1.74224 + 482.502 / (T - 11.46 + 77.861)) * 0.000001);
}


// Calculates density of H2O18 at T
// Sednterp documentation claims constants come from the database,
// but the VB code hard codes them. For now, we'll hardcode them too
function _CalcRhoH2O18AtT(T)
{
    print(">>Function: _CalcRhoH2018AtT(T)");
    //print("\tT=" + T);
    /*
    ** Params could come in as strings, so we need to type convert to
    ** ensure they are only used as floats 
    */
    T         = parseFloat(T);

    // Need to handle temp bounds checking somehow
    //    If fT >= 0.28 And fT <= 80 Then

    return 1.11255 * (1 - Math.pow((T - 4.305), 2) * 
                      (1.74224 + 482.502 / (T - 4.305 + 77.861)) * 0.000001);
}



//Test returning multiple values by dictionary as an alternative to CSV
//function _testHash()
//{
//    print "testing hash"
//    d = {}
//    d["qtField1"] = ["ID=1", ["stringz!", "abcd"], "pie"];
//    
//   return d
//}


/* 
** Separated in VB code into calcVbar, calcMW, calcHydration (maybe others?) 
** But for performance, we probably don't want the overhead of 3+ separate calls
** which all use the same params
**
** \param pH the current pH 
** \param oligomerN the value of "N" if composition should be treated as
                    an oligomer (rather than a monomer)
** \param aminos an array of "AminoAcid" objects, each with attributes of 
**        NAME, ID, ROW and VALUE (where VALUE is # of moles, and ROW is the
**        row of the graphical table in which record sits)
** \param conjugates an array of "Conjugate" objects with attributes of 
**        NAME, ID, ROW and VALUE (where VALUE is # of moles, and ROW is the
**        row of the graphical table in which record sits)
**
** Some of the entries in the database are empty, apparently validly so.
** This causes a problem, when trying to convert a NULL entry to a float.
** Not sure how best to handle this.
**
** We could probably simplify this into one sql query...
**
*/

function calculationsFromComposition(pH, oligomerN, aminos, conjugates,
				     UseTitrableAmino,AminoEndpKaValue,
				     UseTitrableCarboxyl, CarboxylValue, denaturant)
{
    print(">>Function: calculationsFromComposition(pH, oligomerN, aminos, conjugates, UseTitrableAmino,AminoEndpKaValue, UseTitrableCarboxyl, CarboxylValue, denaturant) ");

    //print("\tpH=" + pH.VALUE);
    //print("\toligomerN=" + oligomerN.VALUE);
    //print("\tUseTitrableAmino=" + UseTitrableAmino.VALUE);
    //print("\tAminoEndpKaValue=" + AminoEndpKaValue.VALUE);
    //print("\tUseTitrableCarboxyl=" + UseTitrableCarboxyl.VALUE);
    //print("\tCarboxylValue=" + CarboxylValue.VALUE);
    //print("\tdenaturant= " + denaturant.VALUE);
    pH                   = parseFloat(pH.VALUE);
    UseTitrableAmino     = parseInt(UseTitrableAmino.VALUE);
    AminoEndpKaValue     = parseFloat(AminoEndpKaValue.VALUE);
    UseTitrableCarboxyl  = parseInt(UseTitrableCarboxyl.VALUE);
    CarboxylValue        = parseFloat(CarboxylValue.VALUE);

    // Used for absorbance
    var cext     = 0;
    var cext2    = 0;

    var TotalAminos = 0;

    //print("_calculationsFromComposition");
    //print("\tUseTitrableAmino=" + UseTitrableAmino);
    //print("\tAminoEndpKaValue=" + AminoEndpKaValue);
    //print("\tUseTitrableCarboxyl=" + UseTitrableCarboxyl);
    //print("\tCarboxylValue=" + CarboxylValue);
    //print("\toligomerN=" + oligomerN);
    //print("\tpH=" + pH);

    var retVal = new Object();
    retVal.AMINOS = aminos;
    retVal.CONJUGATES = conjugates;
    
    /*
    ** Iterate through each Array, storing the ID's in another array,
    ** so we can easily join(",") the array into a set of ID's for use in
    ** the WHERE sql clause
    */
    var aminoID = new Array();
    //print("aminos.length=" + aminos.length);
    for (var i = 0; i < aminos.length; i++)
    {
	//print ("Amino[" + i + "] " + aminos[i].ID);
	aminoID[i] = aminos[i].ID

	/*
	** Clear the stylesheet and tooltip of the cell at ROW, 2
	*/
	aminos[i].COL = 2;
	aminos[i].STYLESHEET = "background-color:rgb(#FFFFFF);";
	aminos[i].TOOLTIP = "";
    }

    var conjugateID = new Array();
    for (var i = 0; i < conjugates.length; i++)
    {
	//print ("conjugates[" + i + "] " + conjugates[i].ID);
	conjugateID[i] = conjugates[i].ID;

	/*
	** Clear the stylesheet and tooltip of the cell at ROW, 3
	*/
	conjugates[i].COL = 1;
	conjugates[i].STYLESHEET = "background-color:rgb(#FFFFFF);";
	conjugates[i].TOOLTIP = "";
    }
    
    /*
    ** Initial values
    */
    var calcVol  = 0;
    var calcMW   = 0;
    var calcMWC  = 0;
    var calcVBar = 0;
    var calcHydration = 0;
    var calcVolcov = 0;
    var calcVBarwater = 0;
    var calcMWCcov = 0;
    //print("new XX calcHydration=" + calcHydration);
    var csumd1 = 0;
    var cSumNt = 0;
    //print("new XX calcMW=" + calcMW);
    
    /*
    ** Query used for both tables
    */
    var query = "SELECT Mr, Vbar, :hipH, Namethree, id FROM :table WHERE id in (:IDset)";
    
    /*
    ** If pH is > 4, query HipH7 value. Otherwise, query HipH4
    */
    var hipH = "HipH4";
    if (pH > 4) 
    {
	hipH = "HipH7";
    }
    /*
    ** Query the amino table
    */
    var aminoQuery = query.replace(":table", "amino");
    aminoQuery     = aminoQuery.replace(":hipH", hipH);
    aminoQuery     = aminoQuery.replace(":IDset", aminoID.join(","));
    m_query.prepare(aminoQuery);
    
    if (!m_query.exec())
    {
	//print ("Failed to exec Amino Query");
	retVal.ERROR = "Failed to execute query!";
	return retVal;
    }
    
    for (var i = 0; m_query.next(); i++)
    {
	//print("Amino Loop");
	//var name = aminos[i].NAME;
	var name = m_query.value(3);
	aminos[i].NAME = "qtAminoAcids";

	var moles;
	//retVal[i] = aminos[i];

	for (var k = 0; k < aminos.length; k++)
	{
		if (aminos[k].ID == m_query.value(4))
		{
		    moles = parseFloat(aminos[k].VALUE);
		}
	}

        var Mr    = parseFloat(m_query.value(0));
        var VBar  = parseFloat(m_query.value(1));
	var hydration = parseFloat(m_query.value(2));

	csumd1 = csumd1 + moles * hydration;

//	print("name=" + name);
//	print("moles from Aminos query=" + moles);
//	print("Mr from Aminos query=" + Mr);
//	print("VBar from Aminos query=" + VBar);
//	print("hydration from Aminos query=" + hydration);

	if (name == "CY2")
	{
	    cext  = cext + moles * 120;
	    cext2 = cext2 + moles * 125;
	    //print("CY2 cext2=" + cext2);
	    TotalAminos += moles * 2;
	    cSumNt += moles;
	}
	else if (name == "TRP")
	{
	    cext  = cext + moles * 5690;
	    cext2 = cext2 + moles * 5500;
	    //print("TRP cext2=" + cext2);
	    //TotalAminos += moles * 2;
	    TotalAminos += moles;
	}
	else if (name == "TYR")
	{
	    cext  = cext + moles * 1280;
	    cext2 = cext2 + moles * 1490;
	    //print("TYR cext2=" + cext2);
	    //TotalAminos += moles * 2;
	    TotalAminos += moles;
	}
	else
	{
	    TotalAminos += moles;
	    cSumNt += moles/2;
	}
	if ((name == "PHE")||(name == "TRP")||(name == "TYR")||(name == "HIS"))
	{
		cSumNt += moles;
	}

	/*
	** Verify that record in DB contains numerical values
	** Mr
	*/
	if (isNaN(Mr))
	{
	    aminos[i].ERROR = 
		"Value of Mr for \'" + name + "\' is Not a Number";
	    aminos[i].STYLESHEET = "background-color:rgb(#DDA6A6);";
	    aminos[i].TOOLTIP = aminos[i].ERROR;
	    continue;
	}

	/*
	** Check that moles is positive. Should we fail or continue here?
	** (for now fail...)
	*/
	if (moles <= 0)
	{
	    aminos[i].STYLESHEET ="background-color:rgb(#DDA6A6);"
	    aminos[i].TOOLTIP =
		"Moles of \'" + name + "\' must be greater than 0."; 
	    aminos[i].ERROR = aminos[i].TOOLTIP;
	    continue; 
	}

	/*
	** Verify that record in DB contains numerical values
	** Vbar
	*/
	if (isNaN(VBar))
	{
	    aminos[i].ERROR = 
		"Value of VBar for \'" + name + "\' is Not a Number";
	    aminos[i].STYLESHEET = "background-color:rgb(#DDA6A6);";
	    aminos[i].TOOLTIP = aminos[i].ERROR;
	    continue;
	}
	
	/*
	** Verify that record in DB contains numerical values
	** HipH. 
	** Can't compute Hydration estimate, but can compute everything else
	*/
	if (isNaN(hydration))
	{
	    aminos[i].ERROR = 
		"Unable to calculate hydration estimate for this record.\n\n" +
		"Value of " + hipH + " for \'" + name + "\' is Not a Number.";
	    aminos[i].STYLESHEET = "background-color:rgb(#FFFF69);";
	    aminos[i].TOOLTIP = aminos[i].WARNING;
	}
	
	/*
	** Sum up hydration
	*/
	else
	{
	    //print("new XX calcHydration=" + calcHydration + " += " + moles + " * " + hydration);
            calcHydration += moles * hydration
	    //print("new XX calcHydration=" + calcHydration);
	}
	//print("teh 2calcHydration=" + calcHydration);
        
        /*
	** Sum up Mass, Volume and MW
	*/
        calcVol       += moles * Mr * VBar
        calcMW        += moles * Mr 
    //print("new XX calcMW=" + calcMW);
	
	//print("\tcalcVol=" + calcVol);
	//print("\tcalcMW=" + calcMW);
	//print("teh calcMW=" + calcMW);
	retVal.TOTALAMINOS = new Object();
	retVal.TOTALAMINOS.NAME  = "qtTotalAminos";
	retVal.TOTALAMINOS.VALUE = TotalAminos;
	//print("TotalAminps=" + TotalAminos);
    }
    cSumNt += -0.5;
    calcVolcov = calcVol;
 
    /*
    ** Query the Conjugates table
    */
    query = "SELECT Mr, Vbar, :hipH, covalent, name, carbo FROM :table WHERE id in (:IDset)";
    query = query.replace(":table", "conjugates");
    query = query.replace(":hipH", hipH);
    query = query.replace(":IDset", conjugateID.join(","));
    m_query.prepare(query);
    
    if (!m_query.exec())
    {
	//print ("Failed to exec Conjugate Query");
	retVal.ERROR = "Failed to execute query!";
	return retVal;
    }

 
    for (var i = 0; m_query.next(); i++)
    {
	//print("Conjugate Loop");
	//var name = conjugates[i].NAME;
	var name = m_query.value(4);
	var location = 0;
	for (var j = 0; j < conjugates.length; j++)
	{
/*	print("\n\n\n\n ==============================\n")
	print("name = " + name);
	print("conjugates["+j+"].NAME = " +conjugates[j].NAME);
	print("\n====================================");
*/
		if (name == conjugates[j].NAME)
		{
			conjugates[j].NAME = "qtConjugates";
			location = j;
			break;
		}
	}
	//retVal[i] = conjugates[i];
	
	var moles = parseFloat(conjugates[location].VALUE);
        var Mr    = parseFloat(m_query.value(0));
        var VBar  = parseFloat(m_query.value(1));
	var hydration = parseFloat(m_query.value(2));
	var covalent  = parseFloat(m_query.value(3));
	var carbo = parseFloat(m_query.value(4));
	csumd1 = csumd1 + moles * hydration;

//	print("\tname=" +name);
//	print("\tmoles from conjugates query=" + moles);
//	print("\tMr from conjugates query=" + Mr);
//	print("\tVBar from conjugates query=" + VBar);
//	print("\thydration from conjugates query=" + hydration);
//	print("\tcarbo from conjugates query=" + carbo);
	/*
	** Verify that record in DB contains numerical values
	** Mr
	*/
	if (isNaN(Mr))
	{
	    conjugates[i].ERROR = 
		"Value of Mr for \'" + name + "\' is Not a Number";
	    conjugates[i].STYLESHEET = "background-color:rgb(#DDA6A6);";
	    conjugates[i].TOOLTIP = conjugates[i].ERROR;
	    continue;
	}

	/*
	** Check that moles is positive. Should we fail or continue here?
	** (for now fail...)
	*/
	if (moles <= 0)
	{
	    conjugates[i].STYLESHEET ="background-color:rgb(#DDA6A6);"
	    conjugates[i].TOOLTIP =
		"Moles of \'" + name + "\' must be greater than 0."; 
	    conjugates[i].ERROR = conjugates[i].TOOLTIP;
	    continue; 
	}
	else
	{
	    if (m_query2.exec("SELECT ext FROM extinction WHERE name='" + name +"' AND nm=280")
		&& m_query2.next())
	    {
		cext  += cext + moles * parseFloat(m_query2.value(0));
		cext2 += cext2 + moles * parseFloat(m_query2.value(0));
	    }
	}
	if ( carbo == 1)
	{
	    cSumNt += moles;
	}
	/*
	** Verify that record in DB contains numerical values
	** Vbar
	*/
	if (isNaN(VBar))
	{
	    conjugates[i].ERROR = 
		"Value of VBar for \'" + name + "\' is Not a Number";
	    conjugates[i].STYLESHEET = "background-color:rgb(#DDA6A6);";
	    conjugates[i].TOOLTIP = conjugates[i].ERROR;
	    continue;
	}
	
	/*
	** Verify that record in DB contains numerical values
	** HipH. 
	** Can't compute Hydration estimate, but can compute everything else
	*/
	if (isNaN(hydration))
	{
	    conjugates[i].WARNING = 
		"Unable to calculate hydration estimate for this record.\n\n" +
		"Value of " + hipH + " for \'" + name + "\' is Not a Number.";
	    conjugates[i].STYLESHEET = "background-color:rgb(#FFFF69);";
	    conjugates[i].TOOLTIP = conjugates[i].WARNING;
	}
	
	/*
	** Sum up hydration
	*/
	else
            calcHydration += moles * hydration 
   //print("new XX calcHydration=" + calcHydration);
        
        /*
	** Sum up Mass, Volume and MW
	*/
        calcVol       += moles * Mr * VBar
        calcMWC        += moles * Mr 
        if ((covalent == 1)||(covalent == -1))
	{
           calcVolcov += moles * Mr *VBar;
	   calcMWCcov += moles * Mr;
	}
    //print("new XX calcMW=" + calcMW);
	//print("teh calcMW=" + calcMW);
    }
    
    //print("teh before hydration calc calcHydration=" + calcHydration);
    //print("teh before hydration calc calcMW=" + calcMW);
    // Equation 26 in documentation
    calcHydration =  calcHydration * (18 / (calcMW+calcMWC));
    //print("new XX calcHydration=" + calcHydration);
	
    /*
    ** Correct MR for "oligomer N"
    */
    //print("going in calcMW=" + calcMW);
    //print("going in oligomerN.VALUE=" + parseInt(oligomerN.VALUE));
    //print("new XX calcMW=" + calcMW);

    // I Tucker Hurton added oligomerMW in order to keep track of 
    // the oligomer version of MW alongside the monomer version so that
    // values dependent on the monomer MW do not get modified when the
    // user wants the oligomer version of MW.
    var oligomerMW = calcMW * parseInt(oligomerN.VALUE);
    //calcMW = calcMW * parseInt(oligomerN.VALUE);
    //print("2 calcMW=" + calcMW);

    /*
    ** Calculate VBar, but make sure we don't divide by zero
    ** //! For some reason we need to add 18, something to do with a 
    ** //! missing h2o?
    */ 
//var calcSampleVBar = 0;
//print("2 calcSampleVBar"+ calcSampleVBar);
//print("2 sedEqTemp"+ sedEqTemp.VALUE);
    if ((calcMW > 0)||(calcMWC > 0))
    {
	calcVBar = calcVol / (oligomerMW/parseInt(oligomerN.VALUE) + calcMWC) 
	calcVBarwater = calcVolcov / (oligomerMW/parseInt(oligomerN.VALUE) + calcMWCcov);
 	oligomerMW += 18.0152 * oligomerN.VALUE + oligomerN.VALUE*calcMWC; 

	//print("3 calcVBar=" + calcVBar);
	//print("3 calcMW=" + calcMW);
	//print("3 oligomerMW=" + oligomerMW);
    }
//    oligomerMW += calcMWC;
//    retVal.SAMPLEVBAR = new Object();
//    retVal.SAMPLEVBAR.NAME = "qtSampleVbar";
//    retVal.SAMPLEVBAR.VALUE = calcSampleVBar;
//    print("calcSampleVBar=" + calcSampleVBar);
//    print("This is the text of denaturant " + denaturant.VALUE);
    var phi2 = 0;
    phi2 = calcVBar;
    if (denaturant.VALUE != "")
    {
	var delta3 = 0;
	var delta1 = calcHydration;
	query =  "SELECT rhomin, v3, g3, Md, carbo FROM denature WHERE id = ':ID'";
	query = query.replace(":ID", denaturant.VALUE);
//	print("This is the SQLQuery \n---------\n" + query + "\n----------");
        m_query.prepare(query);
        
	if (!m_query.exec())
	{
	   print("Failed to excute query");
	}
	else
	{
	    print("Successful query");
	}
        for (var i = 0; m_query.next(); i++)
        {
	    var rhomin = parseFloat(m_query.value(0));
	    var v3     = parseFloat(m_query.value(1));
	    var g3     = parseFloat(m_query.value(2));
	    var Md     = parseFloat(m_query.value(3));
	    var carbo  = parseFloat(m_query.value(4));

//	    print("\trhomin="+rhomin);
//	    print("\tv3="+v3);
//	    print("\tg3="+g3);
//	    print("\tMd="+Md);
//	    print("\tcarbo="+carbo);
//	    print("\tcSumNt="+cSumNt);

	    delta3 = (Md/(calcMW))*(cSumNt);
//	    print("\tdelta3 = " + delta3);
	    phi2 = calcVBar - (1/rhomin - v3) * (delta3 - g3*delta1);
//	    print("\tphi2 = "+ phi2);
//	    calcVBar = phi2;	
	}
    }

    retVal.UNDENATUREDVBAR = new Object();
    retVal.UNDENATUREDVBAR.NAME = "qtUnDenaturedVBar";
    retVal.UNDENATUREDVBAR.VALUE = _calculateVBar20(calcVBar,25);

    retVal.VBARWATER = new Object();
    retVal.VBARWATER.NAME = "qtCalculatedVBarWater";
    retVal.VBARWATER.VALUE = _calculateVBar20(calcVBarwater, 25);

    retVal.VBAR = new Object();
    retVal.VBAR.NAME  = "qtCalculatedVBar";
    retVal.VBAR.VALUE = phi2;

   
    retVal.MR = new Object();
    retVal.MR.NAME  = "qtCalculatedMR";
    //retVal.MR.VALUE = calcMW;
    retVal.MR.VALUE = oligomerMW;
    //print("calcMW=" + calcMW);
    
    retVal.HYDRATION = new Object();
    retVal.HYDRATION.NAME  = "qtCalculatedHydration";
    //print("new XX calcHydration to xml=" + calcHydration);
    retVal.HYDRATION.VALUE = calcHydration;
    //print("calcMW=" + calcHydration)

    //print "VBar: " + str(calcVBar)
    //print "Mr: " + str(calcMW)
    //print "Hydration: " + str(calcHydration)

    //_printProps(" ", retVal);
  
    // While we're here, compute the values necessary to graph

    retVal.GRAPHCHARGEX = new Array(121);
    retVal.GRAPHCHARGEY = new Array(121);
    _createChargeGraphData(retVal.GRAPHCHARGEX, retVal.GRAPHCHARGEY,UseTitrableAmino,
			   AminoEndpKaValue,UseTitrableCarboxyl,
			   CarboxylValue,oligomerN,aminos,conjugates);

    retVal.GRAPHABSX = new Array(100);
    retVal.GRAPHABSY = new Array(100);
    _createAbsGraphData(retVal.GRAPHABSX,retVal.GRAPHABSY,aminos,conjugates);

    if (parseInt(oligomerN.VALUE) > 1)
    {
	cext = parseInt(oligomerN.VALUE) * cext;
	cext2 = parseInt(oligomerN.VALUE) * cext2;
    }

    /*
    ** Calculate Isoelectric Charge and charge at pH  7.0
    */
    var isopoint = _calcPi(UseTitrableAmino,AminoEndpKaValue,UseTitrableCarboxyl,
			   CarboxylValue,oligomerN,aminos,conjugates);
    //print("calcPi=" + isopoint);

    retVal.PI = new Object();
    retVal.PI.NAME  = "qtIsoElectricPoint";
    retVal.PI.VALUE = isopoint.toPrecision(3);

    var pHCharge = _chargeFcn(pH,UseTitrableAmino,AminoEndpKaValue,UseTitrableCarboxyl,
			      CarboxylValue,oligomerN,aminos,conjugates);
    //print("pHCharge=" + pHCharge);
    retVal.CHARGE = new Object();
    retVal.CHARGE.NAME = "qtpHCharge";
    retVal.CHARGE.VALUE = pHCharge;

    retVal.CHARGELABEL = new Object();
    retVal.CHARGELABEL.NAME = "qtpHLabel";
    retVal.CHARGELABEL.ISLABEL = "true";
    retVal.CHARGELABEL.VALUE = "Charge at pH " + pH.toPrecision(1);

    retVal.EXTINCTION1 = new Object();
    retVal.EXTINCTION1.NAME = "qtAbsExtinction";
    retVal.EXTINCTION1.VALUE = parseFloat(cext).toExponential(3);

    retVal.ABSORPTIVITY1 = new Object();
    retVal.ABSORPTIVITY1.NAME = "qtAbsMgMl";
    retVal.ABSORPTIVITY1.VALUE = parseFloat(cext/(calcMW *parseInt(oligomerN.VALUE))).toExponential(3);

    retVal.EXTINCTION2 = new Object();
    retVal.EXTINCTION2.NAME = "qtEstAbsExtinction";
    retVal.EXTINCTION2.VALUE = parseFloat(cext2).toExponential(3);

    retVal.ABSORPTIVITY2 = new Object();
    retVal.ABSORPTIVITY2.NAME = "qtEstAbsMgMl";
    retVal.ABSORPTIVITY2.VALUE = parseFloat(cext2/(calcMW *parseInt(oligomerN.VALUE))).toExponential(3);

    return retVal;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////
function calculateSampleVbar(vBar,sedEqTemp)
{
    print(">>calculateSampleVbar(vBar,sedEqTemp)")
    //print("vBar = " + vBar.VALUE);
    //print("sedEqTemp=" + sedEqTemp.VALUE);
    var calcSampleVBar = 0;
    var retVal = new Object();
    var t = 0;
    var r = 0;
    var g = 0;
    
    t=parseFloat(vBar.VALUE);
    r=parseFloat(sedEqTemp.VALUE);
    g=parseFloat(4.25*(Math.pow(10,-4)))*(r-25);
    //print("t =  " + t);
    //print("g =  " + g);
    calcSampleVBar = t + g;
    retVal.SAMPLEVBAR = new Object();
    retVal.SAMPLEVBAR.NAME = "qtSampleVbar";
    retVal.SAMPLEVBAR.VALUE = calcSampleVBar;

    return retVal;
}

function _calculateSampleVbar(vBar,sedEqTemp)
{
    print(">>_calculateSampleVbar(vBar,sedEqTemp)")
    //print("vBar = " + vBar.VALUE);
    //print("sedEqTemp=" + sedEqTemp.VALUE);
    var calcSampleVBar = 0;
    var retVal = new Object();
    var t = 0;
    var r = 0;
    var g = 0;
    
    t=parseFloat(vBar.VALUE);
    
    r=parseFloat(sedEqTemp.VALUE);
    g=parseFloat(4.25*(Math.pow(10,-4)))*(r-20);
    //print("t =  " + t);
    //print("g =  " + g);
    calcSampleVBar = t + g;
    retVal.SAMPLEVBAR = new Object();
    retVal.SAMPLEVBAR.NAME = "qtSampleVbar";
    retVal.SAMPLEVBAR.VALUE = calcSampleVBar;
    
    return retVal;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////

/*
** Given a single object, (and an indentation) print all of the object's
** attributes/properties, processing child objects recursively
*/
function _printProps(indent, object)
{
    for (var prop in object)
    {
	//print(indent + prop + " = " + object[prop]);

	if (typeof(object[prop]) == "object")
	    _printProps(indent + "  ", object[prop]);
    }
}


//function testSQL(arrayObj)
//{
//    return arrayObj;
//
//    var error  = new Array();
//    error.name = "DensityRecordError";
//    error[0] = new Object();
//    error[0].id = 1;
//    error[0].errorObj = new Object();
//    error[0].errorObj.value = "Error Message";
//    error[1] = new Object();
//    error[1].SomeAttribute = 123;
//    error[1].value = "Hello World";
//    error[2] = new Object();
//    error[2].id = 5;
//    error[2].value = "The molar concentration of 1 proponal...";
//    
//    var increment = new Object();
//    increment.name = "DensityIncrement"
//    increment.value = 0;
//    
//    var retVal = new Object();
//    retVal.sillyAttr ="random attribute";
//    retVal.errors = error;
//    retVal.density = increment;
//    return retVal;
//    
//
//    //return arrayObj;
//    /*
//    printProps("", arrayObj);
//    var retVal = new Array();
//    //retVal.name = "bill";
//    return retVal;
//
//
//    print("testSQL");
//    for (var prop in arrayObj)
//    {
//	print (prop + " = " + arrayObj[prop]);
//    }
//    */
//
//
//    var retVal = new Object();
//    var aa = new Object();
//    aa.NAME = "Acetic Acid";
//    aa.id   = 1;
//    
//    var bb = new Object();
//    bb.NAME = "1-proponal";
//    bb.id   = 2;
//
//    retVal[1] = aa;
//    retVal[2] = bb;
//    return retVal;
//
//
//    var c = new Object();
//    arrayObj["Acetic Acid"].child = c;
//    c.Property = 123;
//
//    return arrayObj;
//
//
//    print ("TestSQL()");
//    //m_db.setDatabaseName(PHYCONST_DB);
//    var isOpen = m_db.open();
//
//    if (!isOpen)
//    {
//	print("Failed to open " + PHYCONST_DB);
//	return;
//    }
//
//
//    //var sqlEngine = m_db.exec("SELECT * FROM rhoseries");
//    m_query.prepare("SELECT * FROM rhoseries");
//    m_query.bindValue(":id", 5);
//    var success = m_query.exec();
//    
//    if (!success)
//	print ("Failed to exec query");
//    
//    while (m_query.next())
//    {
//	print ("blah");
//	print (m_query.value(0) 
//	       + " " + m_query.value(1)
//	       + " " + m_query.value(2) 
//	       + " " + m_query.value(3)
//	       + " " + m_query.value(4)
//	       + " " + m_query.value(5));
//    }
//}


// We may want to fix this to query all records at once, rather than
// looping and making individual queries. (If possible..)

/*
** Calculates the bufferDensity for a list of added components
** \param  Array object that contains density "records" where each consist
**         of Name, recordId, Concentration, units, and Frontend table "row"
** \retVal Object with array of returned records (with or without errors),
**         and Density value
*/
function calculateBufferDensity(params)
{
    //print(">>Function: calculateBufferDensity(params)");

    //for (var i = 0; i < params.length; i++)
	//print ("Param " + i + " " + params[i]);
    
    // Start with no density, and sum up increments
    var retVal = new Array();
    //retVal.NAME = "qtAt20Value";
    //retVal.VALUE = 0;
    retVal.qtDensity = new Object();
    retVal.qtDensity.NAME = "qtDensity";
    retVal.qtDensity.VALUE = 0;

    
    //var ID, C, units;
    
    /*
    ** Loop through each parameter, pulling out ID,C,units in threes 
    */

    for (var i = 0, errorCount = 0; i < params.length; i++)
    {
	///////////////////////
	var name          = params[i].NAME;
	var id            = parseInt(params[i].ID);
	var concentration = parseFloat(params[i].VALUE);
	var units         = params[i].UNITS;
	
	//print ("Params[i].NAME " + name);
	//print ("Params[i].ID " + id);
	//print ("Params[i].VALUE " + concentration);
	//print ("Params[i].UNITS " + units);
	

	/*
	** Return the row with new STYLESHEET information
	** Cheat a little bit by changing name to the TableWidget's
	** so that everything gets returned as it was passed in 
	** (With stylesheet/tooltip modifications)
	*/
	// Must Already have ROW information
	params[i].NAME = "qtAddedComponents";
	params[i].COL  = 1; 
	params[i].STYLESHEET = "background-color:rgb(#FFFFFF);";
	params[i].TOOLTIP = "";
	retVal[i] = params[i];

	/*
	** Make sure the entered concentration is a number
	*/
	if (isNaN(concentration))
	{
	    params[i].STYLESHEET = "background-color:rgb(#DDA6A6)";
	    continue;
	}
	
	/*
	** Check that C is positive. Should we fail or continue here?
	** (for now fail...)
	*/
	if (concentration <= 0)
	{
	    params[i].STYLESHEET ="background-color:rgb(#DDA6A6);"
	    params[i].TOOLTIP =
		"The Molar Concentration of \'" + name +
		"\' must be greater than 0."; 
	    params[i].ERROR = params[i].TOOLTIP;
	    continue; 
	}

	
	/*
	** Query the density record from the database
	*/
	//m_query.prepare("SELECT name, Cmax, a, b, c, d, e, f " +
	//	"FROM rhoseries WHERE id = :id");
	
	/*
	** It shouldn't ever be the case that a rhoseries record is "linked"
	** to by more than one bufferComponent, but its not enforced by the 
	** database. It might be appropriate to force a 1 to 1 relationship
	** at the DB level in the future. 
	*/
	m_query.prepare("SELECT componentMr, componentDensity, " +
			"bufferComponent.name, " +
			"Cmax, a, b, c, d, e, f " +
			"FROM rhoseries " +
			"JOIN bufferComponent ON " +
			"bufferComponent.componentRhoId = rhoseries.id " + 
			"WHERE rhoseries.id = :id ");
	m_query.bindValue(":id", id);
	
	/*
	** Query failed to execute
	*/
	if (!m_query.exec())
	{
	    // Get rid of whatever junk was in retVal
	    delete retVal;
	    retVal = new Object();
	    retVal.ERROR = "Query failed to execute!";
	    return retVal;
	}
	
	/* Record wasn't found in rhoseries table */
	if (!m_query.next())
	{
	    //"Density record not found. Id: " + params[i].ID;
	    //return retVal;
	    params[i].ERROR = "Density record not found for ID: " + id; 
	    continue;
	}
	
	/* Convert to MOLAR units, reporting errors via ERROR attribute */
	var conversion = _convertToMolar(m_query.value(2).toString(),
					 parseFloat(m_query.value(0)),
					 parseFloat(m_query.value(1)),
					 concentration, units);
	
	/* 
	** GUI tries to display only units which are valid for any given 
	** component, but its not always possible. 
	** 
	** A solvent may be saved with a component which has valid units.
	** Component record may be changed. Solvent may be reloaded.
	** Component may now have units which are no longer valid given its new 
	** record data.
	*/
	if ('WARNING' in conversion)
	{
	    params[i].WARNING = conversion.WARNING;
	    continue;
	}
	
	if ('ERROR' in conversion)
	{
	    params[i].ERROR = conversion.ERROR;
	    continue;
	}
	
	
	/* Set molar concetration */
	concentration = conversion.CONCENTRATION;
		
	/* Grab each field from the query */
	var name = m_query.value(2);
	var Cmax = parseFloat(m_query.value(3));
	var a    = parseFloat(m_query.value(4));
	var b    = parseFloat(m_query.value(5));
	var c    = parseFloat(m_query.value(6));
	var d    = parseFloat(m_query.value(7));
	var e    = parseFloat(m_query.value(8));
	var f    = parseFloat(m_query.value(9));
	
	/*
	** Check CMax is valid. 
	** Notify user on error, but continue calculating anyway
	*/
	if (concentration > Cmax)
	{
	    params[i].WARNING = 
		"The Molar Concentration of \'" + name + 
		"\' is above the\n" +Cmax + "M maximum of the Density formula.";
	    params[i].STYLESHEET = "background-color:rgb(#FFFF69);";
	    params[i].TOOLTIP = params[i].WARNING;
	}
	
	/*
	** Calculate and add the density increment
	*/
	retVal.qtDensity.VALUE += a + 
            b * 0.001 * Math.pow(concentration, 1.0 / 2.0) +
            c * 0.01 * concentration + 
	    d * 0.001 * Math.pow(concentration, 2) + 
            e * 0.0001 * Math.pow(concentration, 3) + 
            f * 0.000001 * Math.pow(concentration, 4) - Density_20_Water;
    }
    
    retVal.qtDensity.VALUE += Density_20_Water;
    return retVal;
}


/*
** Calculates the bufferViscosity for a list of added components
** \param  Array object that contains Viscosity "records" which each consist
**         of Name, recordId, Concentration, units, and Frontend table "row"
** \retVal Object with array of returned records (with or without errors),
**         and Viscosity value
*/
function calculateBufferViscosity(params)
{
    //! Debug
  //  print("CaluclateBufferViscosity");
    var i = 0;
    //for (i = 0; i < params.length; i++)
	//print ("Param " + i + " " + params[i]);
    
    // Start with no Viscosity, and sum up increments
    var retVal = new Array();
    //retVal.NAME = "qtAt20Value";
    //retVal.VALUE = 0;
    retVal.qtViscosity = new Object();
    retVal.qtViscosity.NAME = "qtViscosity";
    retVal.qtViscosity.VALUE = 0;
    
    //var ID, C, units;
    
    /*
    ** Loop through each parameter, pulling out ID,C,units in threes 
    */
    for (i = 0, errorCount = 0; i < params.length; i++)
    {
	///////////////////////
	var name          = params[i].NAME;
	var id            = parseInt(params[i].ID);
	var concentration = parseFloat(params[i].VALUE);
	var units         = params[i].UNITS;
	
	//print ("Params[i].NAME " + name);
	//print ("Params[i].ID " + id);
	//print ("Params[i].VALUE " + concentration);
	//print ("Params[i].UNITS " + units);
	
	/*
	** Convert to MOLAR units
	*/
	//! 
	//  if (units != "molar")
	//!

	/*
	** Return the row with new STYLESHEET information
	** Cheat a little bit by changing name to the TableWidget's
	** so that everything gets returned as it was passed in 
	** (With stylesheet/tooltip modifications)
	*/
	// Must Already have ROW information
	params[i].NAME = "qtAddedComponents";
	params[i].COL  = 1; 
	params[i].STYLESHEET = "background-color:rgb(#FFFFFF);";
	params[i].TOOLTIP = "";
	retVal[i] = params[i];

	/*
	** Make sure the entered concentration is a number
	*/
	if (isNaN(concentration))
	{
	    params[i].STYLESHEET = "background-color:rgb(#DDA6A6)";
	    continue;
	}

	/*
	** Check that C is positive. Should we fail or continue here?
	** (for now fail...)
	*/
	if (concentration <= 0)
	{
	    params[i].STYLESHEET ="background-color:rgb(#DDA6A6);"
	    params[i].TOOLTIP =
		"The Molar Concentration of \'" + name +
		"\' must be greater than 0."; 
	    params[i].ERROR = params[i].TOOLTIP;
	    continue; 
	}

	
	/*
	** Query the viscosity record from the database
	*/
	//m_query.prepare("SELECT name, Cmax, a, b, c, d, e " +
	//	"FROM viscseries WHERE id = :id");
	/*
	** It shouldn't ever be the case that a viscseries record is "linked"
	** to by more than one bufferComponent, but its not enforced by the 
	** database. It might be appropriate to force a 1 to 1 relationship
	** at the DB level in the future. 
	*/
	m_query.prepare("SELECT componentMr, componentDensity, " +
			"bufferComponent.name, " +
			"Cmax, a, b, c, d, e " +
			"FROM viscseries " +
			"JOIN bufferComponent ON " +
			"bufferComponent.componentViscId = viscseries.id " + 
			"WHERE viscseries.id = :id ");
	m_query.bindValue(":id", id);
	
	/*
	** Query failed to execute
	*/
	if (!m_query.exec())
	{
	    // Get rid of whatever junk was in retVal
	    delete retVal;
	    retVal = new Object();
	    retVal.ERROR = "Query failed to execute!";
	    return retVal;
	}
	
	/* 
	** Record wasn't found in viscseries table
	*/
	if (!m_query.next())
	{
	    params[i].ERROR = "Viscosity record not found for ID: " + id; 
	    continue;
	}
	
	/* Convert to MOLAR units, reporting errors via ERROR attribute */
	var conversion = _convertToMolar(m_query.value(2).toString(),
					 parseFloat(m_query.value(0)),
					 parseFloat(m_query.value(1)),
					 concentration, units);
	
	/* 
	** GUI tries to display only units which are valid for any given 
	** component, but its not always possible. 
	** 
	** A solvent may be saved with a component which has valid units.
	** Component record may be changed. Solvent may be reloaded.
	** Component may now have units which are no longer valid given its new 
	** record data.
	*/
	if ('WARNING' in conversion)
	{
	    params[i].WARNING = conversion.WARNING;
	    continue;
	}
	else if ('ERROR' in conversion)
	{
	    params[i].ERROR = conversion.ERROR;
	    continue;
	}
	
	/* Set molar concetration */
	concentration = conversion.CONCENTRATION;

	/*
	** Grab each field from the query
	*/
	var name = m_query.value(2);
	var Cmax = parseFloat(m_query.value(3));
	var a    = parseFloat(m_query.value(4));
	var b    = parseFloat(m_query.value(5));
	var c    = parseFloat(m_query.value(6));
	var d    = parseFloat(m_query.value(7));
	var e    = parseFloat(m_query.value(8));
	var f    = 0.0;//parseFloat(m_query.value(7));
	
	/*
	** Check CMax is valid. 
	** Notify user on error, but continue anyway
	*/
	if (concentration > Cmax)
	{
	    params[i].WARNING = 
		"The Molar Concentration of \'" + name + 
		"\' is above\nthe " + Cmax + "M maximum of the Viscosity " +
		"formula.";
	    params[i].STYLESHEET = "background-color:rgb(#FFFF69);";
	    params[i].TOOLTIP = params[i].WARNING;
	}
	
	/*
	** Calculate and add the density increment
	*/
	retVal.qtViscosity.VALUE += 
	Viscosity_20_Water * ((a - 1) + 
			      b * 0.001 * Math.pow(concentration, 1.0 / 2.0) +
			      c * 0.01 * concentration + 
			      d * 0.001 * Math.pow(concentration, 2) + 
			      e * 0.0001 * Math.pow(concentration, 3) + 
			      f * 0.000001 * Math.pow(concentration, 4))
    }
    
    retVal.qtViscosity.VALUE += Viscosity_20_Water;
    return retVal;
}


/*
** Corrects calculatedBufferViscosity for temperature
** (Used in GuiBufferComponents form)
*/
function correctBufferViscosityAtT(viscosity, temperature)
{
    print(">>Function: correctBufferViscosityAtT(viscosity, temperature) ");
    //print("\tviscosity=" + viscosity.VALUE);
    //print("\ttemperature=" + temperature.VALUE);

    visc = parseFloat(viscosity.VALUE);
    temp = parseFloat(temperature.VALUE);
    
    var obj = new Object(); 
    obj.NAME = "qtCorrectedViscosity";
    obj.VALUE = _calculateViscAtT(visc, temp, 1);
    return obj;
}


/*
** Corrects calculatedBufferDensity for temperature
** (Used in GuiBufferComponents form when no isotopes of water are added)
*/
function correctBufferDensityAtT_Normal(density, temperature)
{
    print(">>Function: correctBufferDensityAtT(density, temperature) ");
    //print("\tdensity=" + density.VALUE);
    //print("\ttemperature=" + temperature.VALUE);

    dens = parseFloat(density.VALUE);
    temp = parseFloat(temperature.VALUE);
    
    var obj = new Object(); 
    obj.NAME = "qtCorrectedDensity";
    obj.VALUE = _calculateDensityAtT(temp, dens);
    return obj;
}


/*
** Corrects calculatedBufferDensity for temperature
** (Used in GuiBufferComponents form when isotopes of water ARE added)
*/
function correctBufferDensityAtT_Heavy(density, temperature, D2O, D2O18, H2O18)
{
    print(">>Function: correctBufferDensityAtT_Heavy(density, temperature, D20, D2018,H2018) ");
    //print("\tdensity=" + density.VALUE);
    //print("\ttemperature=" + temperature.VALUE);
    //print("\tD20=" + D20.VALUE);
    //print("\tD2018=" + D2018.VALUE);
    //print("\tH2018=" + H2018.VALUE);

    density = parseFloat(density.VALUE);
    T = parseFloat(temperature.VALUE);

    //print("density=" + density);
    //print("T=" + T);
    D2O       = parseFloat(D2O.VALUE);
    D2O18     = parseFloat(D2O18.VALUE);
    H2O18     = parseFloat(H2O18.VALUE);
    
    //#Need to handle freezing point error checking eventually, although
    //#Not sure the original actually does anything?
    
    //# If (fD2O > 0 And fT < 3.81) Or (fD2O18 > 0 And fT < 4.02) 
    //  Or (fH2O18 > 0 And fT < 0.28) Then
    //#            'we are below freezing point for
    //#            'one of the isotopes that is present
    //#            'so return error
    //#            CalcRhoAtTHeavy = 0

    var delrho   = 0;
    var rhowater = _calculateDensityAtT(T, Density_20_Water);

    if (D2O > 0)
	delrho = delrho + D2O * (_CalcRhoD2OAtT(T) - rhowater);

    if (D2O18 > 0)
	delrho = delrho + D2O18 * (_CalcRhoD2O18AtT(T) - rhowater);
    
    if (H2O18 > 0)
	delrho = delrho + H2O18 * (_CalcRhoH2O18AtT(T) - rhowater);

    var obj = new Object(); 
    obj.NAME = "qtCorrectedDensity";
    obj.VALUE = density * rhowater / Density_20_Water + delrho;
    return obj;
}


/*
** Calculates density of D2O at T
**  Sednterp documentation claims constants come from the database,
** but the VB code hard codes them. For now, we'll hardcode them too
*/
function _CalcRhoD2OAtT(T)
{
    print(">>Function: _CalcRhoD2OAtT(T) ");
    //print("\tT=" + T);
    // Need to handle temp bounds checking somehow...
    //     If fT >= 3.81 And fT <= 80 Then

    return 1.10602 * (1 - 1.0555 * Math.pow((T - 11.24), 2) * 
                      (1.74224 + 482.502 / (T - 11.24 + 77.861)) * 0.000001);
}


/*
** Calculates density of D2O18 at T
** Sednterp documentation claims constants come from the database,
** but the VB code hard codes them. For now, we'll hardcode them too
*/
function _CalcRhoD2O18AtT(T)
{
    print(">>Function: _CalcRhoD2O18AtT(T) ");
    //print("\tT=" + T);
    // Need to handle temp bounds checking somehow
    //     If fT >= 4.02 And fT <= 80 Then
    return 1.21691 * (1 - 1.0555 * Math.pow((T - 11.46), 2) * 
                      (1.74224 + 482.502 / (T - 11.46 + 77.861)) * 0.000001);
}
    

/*
** Calculates density of H2O18 at T
** Sednterp documentation claims constants come from the database,
** but the VB code hard codes them. For now, we'll hardcode them too
*/
function _CalcRhoH2O18AtT(T)
{
    print(">>Function: _CalcRhoH2O18AtT(T) ");
    //print("\tT=" + T);
    // Need to handle temp bounds checking somehow
    //    If fT >= 0.28 And fT <= 80 Then
    
    return 1.11255 * (1 - Math.pow((T - 4.305), 2) *
                      (1.74224 + 482.502 / (T - 4.305 + 77.861)) * 
                      0.000001);
}

/*! Returns Amino records from the amino table */
function sqlFetchSampleComponents()
{
    var retVal      = new Object();
    var aminos      = new Array();
    var conjugates  = new Array();
    var denaturants = new Array();
   
    m_query.prepare("SELECT id, Namethree, Nameone, Mr, pka " +
                    "FROM amino " +
                    "WHERE active > 0 ORDER BY Namethree COLLATE NOCASE ASC");

    // We should change this to lastError().text() if its accessible
    if (!m_query.exec())
    {
        retVal.ERROR = "Failed to execute amino query!";
        return retVal;
    }

    for (var i = 0; m_query.next(); i++)
    {
        aminos[i] = new Object();
        aminos[i].NAME  = "qtAminoAcids";
        aminos[i].AMINOID = m_query.value(0).toString();
        aminos[i].NAMETHREE = m_query.value(1).toString();
        aminos[i].NAMEONE = m_query.value(2).toString();
        aminos[i].MR = m_query.value(3).toString();
        aminos[i].PKA = m_query.value(4).toString();
    }

    m_query.prepare("SELECT id, Name " +
                    "FROM conjugates " +
                    "WHERE active > 0 ORDER BY Name COLLATE NOCASE ASC");

    // We should change this to lastError().text() if its accessible
    if (!m_query.exec())
    {
        retVal.ERROR = "Failed to execute conjugates query!";
        return retVal;
    }

    for (var i = 0; m_query.next(); i++)
    {
        conjugates[i] = new Object();
        conjugates[i].NAME  = "qtConjugates";
        conjugates[i].CONJUGATEID = m_query.value(0).toString();
        conjugates[i].CONJUGATE = m_query.value(1).toString();
    }

    m_query.prepare("SELECT id, Name " +
                    "FROM denature " +
                    "WHERE active > 0 ORDER BY Name COLLATE NOCASE ASC");

    // We should change this to lastError().text() if its accessible
    if (!m_query.exec())
    {
        retVal.ERROR = "Failed to execute denatures query!";
        return retVal;
    }

    for (var i = 0; m_query.next(); i++)
    {
        denaturants[i] = new Object();
        denaturants[i].NAME  = "qtDenaturants";
        denaturants[i].DENATURANTID = m_query.value(0).toString();
        denaturants[i].DENATURANTNAME = m_query.value(1).toString();
    }

    retVal.AMINOS = aminos;
    retVal.CONJUGATES = conjugates;
    retVal.DENATURANTS = denaturants;
    return retVal;
}
/*! Loads the Buffer Page */
function loadSolventPage(id)
{
   var retVal	   = new Array();
   var i = 0;
   id = parseInt(id.VALUE)
   if (id == -1)
	return;
   else
   {
	m_query2.prepare("SELECT densityDirectEntry, viscosityDirectEntry, " +
                    "densityStandard, viscosityStandard, " +
                    "densityStandardT, viscosityStandardT, " +
                    "computeDensity, computeViscosity, " +
                    "pH, isHeavyWater," +
                    "D2O, D2O18, H2O18, H2O " + 
                    "FROM buffer " +
                    "WHERE active > 0 AND id = :id");
	m_query2.bindValue(":id", id);

	if (!m_query2.exec())
	{
	   retVal.ERROR = "Failed to execute buffer query!";
	   return retVal;
	}

	m_query2.next();

	if (m_query2.value(0).toString().length > 0)
	{
           retVal[i] = new Object();
	   retVal[i].NAME = "qtDensityDirectEntry";
	   retVal[i].CHECKED = true;
	   i++;

	   retVal[i] = new Object();
	   retVal[i].NAME = "qtBufferDensity";
	   retVal[i].VALUE = m_query2.value(0).toString();
	   i++

	}
	else
	{
           retVal[i] = new Object();
           retVal[i].NAME = "qtDensityDirectEntry";
           retVal[i].CHECKED = false;
           i++;
	}
	if (m_query2.value(1).toString().length > 0)
	{
           retVal[i] = new Object();
	   retVal[i].NAME = "qtViscosityDirectEntry";
	   retVal[i].CHECKED = true;
	   i++;

	   retVal[i] = new Object();
	   retVal[i].NAME = "qtBufferViscosity";
	   retVal[i].VALUE = m_query2.value(1).toString();
	   i++
	}
	else
	{
           retVal[i] = new Object();
           retVal[i].NAME = "qtViscosityDirectEntry";
           retVal[i].CHECKED = false;
           i++;
	}
	if (m_query2.value(2).toString().length > 0)
	{
           retVal[i] = new Object();
	   retVal[i].NAME = "qtDensityStandard";
	   retVal[i].CHECKED = true;
	   i++;

	   retVal[i] = new Object();
	   retVal[i].NAME = "standardDensity";
	   retVal[i].VALUE = m_query2.value(2).toString();
	   i++

	   retVal[i] = new Object();
	   retVal[i].NAME = "standardDensityTemp";
	   retVal[i].VALUE = m_query2.value(4).toString();
	   i++

//	   var TemperatureObject = new Object();
//	   TemperatureObject.NAME = "Temperature";
//	   TemperatureObject.VALUE = m_query2.value(4).toString();

//	   var VBarObject = new Object();
//	   VBarObject.NAME = "VBar";
//	   VBarObject.VALUE = m_query2.value(2).toString();

//	   calculateVBar20(VBarObject,TemperatureObject);
	}
	else
	{
           retVal[i] = new Object();
           retVal[i].NAME = "qtDensityStandard";
           retVal[i].CHECKED = false;
           i++;
	}
	if (m_query2.value(3).toString().length > 0)
	{
           retVal[i] = new Object();
	   retVal[i].NAME = "qtViscosityStandard";
	   retVal[i].CHECKED = true;
	   i++;

	   retVal[i] = new Object();
	   retVal[i].NAME = "standardViscosity";
	   retVal[i].VALUE = m_query2.value(3).toString();
	   i++

	   retVal[i] = new Object();
	   retVal[i].NAME = "standardViscosityTemp";
	   retVal[i].VALUE = m_query2.value(5).toString();
	   i++
	}
	else
	{
           retVal[i] = new Object();
           retVal[i].NAME = "qtViscosityStandard";
           retVal[i].CHECKED = false;
           i++;
	}
	if (m_query2.value(6).toString().length > 0)
	{
	    retVal[i] = new Object();
	    retVal[i].NAME = "qtDensityCompute";
	    retVal[i].CHECKED = true;
	    i++

	    retVal[i] = new Object();
	    retVal[i].NAME = "qtCalculateDensity";
	    retVal[i].CHECKED = true;
	    i++

	    retVal[i] = new Object();
	    retVal[i].NAME = "qtBufferDensity";
	    retVal[i].VALUE = m_query2.value(6).toString();
	    i++
	}
	else
	{
            retVal[i] = new Object();
            retVal[i].NAME = "qtDensityCompute";
            retVal[i].CHECKED = false;
            i++

            retVal[i] = new Object();
            retVal[i].NAME = "qtCalculateDensity";
            retVal[i].CHECKED = false;
            i++
	}
	if (m_query2.value(7).toString().length > 0)
	{
	     retVal[i] = new Object();
	     retVal[i].NAME = "qtViscosityCompute";
	     retVal[i].CHECKED = true;
	     i++

	     retVal[i] = new Object();
	     retVal[i].NAME = "qtCalculateViscosity";
	     retVal[i].CHECKED = true;
	     i++

	     retVal[i] = new Object();
	     retVal[i].NAME = "qtBufferViscosity";
	     retVal[i].VALUE = m_query2.value(7).toString();
	     i++
	}
	else
	{
             retVal[i] = new Object();
             retVal[i].NAME = "qtViscosityCompute";
             retVal[i].CHECKED = false;
             i++

             retVal[i] = new Object();
             retVal[i].NAME = "qtCalculateViscosity";
             retVal[i].CHECKED = false;
             i++

	}
	retVal[i] = new Object();
	retVal[i].NAME = "qtBufferpH";
	retVal[i].VALUE = m_query2.value(8).toString();
	i++


        m_query2.prepare("SELECT name, componentID, concentration, units " +
			 "FROM bufferComposition " +
			 "JOIN bufferComponent ON " +
			 "bufferComposition.componentID = bufferComponent.id " +
			 "WHERE bufferId = :bufId AND bufferComposition.active > 0");
	m_query2.bindValue(":bufId", id);
	if (!m_query2.exec())
	{
	   retVal.ERROR = "Failed to execute bufferComposition & bufferComponent quer!";
	  return retVal;
	}
	var params = new Array();
	var j = 0;
	while (m_query2.next())
	{
           params[j] = new Object();
	   params[j].NAME = "qtAddedComponents";//m_query2.value(0).toString();
           params[j].ID = parseInt(m_query2.value(1).toString());
           params[j].VALUE = parseFloat(m_query2.value(2).toString());
	   params[j].UNITS = m_query2.value(3).toString();
	   params[j].COL = 1;
	   params[j].STYLESHEET = "background-color:rgb(#FFFFFF);";
	   params[j].TOOLTIP = "";
	   j++
	}
	retVal[i] = params;
	
	
   }
   return retVal;
}
/*!xxx Returns experiment records from the amino table */
function sqlGetExperimentByID(id)
{
    var retVal	    = new Array();

    id = parseInt(id.VALUE);
  
    var i = 0;
    retVal[i] =new Object()
    m_query.prepare("SELECT Tc,    	 " +  //0
		    "	    RPM,   	 " +  //1
		    "	    sigma, 	 " +  //2
		    "	    sstart, 	 " +  //3
		    "	    mgml,  	 " +  //4
		    "	    sstar,	 " +  //5
		    "	    vbar,  	 " +  //6
		    "	    rho,   	 " +  //7
		    "	    vis,   	 " +  //8
		    "	    Mp,	   	 " +  //9
		    "	    Mb,	   	 " +  //10
		    "	    calculatedBy," +  //11
		    "	    sample_id,	 " +  //12
		    "	    buffer_id,	 " +  //13
		    "	    results_id	 " +  //14
		    "FROM   experiment   " +
		    "WHERE  id = :id	 ");

    m_query.bindValue(":id", id);
    if (!m_query.exec() || !m_query.next())
    {
	retVal.ERROR = "Failed to execute experiment query!";
	return retVal;
    }
    if (parseInt(m_query.value(13).toString()) >=0)
    {
	BufferID = new Object();
	BufferID.NAME = "id";
	BufferID.VALUE = parseInt(m_query.value(13).toString());
	var SolventArray = loadSolventPage(BufferID);
        for (var j = 0; j < SolventArray.length; j++)
	{
	   retVal[i] = SolventArray[j];
	   i++;
	}
    }
    else
    {
	if (m_query.value(8).toString().length > 0)
	{
		retVal[i] = new Object();
		retVal[i].NAME = "qtBufferViscosity"
		retVal[i].VALUE = m_query.value(8).toString();
		i++
	}
        if (m_query.value(7).toString().length > 0)
        {
	   retVal[i] = new Object();
	   retVal[i].NAME = "qtBufferDensity";
	   retVal[i].VALUE = m_query.value(7).toString();
	   i++;
        }
    }
    if (parseInt(m_query.value(14).toString()) >= 0)
    {
	ResultsID = new Object();
	ResultsID.NAME = "id";
	ResultsID.VALUE = parseInt(m_query.value(14).toString());
	retVal[i] =loadResultsPage(ResultsID);
	i++;
    }
    if (m_query.value(0).toString().length > 0)
    {
	retVal[i] = new Object();
	retVal[i].NAME = "qtSedEquilibriumTemp";
	retVal[i].VALUE = m_query.value(0).toString();
	i++;
    }
    if (m_query.value(1).toString().length > 0)
    {
	retVal[i] = new Object();
	retVal[i].NAME = "qtSedEquilibriumRPM";
	retVal[i].VALUE = m_query.value(1).toString();
	i++;
    }
    if (m_query.value(2).toString().length > 0)
    {
	retVal[i] = new Object();
	retVal[i].NAME = "qtSedEquilibriumSigma";
	retVal[i].VALUE = m_query.value(2).toString();
	i++;
    }
    if (m_query.value(3).toString().length > 0)
    {
	retVal[i] = new Object();
	retVal[i].NAME = "qtSedVelocityTemp";
	retVal[i].VALUE = m_query.value(3).toString();
	i++;
    }
    if (m_query.value(4).toString().length > 0)
    {
	retVal[i] = new Object();
	retVal[i].NAME = "qtSedVelocityConcentration";
	retVal[i].VALUE = parseFloat(m_query.value(4).toString()).toExponential(2);
	i++;
    }
    if (m_query.value(5).toString().length > 0)
    {
	retVal[i] = new Object();
	retVal[i].NAME = "qtSedVelocityCoefficient";
	retVal[i].VALUE = m_query.value(5).toString();
	i++;
    }
    if (m_query.value(6).toString().length > 0)
    {
//	retVal[i] = new Object();
//	retVal[i].NAME = "qtSampleVbar";
//	retVal[i].VALUE = m_query.value(6).toString();
//	i++;
    }
    if (m_query.value(9).toString().length > 0)
    {
	retVal[i] = new Object();
	retVal[i].NAME = "qtSedEquilibriumMR";
	retVal[i].VALUE = m_query.value(9).toString();
	i++;
    }
    if (m_query.value(10).toString().length > 0)
    {
	retVal[i] = new Object();
	retVal[i].NAME = "qtExperimentResultsMb";
	retVal[i].VALUE = m_query.value(10).toString();
	i++;
    }
    if (m_query.value(11).toString() == "qtExactMolecularWeight")
    {
	retVal[i] = new Object();
	retVal[i].NAME = "qtExactMolecularWeight";
	retVal[i].CHECKED = true;
	i++;

	retVal[i] = new Object();
	retVal[i].NAME = "qtFromSigma";
	retVal[i].CHECKED = false;
	i++;

	retVal[i] = new Object();
	retVal[i].NAME = "qtFromComposition";
	retVal[i].CHECKED = false;
	i++;
	// need to call calculate function some how
    }
    else if (m_query.value(11).toString() == "qtFromSigma")
    {
	retVal[i] = new Object();
	retVal[i].NAME = "qtFromSigma";
	retVal[i].CHECKED = true;
	i++;

	retVal[i] = new Object();
	retVal[i].NAME = "qtExactMolecularWeight";
	retVal[i].CHECKED = false;
	i++;

	retVal[i] = new Object();
	retVal[i].NAME = "qtFromComposition";
	retVal[i].CHECKED = false;
	i++;
    }
    else
    {
	retVal[i] = new Object();
	retVal[i].NAME = "qtFromComposition";
	retVal[i].CHECKED = true;
	i++;

	retVal[i] = new Object();
	retVal[i].NAME = "qtFromSigma";
	retVal[i].CHECKED = false;
	i++;

	retVal[i] = new Object();
	retVal[i].NAME = "qtExactMolecularWeight";
	retVal[i].CHECKED = false;
	i++;
    }

    if (parseInt(m_query.value(12).toString()) >= 0)
    {
	Sampleid = new Object()
	Sampleid.NAME = "id";
	Sampleid.VALUE = parseInt(m_query.value(12).toString());
//	retVal[i] = loadSamplePage(Sampleid);
        
	var SampleArray = sqlGetSampleByID(Sampleid);
        for (var j = 0; j < SampleArray.length; j++)
        {
	  retVal[i] = SampleArray[j];
	  i++;
        }
    }

    return retVal;
}
/*! xxx  Loads Results page */
function loadResultsPage(id)
{
     var retVal		= new Array();
     var i = 0;
     id = parseInt(id.VALUE);
     if (id == -1)
     {
       return;
     }

     m_query2.prepare(  "SELECT R," + // 0
			"Ro, " +	// 1
			"Rp, " +	// 2
			"Fo," +		// 3
			"Fp," +		// 4
			"FFo, " +	// 5
			"FFp, " +	// 6
			"So,  " +	// 7
			"Sp,  " +	// 8
			"Do,  " +	// 9
			"Dp,  " +	// 10
			"Delta1v," +	// 11
			"Delta1t," +	// 12
			"abv,    " +	// 13
			"abt,    " +	// 14
			"av,     " +	// 15
			"at,     " +	// 16
			"bv,	 " +	// 17
			"bt,	 " +	// 18
			"method, " +	// 19
			"options," +	// 20
			"options_value, " + // 21
			"shape,		" + // 22
			"sample_id,  " + // 23
			"buffer_id,	" + // 24
			"experiment_id  " + // 25
			"FROM results " +
			"WHERE id = :id");
     m_query2.bindValue(":id", id);

     if (!m_query2.exec())
     {
	retVal.ERROR = " Failed to excute results query!";
	return retVal;
     }
   m_query2.next()
   
     if (m_query2.value(0).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtRStokes";
	retVal[i].VALUE = m_query2.value(0).toString();
	i++;
     }

     if (m_query2.value(1).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtVbarRo";
	retVal[i].VALUE = m_query2.value(1).toString();
	i++;
     }
     if (m_query2.value(2).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtTellerRp";
	retVal[i].VALUE = m_query2.value(2).toString();
	i++;
     }
     if (m_query2.value(3).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtVbarFo";
	retVal[i].VALUE = m_query2.value(3).toString();
	i++;
     }
     if (m_query2.value(4).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtTellerFp";
	retVal[i].VALUE = m_query2.value(4).toString();
	i++;
     }
     if (m_query2.value(5).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtVbarffo";
	retVal[i].VALUE = m_query2.value(5).toString();
	i++;
     }
     if (m_query2.value(6).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtTellerffp";
	retVal[i].VALUE = m_query2.value(6).toString();
	i++;
     }
     if (m_query2.value(7).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtVbarSmax";
	retVal[i].VALUE = m_query2.value(7).toString();
	i++;
     }
     if (m_query2.value(8).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtTellerSmax";
	retVal[i].VALUE = m_query2.value(8).toString();
	i++;
     }
     if (m_query2.value(9).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtVbarDmax";
	retVal[i].VALUE = m_query2.value(9).toString();
	i++;
     }
     if (m_query2.value(10).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtTellerDmax";
	retVal[i].VALUE = m_query2.value(10).toString();
	i++;
     }
     if (m_query2.value(11).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtVbarDelta1max";
	retVal[i].VALUE = m_query2.value(11).toString();
	i++;
     }
     if (m_query2.value(12).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtTellerDelta1max";
	retVal[i].VALUE = m_query2.value(12).toString();
	i++;
     }
     if (m_query2.value(13).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtVbarmaxab";
	retVal[i].VALUE = m_query2.value(13).toString();
	i++;
     }
     if (m_query2.value(14).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtTellermaxab";
	retVal[i].VALUE = m_query2.value(14).toString();
	i++;
     }
     if (m_query2.value(15).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtVbar2amax";
	retVal[i].VALUE = parseFloat(m_query2.value(15).toString()) * 2 ;
	i++;
     }
     if (m_query2.value(16).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtTeller2amax";
	retVal[i].VALUE = m_query2.value(16).toString() * 2;
	i++;
     }
     if (m_query2.value(17).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtVbar2bmin";
	retVal[i].VALUE = m_query2.value(17).toString() * 2;
	i++;
     }
     if (m_query2.value(18).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtTeller2bmin";
	retVal[i].VALUE = m_query2.value(18).toString() * 2;
	i++;
     }
     if (m_query2.value(19).toString() == "qtVbarMethod")
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtVbarMethod";
	retVal[i].CHECKED = true;
	i++;
     }
     else
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtTellerMethod";
	retVal[i].CHECKED = true;
	i++;
     }
     if (m_query2.value(20).toString() == "qtHydrateNothing")
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtHydrateNothing";
	retVal[i].CHECKED = true;
	i++;
     }
     else if (m_query2.value(20).toString() == "qtHydrateProlate")
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtHydrateProlate";
	retVal[i].CHECKED = true;
	i++;
     }
     else if (m_query2.value(20).toString() == "qtHydrateOblate")
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtHydrateOblate";
	retVal[i].CHECKED = true;
	i++;
     }
     else if (m_query2.value(20).toString() == "qtHydrateComputed")
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtHydrateComputed";
	retVal[i].CHECKED = true;
	i++;
     }
     else if (m_query2.value(20).toString() == "qtHydrateDelta1")
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtHydrateDelta1";
	retVal[i].CHECKED = true;
	i++;
     }

     if (m_query2.value(21).toString().length > 0)
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtHydrateRatio";
	retVal[i].VALUE = m_query2.value(21).toString();
	i++;
     }
     if (m_query2.value(22).toString() == "qtShapeCylinder")
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtShapeCylinder";
	retVal[i].CHECKED = true;
	i++;
     }
     else if (m_query2.value(22).toString() == "qtShapeProlate")
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtShapeProlate";
	retVal[i].CHECKED = true;
	i++;
     }
     else
     {
	retVal[i] = new Object();
	retVal[i].NAME = "qtShapeOblate";
	retVal[i].CHECKED = true;
	i++;
     }
   
   return retVal;
}

/*! xxx  Returns sample records from the amino table */
function sqlGetSampleByID(id)
{
    var retVal      = new Array();

    id = parseInt(id.VALUE);
    var i=0;

    m_query.prepare("SELECT vbarDirectEntry, "  +
		    "       vbarStandard, "    +
		    "       calcVBar, "         +
		    "       calcMR, "           +
		    "       calcHydration, "    +
		    "       useDenaturant,"     +
		    "       sk, "               +
		    "       pkaamino, "         +
		    "       pkacarboxl, "       +
		    "       pH, "               +
		    "       nMer, "             +
		    "       aminoSequence, "    +
		    "       aminoSequenceCodeSize, " +
		    "       VBarStandardT "     +
		    "FROM   sample " +
		    "WHERE  id = :id");
    m_query.bindValue(":id", id);

    // We should change this to lastError().text() if its accessible
    if (!m_query.exec() || !m_query.next())
    {
        retVal.ERROR = "Failed to execute sample query!";
        return retVal;
    }

    if (m_query.value(1).toString().length > 0) //vbarStandard
    {
        retVal[i] = new Object();
	retVal[i].NAME = "qtSampleVbar";
	retVal[i].VALUE = m_query.value(1).toString();
	i++;
    }
    if (m_query.value(0).toString().length > 0) // vbarDirectEntry
    {
/*        retVal[i] = new Object();
	retVal[i].NAME = "qtValue";
	retVal[i].VALUE = m_query.value(0).toString();
	i++;
        retVal[i] = new Object();
	retVal[i].NAME = "qtVbarDirectEntry";
	retVal[i].CHECKED = "true";
	i++;
        retVal[i] = new Object();
	retVal[i].NAME = "qtVbarStandard";
	retVal[i].CHECKED = "false";
	i++;
        retVal[i] = new Object();
	retVal[i].NAME = "qtVbarCompute";
	retVal[i].CHECKED = "false";
	i++;
  */    retVal[i] = new Object();
	retVal[i].NAME = "qtSampleVbarLabel";
	retVal[i].VALUE = "No Temp Correction";
	i++;
        retVal[i] = new Object();
	retVal[i].NAME = "qtTemperature";
	retVal[i].VALUE = m_query.value(12).toString();
	i++;
        retVal[i] = new Object();
	retVal[i].NAME = "qtValue";
	retVal[i].VALUE = m_query.value(0).toString();
	i++;
    }
    
    if (parseFloat(m_query.value(2)) != 0) //calcVBar
    {
        retVal[i] = new Object();
	retVal[i].NAME = "qtSampleVbar";
//	retVal[i].NAME = "qtCalculatedVBar";
	retVal[i].VALUE = m_query.value(2).toString();
	i++;
/*
        retVal[i] = new Object();
	retVal[i].NAME = "qtCalculateVbar";
	retVal[i].CHECKED = "true";
	i++;
        retVal[i] = new Object();
	retVal[i].NAME = "qtVbarCompute";
	retVal[i].CHECKED = "true";
	i++;
        retVal[i] = new Object();
	retVal[i].NAME = "qtVbarDirectEntry";
	retVal[i].CHECKED = "false";
	i++;
        retVal[i] = new Object();
	retVal[i].NAME = "qtVbarStandard";
	retVal[i].CHECKED = "false";
	i++;*/
    }
    else
    {
        retVal[i] = new Object();
	retVal[i].NAME = "qtCalculateVbar";
	retVal[i].CHECKED = false;
	i++;
    }
    
    if (m_query.value(3).toString().length > 0) //calcMr
    {
/*        retVal[i] = new Object();
	retVal[i].NAME = "qtVbarCompute";
	retVal[i].CHECKED = "true";
	i++;
        retVal[i] = new Object();
	retVal[i].NAME = "qtVbarDirectEntry";
	retVal[i].CHECKED = "false";
	i++;
        retVal[i] = new Object();
	retVal[i].NAME = "qtVbarStandard";
	retVal[i].CHECKED = "false";
	i++;
*/
        retVal[i] = new Object();
	retVal[i].NAME = "qtCalculatedMR";
	retVal[i].VALUE = m_query.value(3).toString();
	i++;

        retVal[i] = new Object();
	retVal[i].NAME = "qtCalculateMr";
	retVal[i].CHECKED = true;
	i++;

        retVal[i] = new Object();
	retVal[i].NAME = "qtSedEquilibriumMR";
	retVal[i].VALUE = m_query.value(3).toString();
	i++;
    }
    else
    {
        retVal[i] = new Object();
	retVal[i].NAME = "qtCalculateMr";
	retVal[i].CHECKED = true;
	i++;
    }
    
    if (parseFloat(m_query.value(4)) != 0) //calcHydration
    {
        retVal[i] = new Object();
	retVal[i].NAME = "qtCalculatedHydration";
	retVal[i].VALUE = m_query.value(4).toString();
	i++;

        retVal[i] = new Object();
	retVal[i].NAME = "qtCalculateHydration";
	retVal[i].CHECKED = true;
	i++;

        retVal[i] = new Object();
	retVal[i].NAME = "qtHydrateComputed";
	retVal[i].CHECKED = true;
	i++;

        retVal[i] = new Object();
	retVal[i].NAME = "qtHydrateRatio";
	retVal[i].VALUE = m_query.value(4).toString();
	i++;
    }
    else
    {
        retVal[i] = new Object();
	retVal[i].NAME = "qtHydrateNothing";
	retVal[i].CHECKED = true;
	i++;

        retVal[i] = new Object();
	retVal[i].NAME = "qtCalculateHydration";
	retVal[i].CHECKED = false;
	i++;
    }

    if (m_query.value(7).toString().length > 0) //amino pka
    {
        retVal[i] = new Object();
	retVal[i].NAME = "qtAmino";
	retVal[i].VALUE = m_query.value(7).toString();
	i++;
    }
    if (m_query.value(8).toString().length > 0) //carboxyl pka
    {
        retVal[i] = new Object();
	retVal[i].NAME = "qtCarboxyl";
	retVal[i].VALUE = m_query.value(8).toString();
	i++;
    }
    // sk, pkacarboxl,pkaamino are constants?
    if (m_query.value(10).toString().length > 0 && 
	parseInt(m_query.value(10)) > 0 ) // nMer
    {
        retVal[i] = new Object();
	retVal[i].NAME = "qtMakeOligomer";
	retVal[i].CHECKED = true;
	i++;
        retVal[i] = new Object();
	retVal[i].NAME = "qtOligomerN";
	retVal[i].VALUE = m_query.value(10).toString();
	i++;
    }
    else
    {
        retVal[i] = new Object();
	retVal[i].NAME = "qtMakeOligomer";
	retVal[i].CHECKED = false;
	i++;
        retVal[i] = new Object();
	retVal[i].NAME = "qtOligomerN";
	retVal[i].VALUE = "2";
	i++;
    }
    
    if (m_query.value(11).toString().length > 0)
    {
	if (m_query.value(0).toString() == m_query.value(2).toString())
	{
          retVal[i] = new Object();
	  retVal[i].NAME = "qtCalculateVbar";
	  retVal[i].CHECKED = false;
	  i++;
          retVal[i] = new Object();
	  retVal[i].NAME = "qtVbarCompute";
	  retVal[i].CHECKED = false;
	  i++;

          if (m_query.value(1).toString().length > 0) //vbarStandard
	  {
  	    retVal[i] = new Object();
	    retVal[i].NAME = "qtVbarStandard";
	    retVal[i].CHECKED = true;
	    i++;
            retVal[i] = new Object();
	    retVal[i].NAME = "qtVbarDirectEntry";
	    retVal[i].CHECKED = false;
	    i++;
	  }
	  else
	  {
  	    retVal[i] = new Object();
	    retVal[i].NAME = "qtVbarStandard";
	    retVal[i].CHECKED = false;
	    i++;
            retVal[i] = new Object();
	    retVal[i].NAME = "qtVbarDirectEntry";
	    retVal[i].CHECKED = true;
	    i++;
	  }
	}
	else
	{
          retVal[i] = new Object();
	  retVal[i].NAME = "qtCalculateVbar";
	  retVal[i].CHECKED = true;
	  i++;
          retVal[i] = new Object();
	  retVal[i].NAME = "qtVbarCompute";
	  retVal[i].CHECKED = true;
	  i++;
  	  retVal[i] = new Object();
	  retVal[i].NAME = "qtVbarStandard";
	  retVal[i].CHECKED = false;
	  i++;
          retVal[i] = new Object();
	  retVal[i].NAME = "qtVbarDirectEntry";
	  retVal[i].CHECKED = false;
	  i++;
	}

        retVal[i] = new Object();
	retVal[i].NAME="aminoSequence";
        retVal[i].VALUE = sqlGetAminoSequenceBySampleID(id);
	i++;
	// not implemented
	// aew - commenting out object - it causes problems for web if null
	//       un-comment when implemented
        // retVal[i] = new Object();
	// retVal[i].NAME="CONJUGATES";
        // retVal[i].VALUE = "";
	// i++;
    }
    else
    {
        if (m_query.value(1).toString().length > 0) //vbarStandard
	{
  	  retVal[i] = new Object();
	  retVal[i].NAME = "qtVbarStandard";
	  retVal[i].CHECKED = true;
	  i++;
          retVal[i] = new Object();
	  retVal[i].NAME = "qtVbarDirectEntry";
	  retVal[i].CHECKED = "false";
	  i++;
          retVal[i] = new Object();
	  retVal[i].NAME = "qtCalculateVbar";
	  retVal[i].CHECKED = false;
	  i++;
          retVal[i] = new Object();
	  retVal[i].NAME = "qtVbarCompute";
	  retVal[i].CHECKED = false;
	  i++;
	}
	else
	{
  	  retVal[i] = new Object();
	  retVal[i].NAME = "qtVbarStandard";
	  retVal[i].CHECKED = false;
	  i++;
          retVal[i] = new Object();
	  retVal[i].NAME = "qtVbarDirectEntry";
	  retVal[i].CHECKED = true;
	  i++;
          retVal[i] = new Object();
	  retVal[i].NAME = "qtCalculateVbar";
	  retVal[i].CHECKED = false;
	  i++;
          retVal[i] = new Object();
	  retVal[i].NAME = "qtVbarCompute";
	  retVal[i].CHECKED = false;
	  i++;
	}
    }

    return retVal;
}

/*! Returns an amino acid sequence for a sample*/
function sqlGetAminoSequenceBySampleID(id)
{
    var retVal = "";

    id = parseInt(id);

    m_query2.prepare("SELECT amino.nameone, "  +
		    "       aminoComposition.count "    +
		    "FROM   amino,aminoComposition " +
		    "WHERE  amino.id=aminoComposition.aminoId AND aminoComposition.sampleId = :id");
    m_query2.bindValue(":id", id);

    // We should change this to lastError().text() if its accessible
    if (!m_query2.exec())
    {
        retVal.ERROR = "Failed to execute amino sequence query!";
        return retVal;
    }

    while (m_query2.next())
    {
	var i = 0;
	for (i = 0; i < parseInt(m_query2.value(1)); i++)
	{
	    retVal += m_query2.value(0).toString();
	}
    }
    return retVal;
}

/*! Returns Density/Viscosity records from rhoseries and viscseries tables */
function sqlFetchBufferComponents()
{
    var retVal = new Object();
    var components = new Array();
    
    ///*
    //** Query all components in *both* viscseries and rhoseries
    //** UNION
    //** -- Query all rhoseries components which are NOT in viscseries.
    //** UNION
    //** -- Query all viscseries components which are NOT in rhoseries
    //*/
    //m_query.prepare("SELECT rhoseries.name, rhoseries.id AS rhoID, " +
		//    "viscseries.id AS viscID " +
		//    "FROM rhoseries, viscseries " +
		//    "WHERE (rhoseries.active > 0 AND viscseries.active > 0) " +
		//    "AND rhoseries.name = viscseries.name " +
		//    "" +
		//    "UNION " +
		//    "" +
		//    "SELECT rhoseries.name, rhoseries.id, -1 " +
		//    "FROM rhoseries " +
		//    "WHERE rhoseries.active > 0 AND rhoseries.name " +
		//    "NOT IN (SELECT name FROM viscseries WHERE active > 0) " +
		//    "" +
		//    "UNION " +
		//    "" +
		//    "SELECT viscseries.name, -1, id FROM viscseries " +
		//    "WHERE viscseries.active > 0 AND viscseries.name " +
		//    "NOT IN (SELECT name FROM rhoseries where active > 0) " +
		//    "" +
		//    "ORDER BY name COLLATE NOCASE ASC");
    
    /* Query from BufferComponents table */
    m_query.prepare("SELECT id, name, componentRhoId, componentViscId, " +
		    "componentDensity, componentMr " +
		    "FROM BufferComponent " +
		    "WHERE active > 0 ORDER BY name")
    
    // We should change this to lastError().text() if its accessible
    if (!m_query.exec()) 
    {
	//print ("Failed to exec BufferComponents Query");
	retVal.ERROR = "Failed to execute query!";
	return retVal;
    }
    
    for (var i = 0; m_query.next(); i++)
    {
	components[i] = new Object();
	components[i].NAME  = "Components";
	components[i].VALUE = m_query.value(1).toString();
	components[i].COMPONENTID = m_query.value(0).toString();
	
	/* Add tooltip and stylsheet info if rhoseries id is null */
	//! Compare length because sql_query::value converts the variant to a 
	//! string, and null variant = empty string
	if (m_query.value(2).length == 0)//.isNull())
	{
	    components[i].TOOLTIP = 
		" \'" + components[i].VALUE + "\' " + 
		"does not appear in the Density list.";
	    components[i].STYLESHEET = "background-color:#EDAE6F"
	    components[i].DENSITYID = -1;
	}
	else
	    components[i].DENSITYID = m_query.value(2).toString();
	    
	/* Add tooltip and stylsheet info if viscseries id is null */
	if (m_query.value(3).length == 0)//.isNull())
	{
	    components[i].TOOLTIP = 
		" \'" + components[i].VALUE + "\' " + 
		"does not appear in the Viscosity list.";
	    components[i].STYLESHEET = "background-color:#EDAE6F"
	    components[i].VISCOSITYID = -1;
	}
	else
	    components[i].VISCOSITYID = m_query.value(3).toString();
	
	/* Determine which units are valid for this record */
	var units = new Array();
	units[0] = new Object();
	units[0].NAME = "molar";
	units[0].VALUE = "molar";

	if (m_query.value(4).length > 0 && //.isNull() &&
	    m_query.value(5).length > 0)//.isNull())
	{
	    units[1] = new Object();
	    units[1].NAME = "molar";
	    units[1].VALUE = "molar";
	    units[2] = new Object();
	    units[2].NAME = "mg/ml";
	    units[2].VALUE = "mg/ml";
	    units[3] = new Object();
	    units[3].NAME = "g/ml";
	    units[3].VALUE = "g/ml";
	    units[4] = new Object();
	    units[4].NAME = "% by wt";
	    units[4].VALUE = "% by wt";
	    units[5] = new Object();
	    units[5].NAME = "% by vol";
	    units[5].VALUE = "% by vol";
	}

	components[i].UNITS = units;
    }
    
    retVal.COMPONENTS = components;
    return retVal;
}


//function calcCharge 

function sqlFetchComboBoxes ()
{
    retVal = new Object();
    retVal.SAMPLE = new Array();
    retVal.BUFFER = new Array();
    retVal.EXP    = new Array();
    retVal.RESULTS= new Array();

    retVal.SAMPLE[0] = new Object();
    retVal.SAMPLE[0].NAME = "qtSampleName";
    retVal.SAMPLE[0].ID = -1;
    retVal.SAMPLE[0].VALUE = "New Sample";
    retVal.SAMPLE[0].DATE = "";

    retVal.BUFFER[0] = new Object();
    retVal.BUFFER[0].NAME = "qtBufferName";
    retVal.BUFFER[0].ID = -1;
    retVal.BUFFER[0].VALUE = "New Solvent";
    retVal.BUFFER[0].DATE = "";
    
    retVal.EXP[0] = new Object();
    retVal.EXP[0].NAME = "qtExperimentName";
    retVal.EXP[0].ID = -1;
    retVal.EXP[0].VALUE = "New Experiment";
    retVal.EXP[0].DATE = "";

    retVal.RESULTS[0] = new Object();
    retVal.RESULTS[0].NAME = "qtResultsName";
    retVal.RESULTS[0].ID = -1;
    retVal.RESULTS[0].VALUE = "New Results";
    retVal.RESULTS[0].DATE = "";
    
    
    m_query.prepare("SELECT Sample.id, Sample.name, Sample.modified " +
		    "FROM Sample ORDER BY Sample.modified ");
    if (!m_query.exec())
    {
	// Need to improve upon this later...
	//print("Failed to exec sqlFetch");
	retVal.SAMPLE.ERROR = "Failed to execute query!";
    }
    
    for (var i = 1; m_query.next(); i++)
    {
	record = new Object();
	record.NAME = "qtSampleName";
	record.ID   = m_query.value(0).toString();
	record.VALUE = m_query.value(1).toString();
	var date = m_query.value(2).toString();
	var dateParts = date.split(" ")
	var dateParts = dateParts[0].split("-");
	record.DATE = dateParts[1] + "/" + dateParts[2] + "/" + dateParts[0].substr(2,2);
	retVal.SAMPLE[i] = record;
    }
    
    m_query.prepare("SELECT Buffer.id, Buffer.name, Buffer.modified " +
		    "FROM Buffer ORDER BY Buffer.modified ");
    if (!m_query.exec())
    {
	// Need to improve upon this later...
	//print("Failed to exec sqlFetch");
	retVal.BUFFER.ERROR = "Failed to execute query!";
    }

    for (var i = 1; m_query.next(); i++)
    {
	record = new Object();
	record.NAME = "qtBufferName";
	record.ID   = m_query.value(0).toString();
	record.VALUE = m_query.value(1).toString();
	var date = m_query.value(2).toString();
	var dateParts = date.split(" ")
	var dateParts = dateParts[0].split("-");
	record.DATE = dateParts[1] + "/" + dateParts[2] + "/" + dateParts[0].substr(2,2);
	retVal.BUFFER[i] = record;
    }
 
    m_query.prepare("SELECT experiment.id, experiment.name, " +
		    "experiment.modified FROM experiment ORDER BY experiment.modified ");
    if (!m_query.exec())
    {
	// Need to improve upon this later...
	//print("Failed to exec sqlFetch");
	retVal.EXP.ERROR = "Failed to execute query!";
	return retVal;
    }

    for (var i = 1; m_query.next(); i++)
    {
	record = new Object();
	record.NAME = "qtExperimentName";
	record.ID   = m_query.value(0).toString();
	record.VALUE = m_query.value(1).toString();
	var date = m_query.value(2).toString();
	var dateParts = date.split(" ")
	var dateParts = dateParts[0].split("-");
	record.DATE = dateParts[1] + "/" + dateParts[2] + "/" + dateParts[0].substr(2,2);
	//record.DATE = m_query.value(2).toDateTime().toString("MM/dd/yy");
	retVal.EXP[i] = record;
    }

    m_query.prepare("SELECT results.id, results.name, " +
		    "results.modified FROM results ORDER BY results.modified ");
    if (!m_query.exec())
    {
	// Need to improve upon this later...
	//print("Failed to exec sqlFetch");
	retVal.RESULTS.ERROR = "Failed to execute query!";
	return retVal;
    }

    for (var i = 1; m_query.next(); i++)
    {
	record = new Object();
	record.NAME = "qtResultsName";
	record.ID   = m_query.value(0).toString();
	record.VALUE = m_query.value(1).toString();
	var date = m_query.value(2).toString();
	var dateParts = date.split(" ")
	var dateParts = dateParts[0].split("-");
	record.DATE = dateParts[1] + "/" + dateParts[2] + "/" + dateParts[0].substr(2,2);
	retVal.RESULTS[i] = record;
    }
    
    return retVal;
}


/*! "Private" function. Converts concentration into Molar units
 *! \param componentMr Mr value of this buffercomponent
 *! \param componentDensity Density value of this buffercomponent
 *! \param concentration Concentration in 'units'
 *! \param units The units of concentration */
function _convertToMolar(componentName, componentMr, componentDensity, 
			 concentration, units)
{
    var retVal = new Object();
    retVal.CONCENTRATION = 0;
    
    /* No conversion necessary */
    if (units == "molar")
    {
	retVal.CONCENTRATION = concentration;
	return retVal;
    }
    
    /* 
    ** Units are not 'molar'.
    ** Verify that record has necessary data to convert to molar
    */
    if (isNaN(componentMr) || isNaN(componentDensity))
    {
	retVal.WARNING = 
	    "\'" + componentName + "\' lacks sufficient " +
	    "information to convert \'" + units + "\' to molar";
	return retVal;
    }
    
    /* Convert 'molal' to 'molar' */
    if (units == "molal")
    {
	retVal.ERROR = "molal to molar calculation not supported";
    }
    
    /* Convert 'mg/ml' to 'molar' */
    else if (units == "mg/ml")
    {
	//concentration("g/L") / MR ("g/mole") = molarity
	//retVal.ERROR = "mg/ml to molar calculation not supported";
	retVal.CONCENTRATION = concentration / (componentMr * 1000);
    }
    
    /* Convert 'g/ml' to 'molar' */
    else if (units == "g/ml")
    {
	//retVal.ERROR = "g/ml to molar calculation not supported";
	retVal.CONCENTRATION = 
	    (concentration * 0.001) / (componentMr * 1000);
    }
    
    /* Convert '% by wt' to 'molar' */
    else if (units == "% by wt")
    {
	retVal.ERROR = "% by wt to molar calculation not supported";
    }
    
    /* Convert '% by vol' to 'molar' */
    else if (units == "% by vol")
    {
	retVal.ERROR = "% by vol to molar calculation not supported";
    }
    
    /* Unknown case */
    else
    {
	retVal.ERROR = units + " to molar calculation not supported";
    }
    
    return retVal;
}
    

/*
** Calculates Fp for the Results page. teh
*/
function _calculateFp(Mr)
{
    print(">>Function: _calculateFp(Mr) ");
    //print("\tMr=" + Mr);

    Mr = parseFloat(Mr);

    var Rp = _calculateRp(Mr);

    return 6 * PI * Density_20_Water * Rp;
}

/*
** Calculates Fo for the Results page. teh
*/
function _calculateFo(Mr, Vbar)
{
    print(">>Function: _calculateFo(Mr, Vbar) ");
    //print("\tMr=" + Mr);
    //print("\tVbar=" + Vbar);

    Mr   = parseFloat(Mr);
    Vbar = parseFloat(Vbar);

    var Ro = _calculateRo(Mr,Vbar);

    return 6 * PI * Density_20_Water * Ro;
}



function _calculateFFp(Sp, S20W)
{
    print(">>Function: _calculateFFp(Sp, S20W) ");


    Sp     = parseFloat(Sp);
    S20W   = parseFloat(S20W);

    //print("\tSp=" + Sp);
    //print("\tS20W=" + S20W);
    return (Sp) / (S20W);
}

function _calculateFFo(So, S20W)
{
    print(">>Function: _calculateFFo(So, S20W) ");

    So = parseFloat(So);
    S20W = parseFloat(S20W);

    //print("\tSo=" + So)
    //print("\tS20W=" + S20W);


    var result = So/S20W;
    //print("FFo=" + result);
    return result;
}


/*function _calculateFFp(Mr, Vbar, S20W)
{
    print(">>Function: _calculateFFp(Mr, Vbar, S20W) ");
    print("\tMr=" + Mr);
    print("\tVbar=" + Vbar);
    print("\tS20W=" + S20W);

    Mr     = parseFloat(Mr);
    Vbar   = parseFloat(Vbar);
    S20W   = parseFloat(S20W);
    var Fp = _calculateFp(Mr)

    return (Mr * (1.0 - Vbar * Density_20_Water) / (S20W * Na * Fp));
}

function _calculateFFo(Mr, Vbar, S20W)
{
    print(">>Function: _calculateFFo(Mr, Vbar, S20W) ");
    print("\tMr=" + Mr);
    print("\tVbar=" + Vbar);
    print("\tS20W=" + S20W);

    Mr     = parseFloat(Mr.VALUE);
    Vbar   = parseFloat(Vbar.VALUE);
    S20W   = parseFloat(S20W.VALUE);

    var Fo = _calculateFo(Mr, Vbar)

    print("_calculateFFo");
    print("\tMr=" + Mr);
    print("\tVbar=" + Vbar);
    print("\tS20W=" + S20W);
    print("\tFo=" + Fo);

    var result = (parseFloat(Mr) * (1.0 - parseFloat(Vbar) * Density_20_Water) / (parseFloat(S20W) * Na * parseFloat(Fo)));
    print("FFo=" + result);
    return result;
}*/

function _calculateSp(Mr, Vbar, Fp, Mb)
{
    print(">>Function: _calculateSp(Mr, Vbar, Fp, Mb) ");
    //print("\tMr=" + Mr);
    //print("\tVbar=" + Vbar);
    //print("\tFp=" + Fp);
    //print("\tMb=" + Mb);

    Mr     = parseFloat(Mr);
    Vbar   = parseFloat(Vbar);
    Fp     = parseFloat(Fp);
    Mb     = parseFloat(Mb);

    if (Mb != 0.0 && Fp != 0.0)
      return (Mr * (1.0 - Vbar * Density_20_Water) / (Na * Fp));
    else
      return 0.0;
}

// Dmax (vbar)
function _calculateDo(Fo, Tc)
{
    print(">>Function: _calculateDo(Fo, Tc) ");
    //print("\tFo=" + Fo);
    //print("\tTc=" + Tc);

    Fo     = parseFloat(Fo);
    Tc     = parseFloat(Tc);
   
    if (Fo == 0.0)
	return 0.0;

    return (R * (Tc + 273.15)) / (Na * Fo);
}

// Smax (vbar)
function _calculateSo(Mr, Vbar, Fo, Do, Mb, Sigma, RPM)
{
    print(">>Function: _calculateSo(Mr, Vbar, Fo, Do, Mb, Sigma, RPM)");
    //print("\tMr=" + Mr);
    //print("\tVbar=" + Vbar);
    //print("\tFo=" + Fo);
    //print("\tDo=" + Do);
    //print("\tMb=" + Mb);
    //print("\tSigma=" + Sigma);
    //print("\tRPM=" + RPM);

    Mr     = parseFloat(Mr);
    Vbar   = parseFloat(Vbar);
    Fo     = parseFloat(Fo);
    Do     = parseFloat(Do);
    Mb     = parseFloat(Mb);
    Sigma  = parseFloat(Sigma);
    RPM    = parseFloat(RPM);

    if (isNaN(Mb))
	return 0.0;

    if (Mb != 0.0 && Fo != 0.0)
      return (Mr * (1.0 - Vbar * Density_20_Water) / (Na * Fo));
    else
      {
	  var om = Math.pow((2 * PI * RPM/60),2);
	  return Do * Sigma / om;
      }
}

function _calculateDp(Mr, Vbar, Fp, T)
{
    print(">>Function: _calculateDp(Mr, Vbar, Fp, T)");
    //print("\tMr=" + Mr);
    //print("\tVbar=" + Vbar);
    //print("\tFp=" + Fp);
    //print("\tT=" + T);

    Mr     = parseFloat(Mr);
    Vbar   = parseFloat(Vbar);
    Fp     = parseFloat(Fp);
    T      = parseFloat(T);

    if (Fp == 0.0)
       return 0.0;
    else
       return (R * (T + 273.15)) / (Na * Fp);
}

function _computeDelta1_p(Mr, Vbar, S20W, Mb, Fp)
{
    print(">>Function: _computeDelta1_p(Mr, Vbar, S20W, Mb, Fp)");
    //print("\tMr=" + Mr);
    //print("\tVbar=" + Vbar);
    //print("\tS20W=" + S20W);
    //print("\tMb=" + Mb);
    //print("\tFp=" + Fp);

    Mr     = parseFloat(Mr);
    Vbar   = parseFloat(Vbar);
    S20W   = parseFloat(S20W);
    Mb     = parseFloat(Mb);
    Fp     = parseFloat(Fp);

    if (!isNaN(S20W) && (S20W != 0.0 && Mb != 0/0))
    {
	var f20w = Mr * (1.0 - Vbar * Density_20_Water)/(Na * S20W);
	return (Math.pow((f20w/Fp),3) - 1.0) * 0.7654 / Density_20_Water;
    }
    else
    {
	return 0.0;
    }
}

function _computeDelta1_o(Mr, Vbar, S20W, Mb, Fp)
{
    print(">>Function: _computeDelta1_o(Mr, Vbar, S20W, Mb, Fp)");
    //print("\tMr=" + Mr);
    //print("\tVbar=" + Vbar);
    //print("\tS20W=" + S20W);
    //print("\tMb=" + Mb);
    //print("\tFp=" + Fp);

    Mr     = parseFloat(Mr);
    Vbar   = parseFloat(Vbar);
    S20W   = parseFloat(S20W);
    Mb     = parseFloat(Mb);
    Fp     = parseFloat(Fp);

    // teh NOTE: there may be two different values of vbar here. Check later
    if (!isNaN(S20W) && (S20W != 0.0 && Mb != 0/0))
    {
	var f20w = Mr * (1.0 - Vbar * Density_20_Water)/(Na * S20W);
	return (Math.pow((f20w/Fp),3) - 1.0) * Vbar / Density_20_Water;
    }
    else
    {
	return 0.0;
    }
}


function _computeExactAOverB_o(S20W,Ratio,Mr,Vbar,Fo,A,B)
{
    print(">>Function: _computeExactAOverB_o(S20W,Ratio,Mr,Vbar,Fo,A,B)");
    //print("\tS20W=" + S20W);
    //print("\tRatio=" + Ratio);
    //print("\tMr=" + Mr);
    //print("\tVbar=" + Vbar);
    //print("\tFo=" + Fo);
    //print("\tA.VALUE=" + A.VALUE);
    //print("\tB.VALUE=" + B.VALUE);

    S20W = parseFloat(S20W);
    Ratio = parseFloat(Ratio);
    Mr = parseFloat(Mr);
    Vbar = parseFloat(Vbar);
    Fo = parseFloat(Fo);

    var ansp = 0.0;
    var anso = 0.0;
    var X;
    var f20w;
    var tempnum;
    var tempden;
    var delta;

   // print("_computeExactAOverB_o");

    if (!isNaN(S20W) && S20W != 0.0)
    {
	f20w = Mr * (1.0 - Vbar * Rho_Water) / (Na * S20W);
	//print("\tf20w=" + f20w);
	tempnum = (f20w / Fo);
	//print("\ttempnum=" + tempnum);

	delta = Ratio;
	//print("\tdelta=" + tempnum);

	tempden = Math.pow((1.0 + delta / (Rho_Water * Vbar)),(1.0 / 3.0));
	//print("\ttempden=" + tempden);
	X = (tempnum / tempden) - 1.0;
	//print("\tX=" + X);

	if (X > 0.0) 
	{
            ansp = 1 + 2.939184 * Math.pow(X,0.5) + 8.072775 * X + 8.55676 * Math.pow(X,2.0) - 0.505951 * Math.pow(X,3.0) + 0.0361508 * Math.pow(X,4.0);
            anso = 1 + 3.009376 * Math.pow(X,0.5) + 8.413897 * X + 13.34978 * Math.pow(X,2.0) + 3.18903 * Math.pow(X,3.0) + 0.106307 * Math.pow(X,4.0);
        //conaoverberroro = False
	}
	else
	{
        //'MsgBox ("Maximum s is lower than S20w, no a/b possible")
        //conaoverberroro = True
	}
    }
    //print("\tA=" + ansp);
    //print("\tB=" + anso);
    //print("\tA/B=" + parseFloat(ansp/anso));

    A.VALUE = ansp;
    B.VALUE = anso;


}

function _computeExactAOverB_p(S20W,Delta1,Mr,Vbar,Fp,A,B)
{
 //   print(">>Function: _computeExactAOverB_p(S20W,Delta1,Mr,Vbar,Fp,A,B)");
 //   print("\tS20W=" + S20W);
 //   print("\tDelta1=" + Delta1);
 //   print("\tMr=" + Mr);
 //   print("\tVbar=" + Vbar);
 //   print("\tFp=" + Fp);
 ///   print("\tA.VALUE=" + A.VALUE);
 //   print("\tB.VALUE=" + B.VALUE);

    S20W = parseFloat(S20W);
    Delta1 = parseFloat(Delta1);
    Mr = parseFloat(Mr);
    Vbar = parseFloat(Vbar);
    Fp = parseFloat(Fp);

    var ansp = 0.0;
    var anso = 0.0;
    var X;
    var f20w;
    var tempnum;
    var tempden;
    var delta = Delta1;

 //   print("_computeExactAOverB_p");
//    print("\tS20W=" + S20W);
 //   print("\tDelta1=" + Delta1);
 //   print("\tMr=" + Mr);
 //   print("\tVbar=" + Vbar);
//    print("\tFp=" + Fp);
//
    if (!isNaN(S20W) && S20W != 0.0)
    {
	f20w = Mr * (1.0 - Vbar * Rho_Water) / (Na * S20W);
	tempden = Math.pow((1.0 + delta / (Rho_Water * 0.7654)),(1.0 / 3.0));
	tempnum = (f20w / Fp);

	X = (tempnum / tempden) - 1.0;
	//print("\tX=" + X);

	if (X > 0.0)
	{
            ansp = 1.0 + 2.939184 * Math.pow(X,0.5) + 8.072775 * X + 8.55676 * Math.pow(X,2.0) - 0.505951 * Math.pow(X,3.0) + 0.0361508 * Math.pow(X,4.0);
            anso = 1 + 3.009376 * Math.pow(X,0.5) + 8.413897 * X + 13.34978 * Math.pow(X,2.0) + 3.18903 * Math.pow(X,3.0) + 0.106307 * Math.pow(X,4.0);
            //conaoverberrorp = False
	}
	else
	{
            //'MsgBox ("Maximum s is lower than S20w, no a/b possible")
	    //conaoverberrorp = True
	}
    }
 //   print("\tA=" + ansp);
  //  print("\tB=" + anso);

    A.VALUE = ansp;
    B.VALUE = anso;

}

// A is Rvalu(current).aoverbpp 
// B is Rvalu(current).aoverbpo 
// coninputopt = 0 nothing, 1 prolate, 2 oblate, 3 computed, 4 delta1
// S is Sp or So which is Smax which is also So
function _computeAOverB(S20W, S, A, B)
{
    print(">>Function: _computeAOverB(S20W,S,A,B)");
 //   print("\tS20W=" + S20W);
 //   print("\tS=" + S);
//    print("\tA.VALUE=" + A.VALUE);
 //   print("\tB.VALUE=" + B.VALUE);

    S20W = parseFloat(S20W);
    S = parseFloat(S);
    var errorFound = false;

 //   print("_computeAOverB");

    var X;
    var ansp;
    var anso;

    ansp = 0.0;
    anso = 0.0;

    // if there is a computed or entered s20w
    if (!isNaN(S20W) && S20W != 0.0)
    {
	//print("\tS=" + S);
	//print("\tS20W=" + S20W);
	X = S / S20W - 1.0;
	//print("\tX=" + X);
	if (X > 0.0)
	{
	    ansp = 1.0 + 2.939184 * Math.pow(X,0.5) + 8.072775 * X + 8.55676 * Math.pow(X,2.0) - 0.505951 * Math.pow(X,3.0) + 0.0361508 * Math.pow(X,4.0);
	    anso = 1.0 + 3.009376 * Math.pow(X,0.5) + 8.413897 * X + 13.34978 * Math.pow(X,2.0) + 3.18903 * Math.pow(X,3.0) + 0.106307 * Math.pow(X,4.0);
	    errorFound = false;
	}
	else
	{
	    errorFound = true;
	}
    }
 //   print("\tA=" + ansp);
 //   print("\tB=" + anso);

    A.VALUE = ansp;
    B.VALUE = anso;

    return errorFound;
}

function _computeOblateDimensionsP(Mr,AOverB, Delta, A,B)
{
    print(">>Function: _computeOblateDimensionsP(Mr,AOverB, Delta, A,B)");
    //print("\tMr=" + Mr);
    //print("\tAOverB=" + AOverB);
    //print("\tDelta=" + Delta);
    //print("\tA.VALUE=" + A.VALUE);
    //print("\tB.VALUE=" + B.VALUE);

    Mr     = parseFloat(Mr);
    Delta  = parseFloat(Delta);
    AOverB = parseFloat(AOverB);


    //print("_computeOblateDimensionsP");
    var aoverb = AOverB;
    var delta  = Delta;

    //print("delta = " + delta);
    //print("Mr = " + Mr);
    //print("Na = " + Na);
    //print("Density_20_Water = " + Density_20_Water);
    //print("aoverb = " + aoverb);

    if (aoverb == 0.0)
    {
	A.VALUE = 0.0;
	B.VALUE = 0.0;
	return;
    }
    var volp   = Mr * .7654 * 1e+21 / Na;
    var volw   = Mr * delta * 1e+21 / (Na * Density_20_Water);

    //print("volp = " + volp);
    //print("volw = " + volw);
    
    A.VALUE = Math.pow(((volw + volp) * aoverb *  3.0 / (4.0 * PI)),(1.0/3.0));
    //print("A = " + A.VALUE);
    B.VALUE = A.VALUE / aoverb;
    //print("B = " + B.VALUE);
}

// Rvalue(current).aoverbpp
// if a hydration was entered
//    delta = user entered hydration (Svalu(current).delta1in)
//    aoverb = aoverbpp
//    if (aoverb == 0), then a=0 and b=0
// else if use calculated hydration and delta1c exists
//    delta = composition calculated hydration (Svalu(current).delta1c)
//    aoverb = aoverbpp
//    if (aoverb == 0), then a=0 and b=0
// else 
//    if no hydration was entered
//       delta = 0;
//       aoverb = aoverbpp
//    else
//       delta = Hydration by teller
//       aoverb = user entered prolate or oblate ratio
//    if (aoverb == 0), then a=0 and b=0
//
function _computeProlateDimensionsP(Mr,AOverB,Delta,Ap,Bp)
{
//    print(">>Function: _computeProlateDimensionsP(Mr,AOverB, Delta, Ap,Bp)");
//    print("\tMr=" + Mr);
//    print("\tAOverB=" + AOverB);
//    print("\tDelta=" + Delta);
//    print("\tAp.VALUE=" + Ap.VALUE);
//    print("\tBp.VALUE=" + Bp.VALUE);

    Mr             = parseFloat(Mr);
    AOverB         = parseFloat(AOverB);
    Delta          = parseFloat(Delta);

    //print("_computeProlateDimensionsP");

    var aoverb;
    var delta;

    aoverb = AOverB;
    delta  = Delta;

    if (aoverb == 0)
    {
	Ap.VALUE = 0.0;
	Bp.Value = 0.0;
	return;
    }
    
   // print("\tMr = " + Mr);
   // print("\tNa = " + Na);
  //  print("\taoverb = " + aoverb);
 //   print("\tdelta = " + delta);

    var volp   = Mr * .7654 * 1e+21 / Na;
    var volw   = Mr * delta * 1e+21 / (Na * Density_20_Water);

 //   print("\tvolp = " + volp);
 //   print("\tvolw = " + volw);
    
    Ap.VALUE = Math.pow(((volw + volp) * aoverb * aoverb *  3.0 / (4.0 * PI)),(1.0/3.0));
 //   print("\tAp = " + Ap.VALUE);
    Bp.VALUE = Ap.VALUE / aoverb;
  //  print("\tBp = " + Bp.VALUE);
}

function _computeOblateDimensionsO(Mr,Vbar,AOverB,Delta,A,B)
{
    print(">>Function: _computeOblateDimensionsO(Mr,Vbar,AOverB, Delta, A,B)");
  //  print("\tMr=" + Mr);
 //   print("\tVbar=" + Vbar);
  //  print("\tAOverB=" + AOverB);
 //   print("\tDelta=" + Delta);
  //  print("\tA.VALUE=" + A.VALUE);
  //  print("\tB.VALUE=" + B.VALUE);

    Mr     = parseFloat(Mr);
    AOverB = parseFloat(AOverB);
    Delta = parseFloat(Delta);
    Vbar   = parseFloat(Vbar);


    //print("_computeOblateDimensionsO");
    var aoverb = AOverB;
    var delta  = Delta;

    //print("delta = " + delta);
    //print("Mr    = " + Mr);
    //print("Vbar  = " + Vbar);
    //print("Na    = " + Na);
    //print("aoverb = " + aoverb);
    if (aoverb == 0.0)
    {
	A.VALUE = 0.0;
	B.VALUE = 0.0;
	return;
    }

    var volp     = Mr * Vbar * 1e+21 / Na;
    var volw     = Mr * delta * 1e+21 / (Na * Density_20_Water);

    //print("volp  = " + volp);
    //print("volw  = " + volw);
    
    A.VALUE = Math.pow(((volw + volp) * aoverb *  3.0 / (4.0 * PI)),(1.0/3.0));
    //print("A = " + A.VALUE);
    B.VALUE = A.VALUE / aoverb;
    //print("B = " + B.VALUE);
}

function _computeProlateDimensionsO(Mr,Vbar,AOverB,Delta,A,B)
{
    print(">>Function: _computeProlateDimensionsO(Mr,Vbar,AOverB, Delta, A,B)");
    //print("\tMr=" + Mr);
    //print("\tVbar=" + Vbar);
    //print("\tAOverB=" + AOverB);
    //print("\tDelta=" + Delta);
    //print("\tA.VALUE=" + A.VALUE);
    //print("\tB.VALUE=" + B.VALUE);

    Mr     = parseFloat(Mr);
    Vbar   = parseFloat(Vbar);
    AOverB = parseFloat(AOverB);
    Delta = parseFloat(Delta);


 //   print("_computeProlateDimensionsO");

    var aoverb = AOverB;
    var delta  = Delta;

  //  print("\tdelta = " + delta);
 //   print("\tMr    = " + Mr);
 //   print("\tVbar  = " + Mr);
//    print("\tNa    = " + Na);
 //   print("\tDensity_20_Water = " + Density_20_Water);
 //   print("\taoverb = " + aoverb);

    if (aoverb == 0.0)
    {
	A.VALUE = 0.0;
	B.VALUE = 0.0;
	return;
    }

    var volp     = Mr * Vbar * 1e+21 / Na;
    var volw     = Mr * delta * 1e+21 / (Na * Density_20_Water);

  //  print("\tvolp  = " + volp);
  //  print("\tvolw  = " + volw);
    
    A.VALUE = Math.pow(((volw + volp) * aoverb * aoverb *  3.0 / (4.0 * PI)),(1.0/3.0));
    if (isNaN(A.VALUE))
    {
//	print("\tisNaN()");
	A.VALUE = 0.0;
    }
  //  print("\tA = " + A.VALUE);
    B.VALUE = A.VALUE / aoverb;
//    print("\tB = " + B.VALUE);
}

function calculateResults(Mr, Vbar, UnDenaturedVbar, S20W, Mb, 
	  		  T,  Sigma, RPM,  Ratio,
			  Hydration, HydrationValue,
			  Shape, BufferViscosity)
{
    print(">>Function: calculateResults(Mr, Vbar, UnDenaturedVbar, S20W, Mb, T,  Sigma, RPM,  Ratio, Hydration, HydrationValue, Shape, BufferViscosity)");
    //print("\tMr=" + Mr.VALUE);
    print("\tVbar=" + Vbar.VALUE);
    //print("\tS20W=" + S20W.VALUE);
    //print("\tMb=" + Mb.VALUE);
    //print("\tT=" + T.VALUE);
    //print("\tSigma=" + Sigma.VALUE);
    //print("\tRPM=" + RPM.VALUE);
    //print("\tRatio=" + Ratio.VALUE);
    //print("\tHydration=" + Hydration.VALUE);
    //print("\tHydrationValue=" + HydrationValue.VALUE);
    //print("\tShape=" + Shape.VALUE);
    //print("\tBufferViscosity=" + BufferViscosity.VALUE);
    print("\tUnDenaturedVbar = " + UnDenaturedVbar.VALUE);   

    Mr             = parseFloat(Mr.VALUE);
    Vbar           = parseFloat(Vbar.VALUE);
    UnDenaturedVbar= parseFloat(UnDenaturedVbar.VALUE);
    S20W           = parseFloat(S20W.VALUE);
    Mb             = parseFloat(Mb.VALUE);
    T              = parseFloat(T.VALUE);
    Sigma          = parseFloat(T.VALUE);
    RPM            = parseFloat(RPM.VALUE);
    Ratio          = parseFloat(Ratio.VALUE);
    Hydration      = String(Hydration.VALUE);
    HydrationValue = parseFloat(HydrationValue.VALUE);
    Shape          = String(Shape.VALUE);
    BufferViscosity= String(BufferViscosity.VALUE);

 //   print("calculateResults");
    if (Mr == 0)
    {
	var obj = new Array();
	
	obj[0] = new Object();
	obj[0].NAME  = "qtResultsErrorText";
	obj[0].ERROR = "You must have a molecular weight to get results";
	obj[0].STYLESHEET = "background-color:rgb(#DDA6A6);";
	return obj;
    }

    // Returned array
    var obj = new Array();
    var index = new Object(0);
    var Delta1Max = 0.0;
    var aoverberro = false;
    var aoverberrp = false;

    var Rp        = _calculateRp(Mr);
    var Ro;
    if (UnDenaturedVbar != 0)
        Ro        = _calculateRo(Mr, UnDenaturedVbar);
    else 
        Ro        = _calculateRo(Mr, Vbar);
     
    // These have to be done, regardless of settings
    var Fo        = 6.0 * PI * Viscosity_20_Water * Ro;
    var Fp        = 6.0 * PI * Viscosity_20_Water * Rp;
//    var FFp       = _calculateFFp(Mr, Vbar, S20W);
//    var FFo       = _calculateFFo(Mr, Vbar, S20W);
    var Sp        = _calculateSp(Mr, Vbar, Fp, Mb);
    var Dp        = _calculateDp(Mr, Vbar, Fp, T);
    var Do        = _calculateDo(Fo, T);
    var So        = _calculateSo(Mr, Vbar, Fo, Do, Mb, Sigma, RPM);
    var FFp       = _calculateFFp(Sp, S20W);
    var FFo       = _calculateFFo(So, S20W);
    var delta1p;

    //S20W = _calculateS20W(Tone, Ttwo, S, Vbar20, Visc, Vbar, Density);

    obj[index] = new Object();
    obj[index].NAME = "qtTellerffp";
    if (isNaN(FFp))
    {
	FFp = "";
	obj[index++].VALUE = FFp;
    }
    else
    {
	obj[index++].VALUE = Math.round(FFp*10000)/10000;
    }
//////FFp.toExponential(4)
    obj[index] = new Object();
    obj[index].NAME = "qtVbarffo";
    if (isNaN(FFo))
    {
	FFo = "";
	obj[index++].VALUE = FFo;
    }
    else
    {
	obj[index++].VALUE = Math.round(FFo*10000)/10000;
    }
/////FFo.toExponential(4)

    obj[index] = new Object();
    obj[index].NAME = "qtVbarFo";
    obj[index++].VALUE = Fo.toExponential(4);

    obj[index] = new Object();
    obj[index].NAME = "qtTellerFp";
    obj[index++].VALUE = Fp.toExponential(4);

    obj[index] = new Object();
    obj[index].NAME = "qtTellerRp";
    obj[index++].VALUE = Rp.toExponential(4);

    obj[index] = new Object();
    obj[index].NAME = "qtVbarRo";
    obj[index++].VALUE = Ro.toExponential(4);

    obj[index] = new Object();
    obj[index].NAME = "qtTellerSmax";
    obj[index++].VALUE = Sp.toExponential(4);

    obj[index] = new Object();
    obj[index].NAME = "qtTellerDmax";
    obj[index++].VALUE = Dp.toExponential(4);

    obj[index] = new Object();
    obj[index].NAME = "qtVbarSmax";
    obj[index++].VALUE = So.toExponential(4);

    obj[index] = new Object();
    obj[index].NAME = "qtVbarDmax";
    obj[index++].VALUE = Do.toExponential(4);

    if (!isNaN(S20W))
    {
	var rstokes;
        
       if (UnDenaturedVbar != 0)
          rstokes = Mr * Ro * (1.0 - UnDenaturedVbar * Rho_Water)/(S20W * Na * Fo);
       else
          rstokes = Mr * Ro * (1.0 - Vbar * Rho_Water)/(S20W * Na * Fo);


	obj[index] = new Object();
	obj[index].NAME = "qtRStokes";
	obj[index++].VALUE = rstokes.toExponential(4);
    }

    //if (isNaN(HydrationValue) && Hydration != "qtHydrateDelta1")  // [x1]
    //This condition occurs when there is no calculated hydration value
    //and the user has not selected to use a calculated or user inputed



//print("sgopkg;skg;saksgd;gkg;sdkl;sdkgl;ksl;gkl;kgdl;kgl;gdkg;lkg;glsks;lkg;lkdg;ldk;lsdkg;lgdk")
//print("Hydration = " + Hydration)
    if ((Hydration != "qtHydrateDelta1" && Hydration != "qtHydrateComputed"))  // [x1]
    {
	//print("[x1]: No Hydration value available");
	// No delta1 and no AOverB available.
	if (Hydration == "qtHydrateNothing") // [x2]
	{
	    //print("[x2]: HydrateNothing");
	    var Ap = new Object(0.0);
	    var Bp = new Object(0.0);
	    var Ao = new Object(0.0);
	    var Bo = new Object(0.0);
	    var aoverb = 0.0;
	    var labelo;
	    var labelp;
	    
	    // Teller
	    delta1p = _computeDelta1_p(Mr, Vbar, S20W, Mb, Fp);
	    index = FormSetDelta1(index, obj, "Teller", "Delta1 max", delta1p);

	    
	    // Calculate A/B : Teller
	    if (_computeAOverB(S20W, Sp, Ap, Bp) == true)
	    {
		labelp = "a/b Error S > Smax";
		//print(labelp);
	    }
	    else
	    {
		labelp = "max a/b";
	    }
	    	    	
	    delta1o = _computeDelta1_o(Mr, Vbar, S20W, Mb, Fo);
	    index = FormSetDelta1(index, obj, "Vbar", "Delta1 max", delta1o);

	    // Calculate A/B : Vbar
	    if (_computeAOverB(S20W, So, Ao, Bo) == true)
	    {
		labelo = "a/b Error S > Smax";
		//print(labelo);
	    }
	    else
	    {
		labelo = "max a/b";
	    }
	    
	    if (Shape == "qtShapeProlate") // [x3]
	    {
		//print("[x3]: ShapeProlate");
	    _computeExactAOverB_p(S20W,0,Mr,Vbar,Fp,Ap,Bp);
	    aoverb = parseFloat(Ap.VALUE);
	    index = FormSetMaxAB(index, obj, "Teller", labelp, aoverb);
	    
	    // Calculate and diplay prolate dimensions : Teller
	    _computeProlateDimensionsP(Mr,aoverb,0,Ap,Bp);
	    index = FormSet2AB(index, obj, "Teller", 
			       "2a max", parseFloat(Ap.VALUE), 
			       "2b min", parseFloat(Bp.VALUE));
	    
	    // Calculate a/b : Vbar
	    _computeExactAOverB_o(S20W,0,Mr,Vbar,Fo,Ao,Bo);
	    aoverb = parseFloat(Ao.VALUE);
	    index = FormSetMaxAB(index, obj, "Vbar", labelo, aoverb);
	    
	    // Calculate and diplay prolate dimensions : Vbar
	    _computeProlateDimensionsO(Mr,Vbar,aoverb,0,Ao,Bo);
	    index = FormSet2AB(index, obj, "Vbar", 
			       "2a max", parseFloat(Ao.VALUE), 
			       "2b min", parseFloat(Bo.VALUE));

	    }
	    else if (Shape == "qtShapeOblate") // [x4]
	    {
		//print("[x4]: ShapeOblate");
			    // Calculate a/b : Teller
	    _computeExactAOverB_p(S20W,0,Mr,Vbar,Fp,Ap,Bp);
	    aoverb = parseFloat(Bp.VALUE);
		//print("aoverbt = " + aoverb)
	    index = FormSetMaxAB(index, obj, "Teller", labelp, aoverb);
	    
	    // Calculate and diplay oblate dimensions : Teller
	    _computeOblateDimensionsP(Mr,aoverb,0,Ap,Bp);
	    index = FormSet2AB(index, obj, "Teller", 
			       "2a max", parseFloat(Ap.VALUE), 
			       "2b min", parseFloat(Bp.VALUE));
	    
	    // Calculate a/b : Vbar
	    _computeExactAOverB_o(S20W,0,Mr,Vbar,Fo,Ao,Bo);
	    aoverb = parseFloat(Bo.VALUE);
		//print("aoverbv = " + aoverb)
	    index = FormSetMaxAB(index, obj, "Vbar", labelo, aoverb);
	    
	    // Calculate and diplay prolate dimensions : Vbar
	    _computeOblateDimensionsO(Mr,Vbar,aoverb,0,Ao,Bo);
	    index = FormSet2AB(index, obj, "Vbar", 
			       "2a max", parseFloat(Ao.VALUE), 
			       "2b min", parseFloat(Bo.VALUE));
	    }
	    else // [x5]
	    {
		//print("[x5]: ShapeCylinder");
		var Lp = new Object(0.0);
		var Dp = new Object(0.0);
		var Lo = new Object(0.0);
		var Do = new Object(0.0);
		var loverdp = 0.0;
		var loverdo = 0.0;
		
		//loverdp = _computeLOverD_p(HydrationValue, S20W, Ratio, Mr, Vbar, Lp, Dp);
		loverdp = _computeLOverD_p(HydrationValue, S20W, 0, Mr, Vbar, Lp, Dp,BufferViscosity);
		index = FormSetMaxAB(index, obj, "Teller", "L/d", loverdp);
		
		index = FormSetld(index, obj, "Teller", 
				   "L", parseFloat(Lp.VALUE), 
				   "d", parseFloat(Dp.VALUE));

		//loverdo = _computeLOverD_o(Mr, Hydration, HydrationValue, Vbar, S20W, Ratio, Lo,Do);
		loverdo = _computeLOverD_o(Mr, Hydration, HydrationValue, Vbar, S20W, 0, Lo,Do,BufferViscosity);
		index = FormSetMaxAB(index, obj, "Vbar", "L/d", loverdo);
		
		index = FormSetld(index, obj, "Vbar", 
				   "L", parseFloat(Lo.VALUE), 
				   "d", parseFloat(Do.VALUE));
	    }
	}
	else // [x6] an a/b is entered, no delta1 entered or computed
	{
	    var Ap = new Object(0.0);
	    var Bp = new Object(0.0);
	    var Ao = new Object(0.0);
	    var Bo = new Object(0.0);
	    
	    //print("[x6]");
	    if (Hydration == "qtHydrateProlate") // [x7]
	    {
		//print("[x7]: HydrateProlate");
		// aoverb is user entered
	//	print("cond7: HydrateProlate");
		
		// Calculate and display Delta1 : Teller
		delta1p = _computeExactDelta1p_o(S20W,Mr,Mb,Vbar,Ratio,Fp);
	//	print("index before FormSetDelta=" + index);
		index = FormSetDelta1(index, obj, "Teller", "Delta1", delta1p);
	//	print("index after FormSetDelta=" + index);
		
		// Calculate and diplay prolate dimensions : Teller
		_computeProlateDimensionsP(Mr,Ratio,delta1p,Ap,Bp);
	//	print("index before FormSetMaxAB=" + index);
		index = FormSetMaxAB(index, obj, "Teller", "Exact a/b", Ratio);
	//	print("index after FormSetMaxAB=" + index);
	//	print("index before FormSet2AB=" + index);
		index = FormSet2AB(index, obj, "Teller", 
				   "2a", parseFloat(Ap.VALUE), 
				   "2b", parseFloat(Bp.VALUE));
	//	print("index after FormSet2AB=" + index);
		
		// Calculate and display Delta1 : Vbar
		
		//delta1o = _computeExactDelta1_p(S20W,Mr,Mb,Vbar,Ratio,Fp);
		delta1o = _computeExactDelta1_o(S20W,Mr,Mb,Vbar,Ratio,Fo);
		index = FormSetDelta1(index, obj, "Vbar", "Delta1", delta1o);
		
		// Calculate and diplay prolate dimensions : Vbar
		_computeProlateDimensionsO(Mr,Vbar,Ratio,delta1o,Ao,Bo);
		index = FormSetMaxAB(index, obj, "Vbar", "Exact a/b", Ratio);
		index = FormSet2AB(index, obj, "Vbar", 
				   "2a", parseFloat(Ao.VALUE), 
				   "2b", parseFloat(Bo.VALUE));
	    }
	    else
	    {
		//print("[x8]: HydrateOblate");
		// Calculate and display Delta1 : Teller 
		delta1p = _computeExactDelta1p_o(S20W,Mr,Mb,Vbar,Ratio,Fp);
		index = FormSetDelta1(index, obj, "Teller", "Delta1", delta1p);
		
		// Calculate and diplay oblate dimensions : Teller
		_computeOblateDimensionsP(Mr,Ratio,delta1p,Ap,Bp);
		index = FormSetMaxAB(index, obj, "Teller", "Exact a/b", Ratio);
		index = FormSet2AB(index, obj, "Teller", 
				   "2a", parseFloat(Ap.VALUE), 
				   "2b", parseFloat(Bp.VALUE));
		
		// Calculate and display Delta1 : Vbar 
		delta1o = _computeExactDelta1_o(S20W,Mr,Mb,Vbar,Ratio,Fo);
		index = FormSetDelta1(index, obj, "Vbar", "Delta1", delta1o);
		
		// Calculate and diplay oblate dimensions : Vbar
		_computeOblateDimensionsO(Mr,Vbar,Ratio,delta1o,Ao,Bo);
		index = FormSetMaxAB(index, obj, "Vbar", "Exact a/b", Ratio);
		index = FormSet2AB(index, obj, "Vbar", 
				   "2a", parseFloat(Ao.VALUE), 
				   "2b", parseFloat(Bo.VALUE));
	    }
	}
    }
    else // Hydration is available. Either use the available one or one that
	// was entered by the user.
    {
	// [x9] computed delta1 or inputed delta1
	//print("cond9: Hydration Available");
	var Ap = new Object(0.0);
	var Bp = new Object(0.0);
	var Ao = new Object(0.0);
	var Bo = new Object(0.0);
	var aoverb = 0.0;
	var delta1;
	
	// Determine which Delta1 to use
	if (Hydration == "qtHydrateDelta1") // [x10]
	    delta1 = Ratio;
	else
	    delta1 = HydrationValue;
	
	index = FormSetDelta1(index, obj, "Teller", "Exact Delta1", delta1);
	index = FormSetDelta1(index, obj, "Vbar", "Exact Delta1", delta1);
	
	if (Shape == "qtShapeProlate")
	{
	    //print("[x11]: ShapeProlate");
	    // Calculate a/b : Teller
	    _computeExactAOverB_p(S20W,delta1,Mr,Vbar,Fp,Ap,Bp);
	    aoverb = parseFloat(Ap.VALUE);
	    index = FormSetMaxAB(index, obj, "Teller", "a/b", aoverb);
	    
	    // Calculate and diplay prolate dimensions : Teller
	    _computeProlateDimensionsP(Mr,aoverb,delta1,Ap,Bp);
	    index = FormSet2AB(index, obj, "Teller", 
			       "2a", parseFloat(Ap.VALUE), 
			       "2b", parseFloat(Bp.VALUE));
	    
	    // Calculate a/b : Vbar
	    _computeExactAOverB_o(S20W,Ratio,Mr,Vbar,Fo,Ao,Bo);
	    aoverb = parseFloat(Ao.VALUE);
	    index = FormSetMaxAB(index, obj, "Vbar", "a/b", aoverb);
	    
	    // Calculate and diplay prolate dimensions : Vbar
	    _computeProlateDimensionsO(Mr,Vbar,aoverb,delta1,Ao,Bo);
	    index = FormSet2AB(index, obj, "Vbar", 
			       "2a", parseFloat(Ao.VALUE), 
			       "2b", parseFloat(Bo.VALUE));
	}
	else if (Shape == "qtShapeOblate")
	{
	    //print("[x12]: ShapeOblate");
	    // Calculate a/b : Teller
	    _computeExactAOverB_p(S20W,delta1,Mr,Vbar,Fp,Ap,Bp);
	    aoverb = parseFloat(Bp.VALUE);
		//print("aoverbt = " + aoverb)
	    index = FormSetMaxAB(index, obj, "Teller", "a/b", aoverb);
	    
	    // Calculate and diplay oblate dimensions : Teller
	    _computeOblateDimensionsP(Mr,aoverb,delta1,Ap,Bp);
	    index = FormSet2AB(index, obj, "Teller", 
			       "2a", parseFloat(Ap.VALUE), 
			       "2b", parseFloat(Bp.VALUE));
	    
	    // Calculate a/b : Vbar
	    _computeExactAOverB_o(S20W,Ratio,Mr,Vbar,Fo,Ao,Bo);
	    aoverb = parseFloat(Bo.VALUE);
		//print("aoverbv = " + aoverb)
	    index = FormSetMaxAB(index, obj, "Vbar", "a/b", aoverb);
	    
	    // Calculate and diplay prolate dimensions : Vbar
	    _computeOblateDimensionsO(Mr,Vbar,aoverb,Ratio,Ao,Bo);
	    index = FormSet2AB(index, obj, "Vbar", 
			       "2a", parseFloat(Ao.VALUE), 
			       "2b", parseFloat(Bo.VALUE));
	}
	else
	{
	    //print("[x13]: ShapeCylinder");
	    var Lp = new Object(0.0);
	    var Dp = new Object(0.0);
	    var Lo = new Object(0.0);
	    var Do = new Object(0.0);

	    var loverdp = _computeLOverD_p(HydrationValue, S20W, Ratio, Mr, Vbar, Lp, Dp,BufferViscosity);
	    index = FormSetMaxAB(index, obj, "Teller", "L/d", loverdp);

	    index = FormSetld(index, obj, "Teller", 
			       "L", parseFloat(Lp.VALUE), 
			       "d", parseFloat(Dp.VALUE));
	    var loverdo = _computeLOverD_o(Mr, Hydration, HydrationValue, Vbar, S20W, Ratio, Lo,Do,BufferViscosity);
	    index = FormSetMaxAB(index, obj, "Vbar", "L/d", loverdo);

	    index = FormSetld(index, obj, "Vbar", 
			       "L", parseFloat(Lo.VALUE), 
			       "d", parseFloat(Do.VALUE));
	}
    }

    //print("Results done");
    return obj;
}


function _computeExactDelta1_o(S20W,Mr,Mb,Vbar,aoverboo,Fo)
{
    print(">>Function: _computeExactDelta1_o(S20W,Mr,Mb,Vbar,aoverboo,Fo) ");
    //print("\tS20W=" + S20W);
    //print("\tMr=" + Mr);
    //print("\tMb=" + Mb);
    //print("\tVbar=" + Vbar);
    //print("\taoverboo=" + aoverboo);
    //print("\tFo=" + Fo);
    S20W     = parseFloat(S20W);
    Mr       = parseFloat(Mr);
    Mb       = parseFloat(Mb);
    Vbar     = parseFloat(Vbar);
    aoverboo = parseFloat(aoverboo);
    Fo       = parseFloat(Fo);

    var tempans;
    var CapF;
    var f20w;
    var tempab;


    if (!isNaN(S20W) && !isNaN(Mb))
    {
	f20w = Mr * (1.0 - Vbar * Rho_Water) / (Na * S20W);
	tempab = aoverboo;
	if (tempab != 1.0)
	{
            CapF = (Math.pow((Math.pow(tempab,2) - 1),0.5)) / (Math.pow(tempab,(2 / 3)) * Math.atan(Math.pow((Math.pow(tempab,2) - 1),0.5)));
	}
	else
	{
            CapF = 1;
	}
	tempans = ((Math.pow(((f20w / Fo) / CapF),3)) - 1) * Vbar / Rho_Water;
    }
    else
    {
	tempans = 0.0;
    }

    if (tempans >= 0)
    {
	return tempans;
    }
    else
    {
	return 0.0;
	//MsgBox "Warning--this axial ratio implies hydration < 0", 64, TheProgram
    }
}

function _computeExactDelta1p_o(S20W,Mr,Mb,Vbar,Ratio,Fp)
{
    print(">>Function: _computeExactDelta1p_o(S20W,Mr,Mb,Vbar,Ratio,Fp) ");
/*
    print("\tS20W=" + S20W);
    print("\tMr=" + Mr);
    print("\tMb=" + Mb);
    print("\tVbar=" + Vbar);
    print("\tRatio=" + Ratio);
    print("\tFp=" + Fp);*/

    S20W     = parseFloat(S20W);
    Mr       = parseFloat(Mr);
    Mb       = parseFloat(Mb);
    Vbar     = parseFloat(Vbar);
    Ratio    = parseFloat(Ratio);
    Fp       = parseFloat(Fp);

    var delta1p = 0;
    var CapF;
    var f20w;
    var tempab;
    var tempans;

  /*  print("\tS20W=" + S20W);
    print("\tMr=" + Mr);
    print("\tMb=" + Mb);
    print("\tVbar=" + Vbar);
    print("\tRatio=" + Ratio);
    print("\tFp=" + Fp);*/

    if (!isNaN(S20W) && !isNaN(Mb))
    {
	//print("!isNan");
	f20w = Mr * (1.0 - Vbar * Density_20_Water) / 
	    (Na * S20W);

	//print("\tf20w=" + f20w);

	tempab = Ratio;
	//print("tempab=" + tempab);
	if (tempab != 1.0)
	{
	    CapF = ((Math.pow(Math.pow(tempab,2.0) - 1.0),0.5)) / (Math.pow(tempab,(2.0 / 3.0)) * Math.atan((Math.pow(Math.pow(tempab,2) - 1.0),0.5)));
	}
	else
	{
	    CapF = 1.0;
	}
	delta1p = ((Math.pow(((f20w / Fp) / CapF), 3.0)) - 1.0) * .7654 / Rho_Water;

	//print("\tCapF=" + CapF);
	//print("\tdelta1p= " + delta1p);
    }
    else
    {
	delta1p = 0.0;
    }

    if (delta1p >= 0)
	return delta1p;
    else
    {
	print("\tWarning--this axial ratio implies hydration < 0");
	// QMessageBox:: Warning--this axial ratio implies hydration < 0"
	return 0.0;
    }
}

function _computeExactDelta1_p(S20W,Mr,Vbar,Ao,Fo)
{
    print(">>Function: _computeExactDelta1_p(S20W,Mr,Vbar,Ao,Fo) ");
/*
    print("\tS20W=" + S20W);
    print("\tMr=" + Mr);
    print("\tVbar=" + Vbar);
    print("\tAo=" + Ao);
    print("\tFo=" + Fo);*/

    S20W     = parseFloat(S20W);
    Mr       = parseFloat(Mr);
    Vbar     = parseFloat(Vbar);
    Ao       = parseFloat(Ao);
    Fo       = parseFloat(Ao);

    var f20w;
    var tempba;
    var CapF;
    var tempans;

   // print("_computeExactDelta1_p");
    
    if (!isNaN(S20W) && S20W != 0.0)
    {
	f20w = Mr * (1.0 - Vbar * Rho_Water) / (Na * S20W); 
	tempba = 1 / Ao;

	if (tempba != 1)
	{
	    CapF = Math.pow((1.0 - Math.pow(tempba,2.0)),0.5) / (Math.pow(tempba,(2.0 / 3.0)) * Math.log((1.0 + Math.pow((1.0 - Math.pow(tempba,2.0)),0.5)) / tempba));
	}
	else
	{
	    CapF = 1.0;
	}
	tempans = (Math.pow(((f20w / Fo) / CapF),3) - 1) * Vbar / Rho_Water;
    }
    else
    {
	tempans = 0.0;
    }

    if (tempans >= 0.0)
    {
	return tempans;
    }
    else
    {
	return 0.0;
	//MsgBox "Warning--this axial ratio implies hydration < 0", 64, TheProgram
    }
}

function _computeExactDelta1p_p(S20W,Mr,Mb,Vbar,Ratio,Fp)
{
    print(">>Function: _computeExactDelta1p_p(S20W,Mr,Mb,Vbar,Ratio,Fp) ");
/*
    print("\tS20W=" + S20W);
    print("\tMr=" + Mr);
    print("\tMb=" + Mb);
    print("\tVbar=" + Vbar);
    print("\tRatio=" + Ratio);
    print("\tFp=" + Fp);*/

    S20W     = parseFloat(S20W);
    Mr       = parseFloat(Mr);
    Mb       = parseFloat(Mb);
    Vbar     = parseFloat(Vbar);
    Ratio    = parseFloat(Ratio);
    Fp       = parseFloat(Fp);

    var delta1p;
    var CapF;
    var f20w;
    var tempba;

  //  print("_computeExactDelta1p_p");

    var aoverbpp = parseFloat(Ratio);

   // print("\tS20W=" + S20W);
   // print("\tMr=" + Mr);
   // print("\tMb=" + Mb);
   // print("\tVbar=" + Vbar);
   // print("\tRatio=" + Ratio);
   // print("\tFp=" + Fp);
   // print("\taoverbpp=" + aoverbpp);

    if (!isNaN(S20W) && !isNaN(Mb))
    {
	//print("\t!isNan && !isNaN(Mb)");
	f20w = Mr * (1.0 - Vbar * Density_20_Water) / 
	    (Na * S20W);
	//print("\tf20w=" + f20w);
	tempba = 1.0 / aoverbpp;
	//print("\ttempba=" + tempba);
	if (tempba != 1.0)
	{
	    CapF = (Math.pow((1.0 - Math.pow(tempba,2.0)),0.5)) / (Math.pow(tempba,(2.0 / 3.0)) * Math.log((1.0 + Math.pow((1.0 - Math.pow(tempba,2)),0.5)) / tempba));
	}
	else
	{
	    CapF = 1.0;
	}
	//print("\tCapF=" + CapF);
	delta1p = (Math.pow(((f20w / Fp) / CapF),3) - 1.0) * 0.7654 / Density_20_Water;
    }
    else
    {
	//print("\tisNaN");
	delta1p = 0.0;
    }

    if (delta1p >= 0)
    {
	//print("\tdelta1p=" + delta1p);
	return delta1p;
    }
    else
    {
	//print("\tdelta1p=0.0");
	return 0.0;
    }
}

// Set in COMPUTE.BAS and FRMRESU3.FRM
function _calculateAoverBpp(Hydration)
{
    //print(">>Function: _calculateAoverBpp(Hydration) ");
   // print("\tHydration=" + Hydration);

    var tempans = 1.0;

    if (Hydration == "qtHydrateComputed" || 
        Hydration == "qtHydrateDelta1")
    {
	tempans = 0.4;
    }

    if (Hydration == 'qtHydrateProlate')
    {
	if (tempans < 1.0)
	{
	    tempans = 1.0;
	}
	return tempans;
    }
    else if (Hydration == 'qtHydrateOblate')
    {
	return 0;
    }
    else if (Hydration == 'qtHydrateComputed')
    {
    }
}

/*
function _computeS20W(VelocityTemp, Vbar, Rho)
{
    VelocityTemp = parseFloat(VelocityTemp);
    Vbar = parseFloat(Vbar);
    Rho = parseFloat(Rho);

    var vbarrho;
    var vbarrhow;
    var tempans;
    
    // if qtSedVelocityTemp is set to the default
    if (VelocityTemp == DEFAULT_SedVelocityTemp) 
    {
        vbarrho = 1.0 - Vbar * Rho;
        
        tempans = evalu(current).sstar * (vbarrhow * evalu(current).vis) / (vbarrho * VISWATER)
        //formula 18,pg 10 note .01002 is viscosity of water in poise
    }
    else // qtSedVelocityTemp was changed by the user
    {
        var vbaropt;
        var rhoopt;
        
        vbaropt = Svalu(current).vbarin + 0.000425 * (evalu(current).sstart - evalu(current).Tc)
        if Svalu(current).avbarwater <> 0 
	{
            vbarrhow = 1 - Svalu(current).vbarwater * RHOWATER
            // Vbar rho at 20 c and water
	}
        else
	{
            vbarrhow = 1 - Svalu(current).vbarin * RHOWATER
            // Vbar rho at 20 c and water
	}
        rhoopt = computeRhoT(computerho20(Bvalu(current).rhoin, Bvalu(current).rhoint), evalu(current).sstart)
        vbarrho = 1 - (vbaropt * rhoopt)
        tempans = evalu(current).sstar * (vbarrhow * computeVisT(Bvalu(current).visin, evalu(current).sstart)) / (vbarrho * VISWATER)
        //formula 18,pg 10 note .01002 is viscosity of water in poise
    }
    Rvalu(current).S20w = tempans
    Rvalu(current).aS20w = -1
}
*/

function _computeLOverD_p(HydrationValue, S20W, Delta1, Mr, Vbar, l, d, BufferViscosity)
{
    print(">>Function:  _computeLOverD_p(HydrationValue, S20W, Delta1, Mr, Vbar, l, d, BufferViscosity)");
/*
    print("\tHydrationValue=" + HydrationValue);
    print("\tS20W=" + S20W);
    print("\tDelta1=" + Delta1);
    print("\tMr=" + Mr);
    print("\tVbar=" + Vbar);
    print("\tl=" + l.VALUE);
    print("\td=" + d.VALUE);
    print("\tBufferViscosity=" + BufferViscosity);
    print('Rho_Water = ' +Rho_Water)
*/

    HydrationValue = parseFloat(HydrationValue);
    S20W           = parseFloat(S20W);
    Delta1         = parseFloat(Delta1);
    Mr             = parseFloat(Mr);
    Vbar           = parseFloat(Vbar);

    var delta;
    var volw;
    var volp;
    var molvolume;
    var fricco;

    if (!isNaN(S20W) && S20W != 0.0)
    {
	if (!isNaN(HydrationValue))
	{
	    delta = Delta1;
	}
	else
	{
	    delta = 0.0;
	}   
	volw = 1E+21 * Mr * delta / (Na * Rho_Water);
	volp = 1E+21 * Mr * 0.7654 / Na;

	//print("\tvolw=" + volw);
	//print("\tvolp=" + volp);

	molvolume = (volp + volw) * 1E-21;
	fricco    = Mr * (1.0 - Vbar * Rho_Water) / (Na * S20W);

	//print("\tmolvolume=" + molvolume);
	//print("\tfricco=" + fricco);

	// init guesses
	// assume d is 2.5 nm
	var Guess = new Array(2);
	Guess[0] = 4.0 * molvolume / (PI * Math.pow(0.00000025,2));
	Guess[1] = 0.00000025;

	// solve for L and L/d by Newtons roots
	_MNewton(50,Guess,2,0.0000000000001,0.0000000000001,fricco,BufferViscosity,molvolume);

	var loverdp = Guess[0] / Guess[1];

	l.VALUE = Guess[0];
	d.VALUE = Guess[1];
	//print("\treturn l=" + l.VALUE);
	//print("\treturn d=" + d.VALUE);
	return loverdp;
 
//	if (l / d <= 2 Or l / d >= 20) And (frmResults.Visible) And (Not (frmResults.ActiveControl.Tag = "cmdGraphRatio") Then MsgBox "Be advised that the results from the Teller method are outside the range where the equations are accurate (2 < L/d < 20)", 64, TheProgram

    }
    else
    {
	var loverdp = 0.0;
	//Rvalu(current).loverdp = 0.0
	l.VALUE = 0.0;
	d.VALUE = 0.0;
	//print("\treturn l=" + l.VALUE);
	//print("\treturn d=" + d.VALUE);
	return loverdp;
    }
}

function _computeLOverD_o(Mr, Hydration, HydrationValue, Vbar, S20W, Delta1, l,d,BufferViscosity)
{
    print(">>Function: _computeLOverD_o(Mr, Hydration, HydrationValue, Vbar, S20W, Delta1, l, d, BufferViscosity)");
//    print("\tMr=" + Mr);
//    print("\tHydration=" + Hydration);
//    print("\tHydrationValue=" + HydrationValue);
//    print("\tVbar=" + Vbar);
//    print("\tS20W=" + S20W);
//    print("\tDelta1=" + Delta1);
//    print("\tl=" + l.VALUE);
//    print("\td=" + d.VALUE);
//    print("\tBufferViscosity=" + BufferViscosity);
//    print('ComputeLD_o('+Mr+','+Hydration+',' +HydrationValue+',' +Vbar+',' +S20W+',' +Delta1+',' +l+','+d+','+BufferViscosity+')')
//    print('Na = ' + Na)
//    print('Rhowater = ' + Rho_Water)
//    print('(-2)^(1/2) = ' + Math.pow(-2,.5))

    var guess = new Array(2);
    if (!isNaN(S20W))
    {
	if (Hydration == "qtHydrateDelta1")
	{
	    delta = HydrationValue;
	}
	else if (!isNaN(HydrationValue))
	{
            delta = Delta1;
	}
	else
	{
            delta = 0
	}

	//' compute volumes
	volw = 1E+21 * Mr * delta / (Na * Rho_Water); // ' in cubic nm
	volp = 1E+21 * Mr * Vbar / Na;

//	print("volw=" + volw);
//	print("volp=" + volp);

	molvolume = (volp + volw) * 1E-21; // 'in cubic cm
	fricco = Mr * (1.0 - Vbar * Rho_Water) / (Na * S20W); // 'Equation 3 pg 3
//	print("molvolume=" + molvolume);
//	print("fricco=" + fricco);
	//' init guesses
	//' assume d is 2.5 nm
	guess[0] = 4 * molvolume / (PI * Math.pow(0.00000025,2));
	guess[1] = 0.00000025;
	//' solve for L and L/d by Newtons roots
	_MNewton(50, guess, 2, 0.0000000000001, 0.0000000000001,fricco, BufferViscosity,molvolume);
	var loverdo = guess[0] / guess[1];
	l.VALUE = guess[0];
	d.VALUE = guess[1];
//	print( "guess[0]=   " + guess[0] + "guess[1]=   " + guess[1] );
//	print( 'l = ' + l + '           d = ' +d)
	return loverdo;
	//If (l / d <= 2 Or l / d >= 20) And frmResults.Visible And Not (frmResults.ActiveControl.Tag = "cmdGraphRatio") Then MsgBox "Be advised that the results from the vbar method are outside the range where the equations are accurate (2 < L/d < 20)", 64, TheProgram
    }
    else
    {
	//Rvalu(current).loverdo = 0
	l = 0;
	d = 0;
	return 0;
    }
}

function _MNewton(ntrial, guess, N, tolx, tolf, fricco, BufferViscosity,molvolume)
{
//    print(">>Function: _MNewton(ntrial, guess, N, tolx, tolf, fricco, BufferViscosity,molvolume)  ");

//    print("\tntrial=" + ntrial);
//    print("\tN=" + N);
//    print("\ttolx=" + tolx);
//    print("\ttolf=" + tolf);
//    print("\tfricco=" + fricco);
//    print("\tBufferViscosity=" + BufferViscosity);
//    print("\tmolvolume=" + molvolume);
//    print("\tguess0=" + guess[0]);
//    print("\tguess1=" + guess[1]); 


    var indx = new Array(N);
    var alpha1 = new Array(N);
    var beta = new Array(N);
    var i = 0;

for (i=0; i<N;i++){
alpha1[i]= new Array(N)
}

    for (k = 0; k < ntrial; k++)
    {
	
//	print("Part 1 beta   " + beta);

	_UsrFun(guess,alpha1,beta,fricco,BufferViscosity,molvolume);

	var errf = 0;

//	print("Part 2 beta   " + beta);

	for (i = 0; i < N; i++)
	{
	    errf = errf + Math.abs(beta[i]);
	}
	if (errf < tolf) {break;}

	d = new Object(0);
	

//	print("Part 3 beta  " + beta);
	_LUDecomp(alpha1,N, N, indx, d);
//	print("Part 4 beta  " + beta);
	_LUBacksub(alpha1,N,N,indx, beta);
//	print("Part 5 beta  " + beta);



	var errx = 0;
	for (i = 0; i < N; i++)
	{
		
	    errx = errx + Math.abs(beta[i]);
	    guess[i] = guess[i] + beta[i];
	
	}
	if (errx < tolx){break;}
    }

    if (k == ntrial)
    {
	print('bad news')

    }
}


function _LUBacksub(A, n, np, indx, B)
{
//    print(">>Function: _LUBacksub(A,n,np,indx,B)");
//    print('A=['+A[0][0]+','+A[0][1]+';'+A[1][0]+','+A[1][1]+']')
//    print("\tn=" + n);
 //   print("\tnp=" + np);
//    print("\tindx=" + indx);
//    print("\tB=" +'['+ B[0]+','+B[1]+']');
    //' A() is input n x n matrix from LUDecomp, with dimension np x np
    //' B() is input right-hand side vector, dimension n
    //' B() is output with solution vector
    
    var ll  = 0;
    var ii  = -1; 
    var Sum = 0;
    var i   = 0;
    var j   = 0;

    for (i = 0; i < n; i++)
    {

//print('1: ii = ' + ii)
//print('2: i = ' + i)



        ll = indx[i] 

//print('3: ll = ' + ll)

        Sum = B[ll]

//print('4: Sum = ' + Sum);

        B[ll] = B[i] 

//print('5: B[ll] = ' + B[ll])

        if (ii != (-1)) 
	{
//print('6: ii = ' + ii)

            for (j = ii; j < i; j++) 
	    {

//print('7-a: j = ' + j)
//print('7-b: A['+i+']['+j+'] = ' + A[i][j])
//print('7-c: B['+j+'] = ' + B[j])
//print('7-d: Sum = ' +Sum)

		Sum = Sum - (A[i][j]) * B[j]; 

//print('8: Sum = ' +Sum)
	    }
	}

        else if (Sum != 0) 
	{

//print('9: Sum = ' +Sum)
            ii = i; 
//print('10: ii = ' + ii);

	}

        B[i] = Sum; 
//	print("\tB[" + i + "]=" + B[i]);

    }

    for (i = (n-1); i >= 0; i--) 
    {
//print('11: i = ' + i)

        Sum = B[i] 

//print('12: Sum = ' + Sum)

        if (i < (n-1))
	{

//print('13: i = ' + i)

            for (j = (i + 1); j < n; j++)
	    {

//print('14: j = ' + j)
//print('15: A['+i+']['+j+'] = ' + A[i][j])
//print('16: B['+j+'] = ' + B[j]) 

                Sum = Sum - (A[i][j]) * B[j]

//print('17: Sum = ' + Sum)

           }
        }

//print('18: B['+i+'] = ' + B[i])

        B[i] = Sum / (A[i][i])

//	print('19: B[' + i + '] = ' + B[i]);
    }

//print("\treturned B=" + B);
}
/*
function _LUBacksub(A,N,np,indx,B)
{
    print(">>Function: _LUBacksub(A,N,np,indx,B)");
    print("\tA=" + A);
    print("\tN=" + N);
    print("\tnp=" + np);
    print("\tindx=" + indx);
    print("\tB=" + B);
 
    //' A() is input n x n matrix from LUDecomp, with dimension np x np
    //' B() is input right-hand side vector, dimension n
    //' B() is output with solution vector

    var ii = 0;
    var ll = 0;
    var i = 0;
    var Sum = 0;
    //' when ii becomes positive it will point to first non-zero element of B()
    for(i = 0; i < N; i++)
    {
        ll = indx[i];
        Sum = B[ll];
        B[ll] = B[i];

        if (ii != 0)
	{
            for (j = ii; j < i - 1; j++)
	    {
                Sum = Sum - A[i, j] * B[j];
            }
	}
        else if (Sum != 0)
	{
            ii = i;
        }
        B[i] = Sum;
    }
    //' now do the backsubstitution
    for (i = N-1; i >= 0; i--)
    {
        Sum = B[i];
        if (i < N)
	{
            for (j = i + 1; j < N; j++)
	    {
                Sum = Sum - A[i, j] * B[j];
	    }
        }
        B[i] = Sum / A[i, i];
    }
    print("\treturned B=" + B);
}
*/













function _LUDecomp(A, N, np, indx, d)
{
  //  print(">>Function: _LUDecomp(A, N, np, indx, d)");
  //  print("\tA[0,0]=" + A[0][0]);
  //  print("\tA[0,1]=" + A[0][1]);
  //  print("\tA[1,0]=" + A[1][0]);
  //  print("\tA[1,1]=" + A[1][1]);
  //  print('A=['+A[0][0]+','+A[0][1]+';'+A[1][0]+','+A[1][1]+']')
  //  print("\tN=" + N);
  //  print("\tnp=" + np);
  //  print("\tindx=" + indx);
  //  print("\td=" + d);

    // A() is input n x n matrix, with dimension np x np
    // A() is output with rearrangements recorded in indx()
    // d is output as +/- 1 to indicate even or odd rearrangements
    var tiny = 1E-37
    var d = 1
    var VV = new Array(np);





//print("1: VV = " + VV)

    for (i = 0; i < N; i++)
    {

//print("2: i = " +i)
	
	var aamax = 0;
        for (j = 0; j < N; j++)
	{

//print("3: j = " +j)
            
		if (Math.abs(A[i][j]) > aamax )
	        {
//print("4: A[i,j] = " +A[i][j])

		aamax = Math.abs(A[i][j]);
           	    }
	}

        if (aamax == 0)
	{
            //MsgBox "Singular matrix, aborting...", 32, TheProgram
            //ESCFlag = True
//            print("5: Singular matrix, aborting...");
	    return;
        }

//print('6: aamax = ' + aamax)
//print('7: VV['+i+'] = ' + VV[i])

        VV[i] = 1 / aamax; // ' save scaling

//print('8: VV['+i+'] = ' + VV[i])

    }

    for (j = 0; j < N; j++) // ' loop over columns of Crout's method
    {

//print('9: j = ' +j)

        if (j > 0)
	{

//	print('10: j>0')

            for (i = 0; i < j ; i++)
	    {
//		print('11: i = ' + i)

                Sum = A[i][j];

//		print('12: Sum = ' + Sum)

                if (i > 0)
		{
//			print('13: i = ' + i)

                    for (k = 0; k > i; k++)
		    {

//			print('14: k = ' + k)

                        Sum = Sum-(A[i][k])*(A[k][j]);

//			print('15: Sum = ' + Sum)

		    }
                    A[i][j] = Sum;

//			print('16: A['+i+','+j+'] = ' +Sum)
//print('17: ')
  //  print("\tp1 A[0,0]=" + A[0][0]);
  //  print("\tp1 A[0,1]=" + A[0][1]);
  //  print("\tp1 A[1,0]=" + A[1][0]);
  //  print("\tp1 A[1,1]=" + A[1][1]);
		}
	    }
        }

        aamax = 0;

        for (i = j; i < N; i++)
	{
//print('18: i = ' + i)

            Sum = A[i][j];

//print('19: Sum = ' + Sum)

            if (j > 0)
	    {
//print('20: j>0')
                for (k = 0; k < j; k++)
		{
//print('21: k = ' + k)
                    Sum = Sum - (A[i][k]) * (A[k][j]);
//print('22: Sum = ' + Sum)
		}
//print('23-a:')
  //  print("\tp2 A[0,0]=" + A[0][0]);
    //print("\tp2 A[0,1]=" + A[0][1]);
    //print("\tp2 A[1,0]=" + A[1][0]);
    //print("\tp2 A[1,1]=" + A[1][1]);
                A[i][j] = Sum;
//print('23-b:')
    //print("\tp2 A[0,0]=" + A[0][0]);
    //print("\tp2 A[0,1]=" + A[0][1]);
    //print("\tp2 A[1,0]=" + A[1][0]);
    //print("\tp2 A[1,1]=" + A[1][1]);
            }

            dum = VV[i] * Math.abs(Sum) //' figure of merit for pivot
//print('24: dum = ' +dum)
            if (dum > aamax) //' check for best so far
	    {
//print('25: dum > aamax')
                imax = i;
                aamax = dum;
//print('26: imax = ' + imax)
//print('27: aamax = ' + aamax)
	    }
	}
        if (j != imax) //' we need to interchange rows
	{
//print('28: j = ' + j)
            for (k = 0; k < N; k++)
	    {
//print('29: k = ' +k)
//print('30: dum = ' +dum)
//print('31: imax = ' + imax)
//print('32: A[imax,k] = ' + A[imax][k])
                dum = A[imax][k];
                A[imax][k] = A[j][k];
                A[j][k] = dum;
//print('33: A[imax,k] = ' + A[imax][k])
//print('34: A[j,k] = ' + A[j][k])
            }
            d = -d //' change parity of d
            VV[imax] = VV[j]; //' also interchange scale factor
//print('35: d = ' +d)
//print('36: VV[imax] = ' + VV[imax])
        }
        indx[j] = imax;
//print('37: indx[j] = ' + indx[j])
        if (j != (N-1)) //' divide by the pivot element
	{
//print('38: j!=n')
            if ((A[j][j]) == 0)
	    {
//print('39: A[j,j] = '+ tiny)
		A[j][j] = tiny; // ' it may be better to be tiny than zero
	    }
            dum = 1 / A[j][j];
//print('40: dum = ' + dum)
            for (i = (j + 1); i < N; i++)
	    {
                A[i][j] = A[i][j] * dum;
	    }
        }
    } //' back to next column

    if ((A[N-1][N-1]) == 0)
    { 
	A[N-1][N-1] = tiny;
    }
//    print("\treturned A[0,0]=" + A[0][0]);
//    print("\treturned A[0,1]=" + A[0][1]);
//    print("\treturned A[1,0]=" + A[1][0]);
//    print("\treturned A[1,1]=" + A[1][1]);
}






function _UsrFun(guess, alpha1, beta, fricco, BufferViscosity,molvolume)
{
//    print(">>Function: _UsrFun(guess, alpha1, beta, fricco, BufferViscosity,molvolume) ");

//    print("\tguess=" + guess);
//    print("\talpha1=" + alpha1);
//    print("\tbeta=" + beta);
//    print("\tfricco=" + fricco);
//    print("\tBufferViscosity=" + BufferViscosity);
//    print("\tmolvolume=" + molvolume);
//print('UsrFun(guess=['+guess[0]+','+guess[1]+'], alpha1=['+alpha1[0][0]+','+alpha1[0][1]+';'+alpha1[1][0]+','+alpha1[1][1]+'], beta=['+beta[0]+','+beta[1]+'], fricco=['+fricco+'], BufferViscosity=['+BufferViscosity+'],molvolume=['+molvolume+']')

    // for Newtons roots
    // return derivative in alpha1, function in beta
    // values for molecular volume and friction coefficient
    // are passed thru variables molvolume and fricco
    beta[0] = -guess[0] * Math.pow(guess[1],2.0) + 4.0 * molvolume / PI;
    beta[1] = -3.0 * PI * BufferViscosity * guess[0] + fricco * (Math.log(guess[0] / guess[1]) + 0.312 + 0.561 * guess[1] / guess[0] + 0.1 * (Math.pow((guess[1] / guess[0]),2)));

    alpha1[0][0] = Math.pow(guess[1],2.0);
    alpha1[0][1] = 2.0 * guess[0] * guess[1];
    alpha1[1][0] = 3.0 * PI * BufferViscosity - fricco * (1.0 / guess[0] - 0.561 * guess[1] / Math.pow(guess[0],2.0) - 0.2 * Math.pow(guess[1],2.0) / Math.pow(guess[0],3.0));
    alpha1[1][1] = -fricco * (1.0 / guess[1] + 0.561 / guess[0] - 0.2 * guess[1] / Math.pow(guess[0], 2.0));
}

function FormSetDelta1(index, obj, Method, Label, Value)
{
//    print("FormSetDelta1");
    var data = parseFloat(Value);
    if (isNaN(data))
    {
	data = 0.0;
    }

    if (Method == "Teller")
    {
    	obj[index] = new Object();
	obj[index].NAME = "qtTellerDelta1maxLabel";
	obj[index].VALUE = Label;
	obj[index].ISLABEL = "true";
	index++;
	
	obj[index] = new Object();
	obj[index].NAME = "qtTellerDelta1max";
	obj[index].VALUE = String(Math.round(data*10000)/10000); //////used to be data.toExponential(4) 
	obj[index].ISLABEL = "true";
	index++;

//	print("\tdelta1 teller=" + parseFloat(Value));
//	print("\tindex teller=" + index);
    }
    else
    {
    	obj[index] = new Object();
	obj[index].NAME = "qtVbarDelta1maxLabel";
	obj[index].VALUE = Label;
	obj[index].ISLABEL = "true";
	index++;
	
	obj[index] = new Object();
	obj[index].NAME = "qtVbarDelta1max";
	obj[index].VALUE = String(Math.round(data*10000)/10000);
	index++;
//////used to be data.toExponential(4)
//	print("\tdelta1 vbar=" + Value);
//	print("\tindex vbar=" + index);
    }
    return index;
}

function FormSetMaxAB(index, obj, Method, Label, Value)
{
//    print("FormSetMaxAB");
    var data = parseFloat(Value);
    if (isNaN(data))
    {
	data = 0.0;
    }

    if (Method == "Teller")
    {
    	obj[index] = new Object();
	obj[index].NAME = "qtTellermaxabLabel";
	obj[index].ISLABEL = "true";
	obj[index++].VALUE = Label;
	
	obj[index] = new Object();
	obj[index].NAME = "qtTellermaxab";
	obj[index++].VALUE = String(Math.round(data*10000)/10000);
///data.toExponential(3)

//	print("\tmax ab teller=" + parseFloat(Value));
    }
    else
    {
    	obj[index] = new Object();
	obj[index].NAME = "qtVbarmaxabLabel";
	obj[index].ISLABEL = "true";
	obj[index++].VALUE = Label;
	
	obj[index] = new Object();
	obj[index].NAME = "qtVbarmaxab";
	obj[index++].VALUE = String(Math.round(data*10000)/10000);
///data.toExponential(3)
//	print("\tmax ab vbar=" + parseFloat(Value));
    }
    return index;
}

function FormSet2AB(index, obj, Method, ALabel, AValue, BLabel, BValue)
{
//    print("FormSet2AB");
    var adata = parseFloat(AValue);
    var bdata = parseFloat(BValue);

    if (isNaN(adata))
    {
	adata = 0.0;
    }
    if (isNaN(bdata))
    {
	bdata = 0.0;
    }

    if (Method == "Teller")
    {
    	obj[index] = new Object();
	obj[index].NAME = "qtTeller2amaxLabel";
	obj[index].ISLABEL = "true";
	obj[index++].VALUE = ALabel;
	
	obj[index] = new Object();
	obj[index].NAME = "qtTeller2amax";
	obj[index++].VALUE = String(2.0 * adata.toExponential(3));

//	print("\t2A teller=" + parseFloat(2.0 * parseFloat(AValue)));

    	obj[index] = new Object();
	obj[index].NAME = "qtTeller2bminLabel";
	obj[index].ISLABEL = "true";
	obj[index++].VALUE = BLabel;
	
	obj[index] = new Object();
	obj[index].NAME = "qtTeller2bmin";
	obj[index++].VALUE = String(2.0 * bdata.toExponential(3));

//	print("\t2B teller=" + parseFloat(2.0 * parseFloat(BValue)));
    }
    else
    {
    	obj[index] = new Object();
	obj[index].NAME = "qtVbar2amaxLabel";
	obj[index].ISLABEL = "true";
	obj[index++].VALUE = ALabel;
	
	obj[index] = new Object();
	obj[index].NAME = "qtVbar2amax";
	obj[index++].VALUE = String(2.0 * adata.toExponential(3));

//	print("\t2A vbar=" + parseFloat(2.0 * parseFloat(AValue)));

    	obj[index] = new Object();
	obj[index].NAME = "qtVbar2bminLabel";
	obj[index].ISLABEL = "true";
	obj[index++].VALUE = BLabel;
	
	obj[index] = new Object();
	obj[index].NAME = "qtVbar2bmin";
	obj[index++].VALUE = String(2.0 * bdata.toExponential(3));

//	print("\t2B vbar=" + parseFloat(2.0 * parseFloat(BValue)));
    }
    return index;
}

function FormSetld(index, obj, Method, ALabel, AValue, BLabel, BValue)
{
//    print("FormSetld");
    var adata = parseFloat(AValue);
    var bdata = parseFloat(BValue);

    if (isNaN(adata))
    {
	adata = 0.0;
    }
    if (isNaN(bdata))
    {
	bdata = 0.0;
    }

    if (Method == "Teller")
    {
    	obj[index] = new Object();
	obj[index].NAME = "qtTeller2amaxLabel";
	obj[index].ISLABEL = "true";
	obj[index++].VALUE = ALabel;
	
	obj[index] = new Object();
	obj[index].NAME = "qtTeller2amax";
	obj[index++].VALUE = String(Math.pow(10,7)*adata.toExponential(3));

//	print("\t2A teller=" + parseFloat(parseFloat(AValue)));

    	obj[index] = new Object();
	obj[index].NAME = "qtTeller2bminLabel";
	obj[index].ISLABEL = "true";
	obj[index++].VALUE = BLabel;
	
	obj[index] = new Object();
	obj[index].NAME = "qtTeller2bmin";
	obj[index++].VALUE = String(Math.round(Math.pow(10,7)*bdata*1000)/1000);

//	print("\t2B teller=" + parseFloat(parseFloat(BValue)));
    }
    else
    {
    	obj[index] = new Object();
	obj[index].NAME = "qtVbar2amaxLabel";
	obj[index].ISLABEL = "true";
	obj[index++].VALUE = ALabel;
	
	obj[index] = new Object();
	obj[index].NAME = "qtVbar2amax";
	obj[index++].VALUE = String(Math.pow(10,7)*adata.toExponential(3));

//	print("\t2A vbar=" + parseFloat(parseFloat(AValue)));

    	obj[index] = new Object();
	obj[index].NAME = "qtVbar2bminLabel";
	obj[index].ISLABEL = "true";
	obj[index++].VALUE = BLabel;
	
	obj[index] = new Object();
	obj[index].NAME = "qtVbar2bmin";
	obj[index++].VALUE = Math.pow(10,7)*bdata.toExponential(3);

//	print("\t2B vbar=" + parseFloat(parseFloat(BValue)));
    }
    return index;
}

// UseTitrableAmino set to 1 if checked
// UseTitrableCarboxyl set to 1 if checked
// UseOligomerN set to 1 if checked
function _calcPi(UseTitrableAmino,AminoEndpKaValue,UseTitrableCarboxyl,
		 CarboxylValue,OligomerNValue,aminos,conjugates)
{
    UseTitrableAmino    = parseInt(UseTitrableAmino);
    AminoEndpKaValue     = parseFloat(AminoEndpKaValue);
    UseTitrableCarboxyl = parseInt(UseTitrableCarboxyl);
    CarboxylValue        = parseFloat(CarboxylValue);
   // OligomerNValue       = parseFloat(OligomerNValue);

/*    print("_calcPi");
    print("\tUseTitrableAmino=" + UseTitrableAmino);
    print("\tAminoEndpKaValue=" + AminoEndpKaValue);
    print("\tUseTitrableCarboxyl=" + UseTitrableCarboxyl);
    print("\tCarboxylValue=" + CarboxylValue); */
    var charges            = new Array(8);

    charges[AminoBase]     = AminoBase;
    charges[ConjugateBase] = ConjugateBase;

    //var connumber   = new Object;
    //var aminopka    = new Object;
    //var conpka      = new Object;
    //var aminocharge = new Object;
    //var concharge   = new Object;
    //var AminoBase
    //var ConjugateBase

    var pH = new Object(0.0);
    //_chargeSetup(charges);

//    pH = _roots (pH, 1, 13, 0.000001,UseTitrableAmino,AminoEndpKaValue,UseTitrableCarboxyl,
//		 CarboxylValue,OligomerNValue,aminos,conjugates);

    pH = _roots (pH, 1, 13, 0.000001,UseTitrableAmino,AminoEndpKaValue,UseTitrableCarboxyl,
		 CarboxylValue,OligomerNValue,aminos,0);
    return pH;
    //CalcPI = pH;
}

function _chargeSetup(charges)
{
    //var aminonumber = new Object;  0
    //var connumber   = new Object;  1
    //var aminopka    = new Object;  2
    //var conpka      = new Object;  3
    //var aminocharge = new Object;  4
    //var concharge   = new Object;  5
    //var AminoBase                  6
    //var ConjugateBase              7

    charges[aminonumber] = new Object(0); // aminonumber
    charges[connumber] = new Object(0); // connumber

    //'sets up matrices for charge calculations
    //var aminodb As Database, aminotbl As Table
    //var conjtbl As Table
    var i;      // As Integer
    var factor; // As Integer

    //Set aminodb = OpenDatabase(Mid$(physpath, 1, Len(physpath)) + "\phyconst.mdb")
    //Set aminotbl = aminodb.OpenTable("amino")
    //Set conjtbl = aminodb.OpenTable("conjugates")
    //'aminobase is always in 3-letter order
    //aminotbl.Index = "NameThree"
    //aminotbl.MoveLast
    //conjtbl.MoveLast

    //aminonumber = aminotbl.RecordCount
    m_query.prepare("SELECT COUNT(*) FROM amino");
    if (!m_query.exec() || !m_query.next())
    {
	return 0.0;
    }
    print("count = " +  parseInt(m_query.value(0)));
    charges[aminonumber].VALUE = parseInt(m_query.value(0));

    //connumber = conjtbl.RecordCount
    m_query.prepare("SELECT COUNT(*) FROM conjugates");
    if (!m_query.exec() || !m_query.next())
    {
	return 0.0;
    }
    charges[connumber].VALUE = parseInt(m_query.value(0));

    charges[aminopka]    = new Array(parseInt(charges[0].VALUE));
    charges[conpka]      = new Array(parseInt(charges[1].VALUE));
    charges[aminocharge] = new Array(parseInt(charges[0].VALUE));
    charges[concharge]   = new Array(parseInt(charges[1].VALUE));

    //'Load array with values from database before computation starts
    //'This is to speed up the main loop of charge computation

    //aminotbl.MoveFirst
    //'First loop for amino acids

    if (!m_query.exec("SELECT pka,charge FROM amino"))
	return 0.0;

    var i = 0;
    while (m_query.next())
    {
/*
    Do Until aminotbl.EOF
        i = i + 1
        'get pka if good
        If aminotbl("pka") <> 0 And AminoBase(i, current) <> 0 Then
            If IsNull(aminotbl("pka")) Then aminopka(i) = 0 Else aminopka(i) = aminotbl("pka")
            If IsNull(aminotbl("charge")) Then aminocharge(i) = 0 Else aminocharge(i) = aminotbl("charge")
        Else
            aminopka(i) = 0
            aminocharge(i) = 0
        End If
        aminotbl.MoveNext
    Loop
*/
	//aminopka[i] = m_query.value(0).toDouble();
	//aminocharge[i] = m_query.value(1).toDouble();
	charges[aminopka][i] = parseFloat(m_query.value(0));
	charges[aminocharge][i] = parseFloat(m_query.value(1));
	i++;
    }
    
    //conjtbl.MoveFirst
    if (!m_query.exec("SELECT pka,charge FROM conjugates"))
	return 0.0;
    i = 0
    //'Second loop for covalent conjugates
    
    while (m_query.next())
    {
/*
    Do Until conjtbl.EOF
        i = i + 1
        If conjtbl("pka") <> 0 And ConjugateBase(i, current) <> 0 And conjtbl("covalent") <> 0 Then
            If IsNull(conjtbl("pka")) Then conpka(i) = 0 Else conpka(i) = conjtbl("pka")
            If IsNull(conjtbl("charge")) Then concharge(i) = 0 Else concharge(i) = conjtbl("charge")
        Else
            conpka(i) = 0
            concharge(i) = 0
        End If
        conjtbl.MoveNext
    Loop
*/
	charges[conpka][i] = parseFloat(m_query.value(0));
	charges[concharge][i] = parseFloat(m_query.value(1));
	i++;
    }

    return;
}

function _roots(root, X1, X2, tol,UseTitrableAmino,AminoEndpKaValue,UseTitrableCarboxyl,
		CarboxylValue,OligomerNValue,aminos,conjugates)
{
    print(">>Function: _roots(root, X1, X2, tol,UseTitrableAmino,AminoEndpKaValue,UseTitrableCarboxyl,CarboxylValue,OligomerNValue,aminos,conjugates)"); 

    root = parseFloat(root);
    X1   = parseFloat(X1);
    X2   = parseFloat(X2);
    tol  = parseFloat(tol);
    UseTitrableAmino     = parseInt(UseTitrableAmino);
    AminoEndpKaValue     = parseFloat(AminoEndpKaValue);
    UseTitrableCarboxyl  = parseInt(UseTitrableCarboxyl);
    CarboxylValue        = parseFloat(CarboxylValue);
   // OligomerNValue       = parseFloat(OligomerNValue);

//    print("_roots");
//    print("\tUseTitrableAmino=" + UseTitrableAmino);
//    print("\tAminoEndpKaValue=" + AminoEndpKaValue);
//    print("\tUseTitrableCarboxyl=" + UseTitrableCarboxyl);
//    print("\tCarboxylValue=" + CarboxylValue);
      //print("\tOligomerNValue = "  + OligomerNValue.VALUE)

    //'this is a Van Wijngaarden-Dekker-Brent root finder adapted from the ProMath library
    //'which finds roots of the function ChargeFcn
    var itmax; // As Integer
    var eps;   // As Double
    var A;     // As Double; 
    var B;     // As Double; 
    var c;     // As Double; 
    var d;     // As Double; 
    var E;     // As Double
    var fa;    // As Double; 
    var fb;    // As Double; 
    var fc;    // As Double
    var p;     // As Double; 
    var Q;     // As Double; 
    var r;     // As Double; 
    var S;     // As Double; 
    var t;     // As Double
    var tol1;  // As Double; 
    var xm;    // As Double
    var iter;  // As Integer

    itmax = 100;
    eps   = 0.00000003;

    A     = X1;
    fa    = _chargeFcn(A,UseTitrableAmino,AminoEndpKaValue,UseTitrableCarboxyl,
		       CarboxylValue,OligomerNValue,aminos,conjugates);
    B     = X2;
    fb    = _chargeFcn(B,UseTitrableAmino,AminoEndpKaValue,UseTitrableCarboxyl,
		       CarboxylValue,OligomerNValue,aminos,conjugates);
    fc    = fb;

    for (iter = 0; iter < itmax; iter++)
    {
        if (fb * fc > 0)
	{
	    c = A; 
	    fc = fa;
	    d = B - A;
	    E = d;
	}

        if (Math.abs(fc) < Math.abs(fb))
	{
            A = B;
	    B = c; 
	    c = A; 
	    fa = fb; 
	    fb = fc; 
	    fc = fa;
        }

        tol1 = 2 * eps * Math.abs(B) + 0.5 * tol;
        xm   = 0.5 * (c - B);

        if (Math.abs(xm) <= tol1 || fb == 0)
	{
	    root = B;
	    //print("\troot=" + root);
	    return root;
	}
	
        if (Math.abs(E) >= tol1 && Math.abs(fa) > Math.abs(fb))
	{
            S = fb / fa;
            if (A == c)
	    {
		p = 2 * xm * S;
		Q = 1 - S;
	    }
            else
	    {
		Q = fa / fc;
		r = fb / fc;
		p = S * (2 * xm * Q * (Q - r) - (B - A) * (r - 1));
		Q = (Q - 1) * (r - 1) * (S - 1);
            }

            if (p > 0)
	    {
		Q = -Q;
	    }

            p = Math.abs(p);
            S = 3 * xm * Q - Math.abs(tol1 * Q);
            t = Math.abs(E * Q);
            //'need smaller of s and t
            if (t < S)
	    {
		S = t;
	    }

            if (2 * p < S)
	    {
		E = d;
		d = p / Q;
	    }
            else
	    {
		d = xm;
		E = d;
	    }
         }
         else
         {
             d = xm;
	     E = d;
         }

        A = B;
	fa = fb;

        if (Math.abs(d) > tol1)
	{
            B = B + d
	}
        else
	{
            B = B + Math.abs(tol1) * _Sgn(xm);
	}
        fb = _chargeFcn(B,UseTitrableAmino,AminoEndpKaValue,UseTitrableCarboxyl,
			CarboxylValue,OligomerNValue,aminos,conjugates);
    }
//      Next iter
      //MsgBox "Error: ROOT exceeding maximum iterations."
    print("\nbad root=" + root);
    root = B;
}

function _Sgn(x)
{
    if(x > 0)
	return 1;
    else if(x < 0)
	return -1;
    else 
	return 0;
}

// UseTitrableAmino set to 1 if checked
// UseTitrableCarboxyl set to 1 if checked
// UseOligomerN set to 1 if checked
function _chargeFcn(pH, UseTitrableAmino,AminoEndpKaValue,UseTitrableCarboxyl,
		    CarboxylValue,OligomerNValue,aminos,conjugates)
{
//   print(">>Function: _chargeFcn(pH, UseTitrableAmino,AminoEndpKaValue,UseTitrableCarboxyl,CarboxylValue,OligomerNValue,aminos,conjugates)");

    UseTitrableAmino     = parseInt(UseTitrableAmino);
    AminoEndpKaValue     = parseFloat(AminoEndpKaValue);
    UseTitrableCarboxyl  = parseInt(UseTitrableCarboxyl);
    CarboxylValue        = parseFloat(CarboxylValue);
 //   OligomerNValue       = parseFloat(OligomerNValue.VALUE);
    pH                   = parseFloat(pH);


  //  print("_chargeFcn");
  //  print("\tUseTitrableAmino=" + UseTitrableAmino);
  //  print("\tAminoEndpKaValue=" + AminoEndpKaValue);
  //  print("\tUseTitrableCarboxyl=" + UseTitrableCarboxyl);
  //  print("\tCarboxylValue=" + CarboxylValue);
  //  print("\tUseOligomerN=" + OligomerNValue.VALUE);
  //  print("\tpH=" + pH);


    //'calculates charge at pH
    var cCharge; // As Double
    var cHH;     // As Double

    cCharge = 0.0;
  
    //print("\taminos.length = " + aminos.length);
    for (i = 0; i < aminos.length;i++)
    {
	//print("\naminos[" + i + "]=" + aminos[i].ROW);
	//print("SELECT pka,charge FROM amino WHERE id=" + aminos[i].ID);
	if (!m_query.exec("SELECT pka,charge FROM amino WHERE id=" + aminos[i].ID) ||
	    !m_query.next())
	{
	    return 0;
	}
	else
	{
	    var pka    = parseFloat(m_query.value(0));
	    var charge =  parseFloat(m_query.value(1)); 
	    
	//    print("pka=" + pka + ",charge=" + charge);
	    if (pka != 0)
	    {
		if (pka < 0)
		{
		    cCharge = cCharge + parseFloat(aminos[i].MOLES) * charge;
		}
		else
		{
                    if (charge > 0)
		    {
			cHH = (1 / (1 + Math.pow(10,(pH - pka))))
		    }
                    else
		    {
			cHH = -(1 - (1 / (1 + Math.pow(10,(pH - pka)))))
		    }
                    cCharge = cCharge + parseFloat(aminos[i].MOLES) * cHH
		}
	    }
	}
    }

    for (i = 0; i < conjugates.length;i++)
    {
	//print("\tconjugates[" + i + "]=" + conjugates[i].ROW);
	//print("SELECT pka,charge FROM conjugates WHERE id=" + conjugates[i].ID);
	if (!m_query.exec("SELECT pka,charge FROM conjugates WHERE id=" + conjugates[i].ID) ||
	    !m_query.next())
	{
	    return 0;
	}
	else
	{
	    var pka    = parseFloat(m_query.value(0));
	    var charge =  parseFloat(m_query.value(1)); 
	    
	//    print("\tconjugate pka=" + pka + ",charge=" + charge);
	    if (pka != 0)
	    {
		if (pka < 0)
		{
		    cCharge = cCharge + parseFloat(conjugates[i].MOLES) * charge;
		}
		else
		{
                    if (charge > 0)
		    {
			cHH = (1 / (1 + Math.pow(10,(pH - pka))))
		    }
                    else
		    {
			cHH = -(1 - (1 / (1 + Math.pow(10,(pH - pka)))))
		    }
 //                   cCharge = cCharge + parseFloat(conjugates[i].MOLES) * cHH
 cCharge = cCharge + parseFloat(0) * cHH

		}
	    }
	}
    }

    //print("cCharge=" + cCharge);

    //'Finish computing charge with end groups
    if (UseTitrableAmino == 1)
    {
        cHH = (1 / (1 + Math.pow(10,(pH - AminoEndpKaValue))));
        cCharge = cCharge + cHH
    }
    //if (Svalu(current).apkacarboxyl <> 0)
    if (UseTitrableCarboxyl == 1)
    {
        cHH = -(1 - (1 / (1 + Math.pow(10,(pH - CarboxylValue)))));
        cCharge = cCharge + cHH
    }
    //'fixup N-mer
    if (parseInt(OligomerNValue.VALUE) > 1)
    {
        //cCharge = cCharge * cVsng((frmcomposition.txtNmer));
        cCharge = cCharge * Math.pow(parseInt(OligomerNValue.VALUE),2);
    }
  //  print(cCharge)
    return cCharge;
}

function _createChargeGraphData(xData, yData,UseTitrableAmino,AminoEndpKaValue,UseTitrableCarboxyl,CarboxylValue,OligomerNValue,aminos,conjugates)
{
    print("_createChargeGraphData");
    var lowph    = 1;
    var highph   = 13;
    var pointsph = 121;
    var stepph   = (highph-lowph)/(pointsph-1);
    var pH       = lowph;

    for (var j=0; j < pointsph; j++)
    {
	xData[j] = new Object();
	xData[j].GRAPH="pH Graph";
	xData[j].AXIS="x";
	xData[j].INDEX = j;
	xData[j].DATA = pH;
	
	yData[j] = new Object();
	yData[j].GRAPH="pH Graph";
	yData[j].AXIS="y";
	yData[j].INDEX = j;
        yData[j].DATA = _chargeFcn(pH,UseTitrableAmino,AminoEndpKaValue,
				   UseTitrableCarboxyl,CarboxylValue,OligomerNValue,
				   aminos,conjugates)/parseInt(OligomerNValue.VALUE);
	
	pH += stepph;
    }
}

function _createAbsGraphData(xData,yData,aminos,conjugates)
{
    print("_createAbsGraphData");

    //var query = "SELECT Mr, Vbar, :hipH, covalent FROM :table WHERE id in (:IDset)";
    //var aminoQuery = query.replace(":table", "amino");
    //aminoQuery = aminoQuery.replace(":hipH", hipH);
    //aminoQuery = aminoQuery.replace(":IDset", aminoID.join(","));
    var k = 0;
    var lownm  = 250;
    var highnm = 320;
    var points = (highnm - lownm) / 2 + 1;

//    print("aminos.length=" + aminos.length);
//    print("conjugates.length=" + conjugates.length);
    for (var i=lownm; i <= highnm; i+=2)
    {
	var cext = 0;
	for (var j = 0; j < aminos.length; j++)
	{
	//    print("nm=" + i);
	//    print("aminos[j]=" + aminos[j].ITEMNAME);

	    if (m_query.exec("SELECT ext FROM extinction WHERE name='" + aminos[j].ITEMNAME + "' AND nm=" + i) && m_query.next() && !(aminos[j].ITEMNAME == "CY2"))
	    {
		cext += parseFloat(m_query.value(0)) * parseFloat(aminos[j].MOLES);
	    }
	    else if (m_query.exec("SELECT ext FROM extinction WHERE name='" + aminos[j].ITEMNAME + "' AND nm=" + i) && m_query.next() && (aminos[j].ITEMNAME == "CY2"))
	    {
		cext += parseFloat(m_query.value(0)) * parseFloat(aminos[j].MOLES)/2;
	    }
	}
	for (var j = 0; j < conjugates.length; j++)
	{
	    if (m_query.exec("SELECT ext FROM extinction WHERE name='" + conjugates[j].ITEMNAME + "' AND nm=" + i) && m_query.next())
	    {
		cext += parseFloat(m_query.value(0)) * parseFloat(conjugates.MOLES);

	    }
	}
//	print("cext=" + cext);

	xData[k] = new Object();
	xData[k].GRAPH="Absorbance Graph";
	xData[k].AXIS="x";
	xData[k].INDEX = k;
	xData[k].DATA = i;
	
	yData[k] = new Object();
	yData[k].GRAPH="Absorbance Graph";
	yData[k].AXIS="y";
	yData[k].INDEX = k;
        yData[k].DATA = cext;
	k++;
    }
}
//---------------------------------------------------------------
/*Creates the Graph Shape plot*/


function graphShape(shapeOption,model,tamax,tbmin,vamax,vbmin, delta,mr)
{
print(">>graphShape(shapeOption,model,tamax,tbmin,vamax,vbmin,hydrationExpansion)")

mode = parseFloat(model.VALUE);
At = parseFloat(tamax.VALUE);
Bt = parseFloat(tbmin.VALUE);
Av = parseFloat(vamax.VALUE);
Bv = parseFloat(vbmin.VALUE);
delta1 = parseFloat(delta.VALUE);
Mr = parseFloat(mr.VALUE);

/*print("shapeOption = " + shapeOption.VALUE)
print("shapeOption.NAME = " + shapeOption.NAME)
print("mode = " + mode)
print("At = " + At)
print("Bt = " + Bt)
print("Av = " + Av)
print("Bv = " + Bv)
print("delta1 = " + delta1)
print("mr = " + Mr)
*/


var volp   = Mr * .7654 * 1e+21 / Na;
var volw   = Mr * delta1 * 1e+21 / (Na * Density_20_Water);
//print("(volp,volw)=("+volp+","+volw+")")

var hydrationExpansion = Math.pow(((volw + volp)/volp),1/3) - 1;
var He = hydrationExpansion+1;
//print("hydrationExpansion =" + hydrationExpansion)

recVal = new Object();
	if (shapeOption.VALUE=="qtShapeProlate")
	{
		if (mode == 0)
		{
			recVal.GRAPHSHAPEX = new Array(250);
			recVal.GRAPHSHAPEY = new Array(250);
			_createGraphProShapeData(recVal.GRAPHSHAPEX, recVal.GRAPHSHAPEY, Bt,At,He)
		}
		else
		{
			recVal.GRAPHSHAPEX = new Array(250);
			recVal.GRAPHSHAPEY = new Array(250);
			_createGraphProShapeData(recVal.GRAPHSHAPEX, recVal.GRAPHSHAPEY, Bv, Av,He)
		}
	}
	else if (shapeOption.VALUE =="qtShapeOblate")
	{
		if (mode == 0)
		{
			recVal.GRAPHSHAPEX = new Array(250);
			recVal.GRAPHSHAPEY = new Array(250);
			_createGraphObShapeData(recVal.GRAPHSHAPEX, recVal.GRAPHSHAPEY, At,Bt,He)
		}
		else
		{
			recVal.GRAPHSHAPEX = new Array(250);
			recVal.GRAPHSHAPEY = new Array(250);
			_createGraphObShapeData(recVal.GRAPHSHAPEX, recVal.GRAPHSHAPEY, Av,Bv,He)
		}
	}
	else if (shapeOption.VALUE == 'qtShapeCylinder')
	{
		if (mode == 0)
		{
			recVal.GRAPHSHAPEX = new Array(141);
			recVal.GRAPHSHAPEY = new Array(141);
			_createGraphShapeLDData(recVal.GRAPHSHAPEX, recVal.GRAPHSHAPEY, Bt, At,He)
		}
		else
		{
			recVal.GRAPHSHAPEX = new Array(141);
			recVal.GRAPHSHAPEY = new Array(141);
			_createGraphShapeLDData(recVal.GRAPHSHAPEX, recVal.GRAPHSHAPEY, Bv, Av,He)
		}
	}
	else
	{
		return;
	}

	recVal.EXPANSION = new Object();
	recVal.EXPANSION.NAME = "qtGraphShapeHydrationExpansion";
	recVal.EXPANSION.VALUE = parseFloat((He-1)*100).toFixed(2);
	return recVal;
}
function _createGraphObShapeData(xData,yData,x,y,He)
{
print("_createGraphObShapeData(xData, yData, x, y, He)")
var axis=Math.ceil(He*(1.5*x+1.5*y));
	for (j=0; j<100; j++)
	{
		xData[j] = new Object();
		xData[j].GRAPH = "GraphShape";
		xData[j].AXIS = "x";
		xData[j].INDEX = j;
		xData[j].DATA = x/2*He+1/He*x/2*Math.cos(2*Math.PI/99*j);

		yData[j] = new Object();
		yData[j].GRAPH = "GraphShape";
		yData[j].AXIS = "y";
		yData[j].INDEX = j;
		yData[j].DATA = 3*y/4+x/2*He+1/He*x/2*Math.sin(2*Math.PI/99*j)+(x+y)/2;
		
		xData[j+100] = new Object();
		xData[j+100].GRAPH = "GCraphShape";
		xData[j+100].AXIS = "x";
		xData[j+100].INDEX = j;
		xData[j+100].DATA = x/2*He+x/2*Math.cos(2*Math.PI/99*j);

		yData[j+100] = new Object();
		yData[j+100].GRAPH = "GCraphShape";
		yData[j+100].AXIS = "y";
		yData[j+100].INDEX = j;
		yData[j+100].DATA = 3*y/4+x/2*He+x/2*Math.sin(2*Math.PI/99*j)+(x+y)/2;
	//print("xData["+j+"] = " + xData[j].DATA + "   yData["+j+"] = " + yData[j].DATA)
	}
	for (k=200; k<300; k++)
	{
		xData[k] = new Object();
		xData[k].GRAPH = "GraphWShape";
		xData[k].AXIS = "x";
		xData[k].INDEX = k-200;
		xData[k].DATA = x/2*He+1/He*x/2*Math.cos(2*Math.PI/99*(k-200));

		yData[k] = new Object();
		yData[k].GRAPH = "GraphWShape";
		yData[k].AXIS = "y";
		yData[k].INDEX = k-200;
		yData[k].DATA = 3*y/4 + 1/He*y/2*Math.sin(2*Math.PI/99*(k-200))+Math.ceil(y/2*He);

		xData[k+100] = new Object();
		xData[k+100].GRAPH = "GtraphShape";
		xData[k+100].AXIS = "x";
		xData[k+100].INDEX = k-200;
		xData[k+100].DATA = x/2*He+x/2*Math.cos(2*Math.PI/99*(k-200));

		yData[k+100] = new Object();
		yData[k+100].GRAPH = "GtraphShape";
		yData[k+100].AXIS = "y";
		yData[k+100].INDEX = k-200;
		yData[k+100].DATA = 3*y/4+y/2*Math.sin(2*Math.PI/99*(k-200))+Math.ceil(y/2*He);

		xData[k+200] = new Object();
		xData[k+200].GRAPH = "axel";
		xData[k+200].AXIS = "x";
		xData[k+200].INDEX = k-200;
		xData[k+200].DATA = axis;

		yData[k+200] = new Object();
		yData[k+200].GRAPH = "axel";
		yData[k+200].AXIS = "y";
		yData[k+200].INDEX = k-200;
		yData[k+200].DATA = axis;
		axis = 0;
	}
//	for (b=0; b < 250; b++)
//	{
//		print("xData["+b+"] = " + xData[b].DATA + "   yData["+b+"] = " + yData[b].DATA)
//	}
}
function _createGraphShapeLDData(xData, yData, x,y,He)
{
print("_createGraphShapeLDData(xData, yData, x, y, He)");
print("x : " + x);
print("y : " + y);
print("He : " + He);
var t;var t2;var u;var u2;var axis = Math.ceil(He*(x+y));
	for (var j=0; j <= 4; j++)
	{
		if (j==0)
		{
			t=-x/2/He+Math.ceil(x);
			u=y/2+y/2/He;
		}
		else if (j==1)
		{
			t=-x/2/He+Math.ceil(x);
			u=y/2-y/2/He;
		}
		else if (j==2)
		{
			t=x/2/He+Math.ceil(x);
			u=y/2-y/2/He;
		}
		else if (j==3)
		{
			t=x/2/He+Math.ceil(x);
			u=y/2+y/2/He;
		}
		else
		{
			t=-x/2/He+Math.ceil(x);
			u=y/2+y/2/He;	
		}	

		xData[j] = new Object();
		xData[j].GRAPH = "GraphShape";
		xData[j].AXIS = "x";
		xData[j].INDEX = j;
		xData[j].DATA = t;

		yData[j] = new Object();
		yData[j].GRAPH = "GraphShape";
		yData[j].AXIS = "y";
		yData[j].INDEX = j;
		yData[j].DATA = u;

	//print("xData["+j+"] = " + xData[j].DATA + "   yData["+j+"] = " + yData[j].DATA)
	}
	for (var k=5; k <= 70; k++)
	{
		xData[k] = new Object();
		xData[k].GRAPH = "GraphWShape";
		xData[k].AXIS = "x";
		xData[k].INDEX = k-5;
       		xData[k].DATA = axis-x*Math.cos(2*Math.PI/65*(k-5))/2/He-x;

		yData[k] = new Object();
		yData[k].GRAPH = "GraphWShape";
		yData[k].AXIS = "y";
		yData[k].INDEX = k-5;
		yData[k].DATA = y/2+x*Math.sin(2*Math.PI/65*(k-5))/2/He;
	//print("xData["+k+"] = " + xData[k].DATA + "   yData["+k+"] = " + yData[k].DATA)
	}
	for (var h=71; h <= 75; h++)
	{
		if (h==71)
		{
			t=-x/2+Math.ceil(x);
			u=y;
		}
		else if (h==72)
		{
			t=-x/2+Math.ceil(x);
			u=0;
		}
		else if (h==73)
		{
			t=x/2+Math.ceil(x);
			u=0;
		}
		else if (h==74)
		{
			t=x/2+Math.ceil(x);
			u=y;
		}
		else
		{
			t=-x/2+Math.ceil(x);
			u=y;
		}
		xData[h] = new Object();
		xData[h].GRAPH = "GCraphShape";
		xData[h].AXIS = "x";
		xData[h].INDEX = h-71;
		xData[h].DATA = t

		yData[h] = new Object();
		yData[h].GRAPH = "GCraphShape";
		yData[h].AXIS = "y";
		yData[h].INDEX = h-71;
		yData[h].DATA = u
	//print("xData["+h+"] = " + xData[h].DATA + "   yData["+h+"] = " + yData[h].DATA)
	}
	for (var v=76; v <=141; v++)
	{
		xData[v] = new Object();
		xData[v].GRAPH = "GtraphShape";
		xData[v].AXIS = "x";
		xData[v].INDEX = v-76;
       		xData[v].DATA = axis-x*Math.cos(2*Math.PI/65*(v-75))/2-x

		yData[v] = new Object();
		yData[v].GRAPH = "GtraphShape";
		yData[v].AXIS = "y";
		yData[v].INDEX = v-76;
		yData[v].DATA = y/2 +x*Math.sin(2*Math.PI/65*(v-75))/2;

	var taxis = Math.ceil(He*(x+y)-v/100000000000)

		xData[v+66] = new Object();
		xData[v+66].GRAPH = "axel";
		xData[v+66].AXIS = "x";
		xData[v+66].INDEX = v-76;
		xData[v+66].DATA = taxis;

		yData[v+66] = new Object();
		yData[v+66].GRAPH = "axel";
		yData[v+66].AXIS = "y";
		yData[v+66].INDEX = v-76;
		yData[v+66].DATA = taxis;

	}

//	for (b=0;b<=142;b++)
//	{
		//print("xData["+b+"] = " + xData[b].DATA + "   yData["+b+"] = " + yData[b].DATA)
//	}
}
function _createGraphProShapeData(xData, yData, x, y,He)
{
print("_createGraphShapeData(xData, yData, x, y, He)");

var t;var t1;var axis=Math.ceil(He*(x+y));
	for (var j=0; j < 100; j++)
	{
		xData[j] = new Object();
		xData[j].GRAPH = "GraphShape";
		xData[j].AXIS = "x";
		xData[j].INDEX = j;
		xData[j].DATA = Math.ceil(x/2*He) + x/2*Math.cos(2*Math.PI/99*j)/He
		t = x/2*Math.cos(2*Math.PI/99*j);


		yData[j] = new Object();
		yData[j].GRAPH = "GraphShape";
		yData[j].AXIS = "y";
		yData[j].INDEX = j;
		yData[j].DATA = y/2 + y*Math.sin(2*Math.PI/99*j)/2/He
		t1=y*Math.sin(2*Math.PI/99*j)/2;

		xData[j+100] = new Object();
		xData[j+100].GRAPH = "GCraphShape";
		xData[j+100].AXIS = "x";
		xData[j+100].INDEX = j;
		xData[j+100].DATA = Math.ceil(He*x/2) + t;

		yData[j+100] = new Object();
		yData[j+100].GRAPH = "GCraphShape";
		yData[j+100].AXIS = "y";
		yData[j+100].INDEX = j;
		yData[j+100].DATA = y/2+t1;
//	print("xData["+j+"] = " + xData[j].DATA + "   yData["+j+"] = " + yData[j].DATA)
	}
	for (var k=200; k < 300; k++)
	{
		xData[k] = new Object();
		xData[k].GRAPH = "GraphWShape";
		xData[k].AXIS = "x";
		xData[k].INDEX = k-200;
    	        xData[k].DATA = axis-x*Math.cos(2*Math.PI/99*(k-200))/2/He-x;
		t=x*Math.cos(2*Math.PI/99*(k-200))/2;

		yData[k] = new Object();
		yData[k].GRAPH = "GraphWShape";
		yData[k].AXIS = "y";
		yData[k].INDEX = k-200;
		yData[k].DATA = y/2+x*Math.sin(2*Math.PI/99*(k-200))/2/He;
		t1=x*Math.sin(2*Math.PI/99*(k-200))/2;

		xData[k+100] = new Object();
		xData[k+100].GRAPH = "GtraphShape";
		xData[k+100].AXIS = "x";
		xData[k+100].INDEX = k-200;
    	        xData[k+100].DATA = axis - t -x

		yData[k+100] = new Object();
		yData[k+100].GRAPH = "GtraphShape";
		yData[k+100].AXIS = "y";
		yData[k+100].INDEX = k-200;
		yData[k+100].DATA = t1+y/2

		axis = Math.ceil(He*(x+y)-k/1000000000)

		xData[k+200] = new Object();
		xData[k+200].GRAPH = "axel";
		xData[k+200].AXIS = "x";
		xData[k+200].INDEX = k-200;
		xData[k+200].DATA = axis;

		yData[k+200] = new Object();
		yData[k+200].GRAPH = "axel";
		yData[k+200].AXIS = "y";
		yData[k+200].INDEX = k-200;
		yData[k+200].DATA = axis;
		//print("axis = " + axis);

	}

/*for (b = 0; b <=250; b++)
{
	print("xData["+b+"] = " + xData[b].DATA + " yData["+b+"] = " + yData[b].DATA)
}
*/

}


//-----------------------------------------------------------------------
/*Creates the x and y values for the a/b and l/d graphs */
function calculationsAvsBHydration(shapeOption, hydrateOption, model, mr, hydrateValue, s20w, vbar, bufferViscosity,sigma,mb,t,rpm)
{
    print(">>calculationsAvsBHydration(shapeOption, hydrateOption, model, mr, hydrateValue, s20w, vbar, bufferViscosity, sigma, mb, t, rpm)")
    
    //print("shapeOption : " + shapeOption.VALUE)
    //print("hydrateOption : " + hydrateOption.VALUE)
    //print("model : " + model.VALUE)
    //print("mr : " + mr.VALUE )
    //print("hydrateValue : " + hydrateValue.VALUE)
    //print("s20w : " + s20w.VALUE)
    //print("vbar : " + vbar.VALUE)
    //print("bufferViscosity : " + bufferViscosity.VALUE)
    //print("sigma : " + sigma.VALUE)
    //print("mb : " + mb.VALUE)
    //print("t : " + t.VALUE)
    //print("rpm : " + rpm.VALUE)
    
    
    shapeOp = parseFloat(shapeOption.VALUE);
    hydrateOp = parseFloat(hydrateOption.VALUE);
    mode = parseFloat(model.VALUE);
    Mr = parseFloat(mr.VALUE);
    HydrateVal = parseFloat(hydrateValue.VALUE);
    S20W = parseFloat(s20w.VALUE);
    Vbar = parseFloat(vbar.VALUE);
    bufferVis = parseFloat(bufferViscosity.VALUE);
    sigma = parseFloat(sigma.VALUE);
    mb = parseFloat(mb.VALUE);
    t = parseFloat(t.VALUE);
    rpm = parseFloat(rpm.VALUE);
    
    var Fo = 6.0 * PI * Viscosity_20_Water * _calculateRo(Mr,Vbar)
    var Fp = 6.0 * PI * Viscosity_20_Water * _calculateRp(Mr)
    
    var delta1;
    if (hydrateOp == "qtHydrateDelta1")
	delta1 = Ratio;
    else
	delta1 = HydrateVal;
    
    var k=0;
    var points=100;
    var hmax=0.7;
    var delta1in = 0;
    var x = new Array(points);
    var y = new Array(points);
    var Ap = new Object(0.0);
    var Bp = new Object(0.0);
    var Ao = new Object(0.0);
    var Bo = new Object(0.0);
    var Lp = new Object(0.0);
    var Dp = new Object(0.0);
    var Lo = new Object(0.0);
    var Do = new Object(0.0);
    var loverdp = 0.0;
    var loverdo = 0.0;
    
    
    for (k=0;k<points;k=k+1)
    {
	delta1in= (k)*(hmax)/(points)
	x[k] = delta1in;
	//print("x["+k+"]="+x[k]);
	if (shapeOption.VALUE == 'qtShapeCylinder')
	{	
	    //	print("mode : " + mode)
	    if (mode == 1)
	    {
		HydrateVal = Math.pow(-2,1/2);
		loverdp = _computeLOverD_p(HydrateVal, S20W, delta1in, Mr, Vbar, Lp, Dp,bufferVis);
		//		print("loverdp : " + loverdp)
		y[k] = loverdp;		
	    }	
			else
	    {
		loverdo =  _computeLOverD_o(Mr, hydrateOp, delta1in, Vbar, S20W, delta1in, Lo,Do,bufferVis);
		//		print("loverdo : " + loverdo)
		y[k] = loverdo;
	    }
	}
	else if (shapeOption.VALUE == 'qtShapeProlate')
	{
	    if (mode == 1)
	    {
		_computeExactAOverB_p(S20W,delta1in,Mr,Vbar,Fp,Ap,Bp);
		//		print("Exactaoverb_p : " + Ap.VALUE)
		y[k] = Ap.VALUE;
	    }
	    else
	    {
		_computeExactAOverB_o(S20W,delta1in,Mr,Vbar,Fo,Ao,Bo)
		//print("Exactaoverb_o : " + Bo.VALUE)
		y[k] = Bo.VALUE;
	    }
	}
	else if (shapeOption.VALUE == 'qtShapeOblate')
	{
	    if (mode == 1)
	    {
		_computeExactAOverB_p(S20W,delta1in,Mr,Vbar,Fp,Ap,Bp);
		//		print("Exactaoverb_p : " + Ap.VALUE)
		y[k] = Ap.VALUE;
	    }
	    else
	    {
		
		_computeExactAOverB_o(S20W,delta1in,Mr,Vbar,Fo,Ao,Bo);
		//		print("Exactaoverb_o : " + Bo.VALUE)
		y[k] = Bo.VALUE;
	    }
	}
	else
	{
	    return
	}
    }   
    
    reVal = new Object();
    reVal.GRAPHAOVERBX = new Array(points);
    reVal.GRAPHAOVERBY = new Array(points);
    _createAoverBGraphData(reVal.GRAPHAOVERBX, reVal.GRAPHAOVERBY, x, y);
    return reVal;
}

function _createAoverBGraphData(xData, yData, x, y)
{
    for (var j=0; j < 100; j++)
    {
	xData[j] = new Object();
	xData[j].GRAPH = "A over B Graph";
	xData[j].AXIS = "x";
	xData[j].INDEX = j;
	xData[j].DATA = x[j];
	
	yData[j] = new Object();
	yData[j].GRAPH = "A over B Graph";
	yData[j].AXIS = "y";
	yData[j].INDEX = j;
	yData[j].DATA = y[j];
    }
}

function getVersion()
{
    var retVal = new Object();
    
    m_query.prepare("SELECT applicationVersion,databaseVersion FROM version ");
    
    if (!m_query.exec()) 
    {
	retVal.APPVERSION = "UNKNOWN";
	retVal.DBVERSION = "UNKNOWN";
	retVal.ERROR = "Failed to execute query!";
	return retVal;
    }
    
    for (var i = 0; m_query.next(); i++)
    {
	retVal.APPVERSION = m_query.value(0).toString();
	retVal.DBVERSION = m_query.value(1).toString();
    }
    
    return retVal;
}
//This function is strictly for the web version of sednterp modules try not to excute it anywhere else as there are better ways to execute sql queries.
function executeSqlQuery( query ) 
{
    var retVal = new Object();
    m_query.prepare(query.VALUE);
    if (!m_query.exec()) 
    {
	retVal.APPVERSION = "UNKNOWN";
	retVal.DBVERSION = "UNKNOWN";
	retVal.ERROR = "Failed to execute query!";
	return retVal;
    }
    var queryChecker = query.VALUE; 
    var indx = new Array();
    indx[0] = queryChecker.indexOf("SELECT") + 6;
    indx[1] = queryChecker.indexOf("FROM");
    indx[2] = 0;
    indx[3] = queryChecker.substr(indx[0], indx[1]);
    indx[4] = new Array();
    var len = indx[3].length
    for (var i = 0; i < len; i++) {
       if (parseInt(indx[3].search(",")) != parseInt(-1)) {
	  indx[4][indx[2]] = indx[3].substr(0, parseInt(indx[3].search(",")));
	  indx[4][indx[2]] = indx[4][indx[2]].replace(" ", "");
	  indx[2] += 1;
	  indx[3] = indx[3].substr(parseInt(indx[3].search(","))+1, indx[3].length-1);
	 }
     }
    
     for (var i = 0; m_query.next(); i++) {
      if (indx[2] == 0) {
         retVal[i] = new Array();
         retVal[i].NAME = "sqlQueryResult";
         retVal[i].INDEX = 0;
         retVal[i].INDEXROW = i;
         retVal[i].LENGTH = indx[2];
         retVal[i].VALUE = m_query.value(j).toString();
      }
      for (var j = 0; j < indx[2]; j++) {
	 retVal[i*indx[2]+j] = new Array();
	 retVal[i*indx[2]+j].NAME = "sqlQueryResult";
	 retVal[i*indx[2]+j].INDEX = j;
	 retVal[i*indx[2]+j].INDEXNAME = indx[4][j];
	 retVal[i*indx[2]+j].INDEXROW = i;
	 retVal[i*indx[2]+j].LENGTH = indx[2];
	 retVal[i*indx[2]+j].VALUE = m_query.value(j).toString();
        
      }
     } 
         return retVal;
}
